% Simulation environment of a 2-link planar manipulator
% Copyright 2011 Adriano Siqueira
% File sim_loop.m
% Computes the H-infinity controller 

function [Ak,Bk,Ck,Dk,CL] = hin_2AA(kp,kv,fig)

Z=zeros(2);
I=eye(2);
sizekp=size(kp);
if sizekp(1) == 2
    KP= kp;
else
    KP = kp*eye(2);
end

sizekv=size(kv);
if sizekv(1) == 2
    KV= kv;
else
    KV = kv*eye(2);
end

% Nominal system
Ag=[ Z   I
    -KP -KV];
Bg=[Z
    I];
Cg=[I Z];
Dg=Z;
G=pck(Ag,Bg,Cg,Dg);

% Weighting definition
Ms = 1;
E = 0.00000001;
wb = 1;

nwe=(1/Ms)*[1 wb];
dwe=[1 wb*E];
We=tf(nwe,dwe)
We1=nd2sys(nwe,dwe);
We2=nd2sys(nwe,dwe);
We=daug(We1,We2);

Mu = 100;
E1 = 0.0001;
wbc = 1;

nwu=(1/Mu)*[1 wbc];
dwu=[E1 wbc];
Wu=tf(nwu,dwu)
Wu1=nd2sys(nwu,dwu);
Wu2=nd2sys(nwu,dwu);
Wu=daug(Wu1,Wu2);

% Augmented plant
systemnames='G Wu We';
inputvar='[pert(2); dist(2); control(2)]';
outputvar='[Wu ; We; G+dist]';
input_to_G='[control+pert]';
input_to_Wu='[control]';
input_to_We='[G+dist]';
sysoutname='P';
cleanupsysic='yes';
sysic

% H-infinity synthesis
gmin=0.1;
gmax=150;
[K1,CL]=hinfsyn(P,2,2,gmin,gmax,0.05);
[Ak,Bk,Ck,Dk]=unpck(K1);
K=ss(Ak,Bk,Ck,Dk);


if fig ==1,

    % Graphics
    [Agk,Bgk,Cgk,Dgk] = series(Ak,Bk,Ck,Dk,Ag,Bg,Cg,Dg);
    Lo = ss(Agk,Bgk,Cgk,Dgk);

    % Sensitivity function
    So = inv(eye(size(Lo))+Lo);

    We = tf(nwe,dwe);
    WE = eye(2)*We;
    Wu = tf(nwu,dwu);
    WU = eye(2)*Wu;
    graph_SWE =figure(...
        'Unit','normalized',...
        'NextPlot','add',...
        'Visible','on',...
        'NumberTitle','off',...
        'Menubar','none',...
        'Toolbar','none',...
        'Position',[0.3 0.3 0.4 0.5],...
        'Name','Sensitivity, S(s), and W_e^{-1}(s)');


    wsigma = logspace(log10(wb*E)-2,log10(wb)+2,100);
    SV1 = sigma(So,wsigma);
    minSV = min(min(20*log10(SV1)));
    maxSV = max(max(20*log10(SV1)));
    semilogx(wsigma,20*log10(SV1(1,:)),'g');
    hold on
    SV = sigma(inv(WE),wsigma);
    semilogx(wsigma,20*log10(SV),'b');
    semilogx(wsigma,20*log10(SV1(2,:)),'g');
    axis([wsigma(1) wsigma(end) min(min(min(20*log10(SV))),minSV)-10 max(max(max(20*log10(SV))),maxSV)+10])
    leg = legend('S (j\omega)','W_{e}^{-1}(j\omega)',2);
    set(leg,'FontSize',10);
    grid

    xla = xlabel('Frequency (rad/s)');
    set(xla,'FontSize',10);
    yla = ylabel('S(s) and W_{e}^{-1}(s)');
    set(yla,'FontSize',10);

    % Complementary sensitivity function
    To = series(Lo,So);
   

    graph_TWD =figure(...
        'Unit','normalized',...
        'NextPlot','add',...
        'Visible','on',...
        'NumberTitle','off',...%    'Menubar','none',...           'Toolbar','none',...
        'Position',[0.3 0.3 0.4 0.5],...
        'Name','Complementary Sensitivity, T(s), and W_Delta^{-1}(s)');


    wsigma = logspace(log10(wbc)-2,log10(wbc/E1)+2,100);
    SV1 = sigma(To,wsigma);
    minSV = min(min(20*log10(SV1)));
    maxSV = max(max(20*log10(SV1)));
    semilogx(wsigma,20*log10(SV1(1,:)),'g');
    hold on
    SV = sigma(inv(Wu),wsigma);
    semilogx(wsigma,20*log10(SV),'b');
    semilogx(wsigma,20*log10(SV1(2,:)),'g');
    axis([wsigma(1) wsigma(end) min(min(min(20*log10(SV))),minSV)-10 max(max(max(20*log10(SV))),maxSV)+10])
    leg = legend('T (j\omega)','W_{\Delta}^{-1}(j\omega)',1);
    set(leg,'FontSize',10);
    grid

    xla=xlabel('Frequency (rad/s)');
    set(xla,'FontSize',10);
    yla=ylabel('T(s) and W_{\Delta}^{-1}(s)');
    set(yla,'FontSize',10);


end



