% Simulation environment of a 2-link planar manipulator
% Copyright 2011 Adriano Siqueira
% File jtraj_lpb.m
% Compute a joint space trajectory between two points

function [qt,qdt,qddt] = jtraj_lpb(q0, q1, tv, vmax, tb, qd0, qd1)
    
    t0 = tv(1);

	q0 = q0(:);
	q1 = q1(:);

	if nargin == 5,
		qd0 = zeros(size(q0));
		qd1 = qd0;
	end

	% compute the polynomial coefficients
	A = q0;
	B = qd0;
	C = (vmax - qd0)./(2*tb);
	D = q0 + qd0.*tb/2 -vmax*t0 - vmax.*tb/2;
    E = q1; 
	F = qd1;
    G = (qd1-vmax)./(2*tb);
    
    tf = (q1 + qd1.*tb/2 -D + vmax.*tb/2)./vmax
    
    
    for i=1:size(tv,2),
        for j=1:2,
            if tv(i) <= t0 + tb(j)
                qt(j,i) = A(j)+B(j)*(tv(i)-t0)+C(j)*(tv(i)-t0)^2;
                if nargout >= 2,
                    qdt(j,i) = B(j) + 2*C(j)*(tv(i)-t0);
                end
                if nargout == 3,
                    qddt(j,i) = 2*C(j);
               end
            elseif ((tv(i) >= t0+tb(j)) &(tv(i) <= tf(j)-tb(j)))
                qt(j,i) = D(j) + vmax(j)*tv(i);
                if nargout >= 2,
                	qdt(j,i) = vmax(j);
                end
               if nargout == 3,
                   qddt(j,i) = 0;
              end
            elseif (tv(i) >= tf(j)-tb(j) & tv(i) <= tf(j))
                 qt(j,i) = E(j)+F(j)*(tv(i)-tf(j))+G(j)*(tv(i)-tf(j))^2;
                 if nargout >= 2,
                	qdt(j,i) = F(j) + 2*G(j)*(tv(i)-tf(j)) ;
                 end
               if nargout == 3,
                   qddt(j,i) = 2*G(j);
               end
            elseif  tv(i) >= tf(j)
                 qt(j,i) = E(j);
                 if nargout >= 2,
                	qdt(j,i) = 0 ;
                 end
                if nargout == 3,
                   qddt(j,i) = 0;
               end
                 
             end
         end
    end
    
    
    
