function [M,C] = pr3l_nlh(theta,omega,m,l,lc,I,atr)
% computes the dynamic model of a 3-DOF planar manipulator

if (nargin ~= 7)
  error('pr3l.m requires 7 input arguments');
end;

SIN2 = sin(theta(2));
COS2 = cos(theta(2));
SIN3 = sin(theta(3));
COS3 = cos(theta(3));
COS23 = cos(theta(2) + theta(3));
SIN23 = sin(theta(2) + theta(3));

t(1) = lc(1)*lc(1);
t(3) = l(1)*l(1);
t(4) = lc(2)*lc(2);
t(6) = COS2;
t(7) = l(1)*lc(2)*t(6);
t(10) = l(2)*l(2);
t(11) = lc(3)*lc(3);
t(13) = l(1)*l(2)*t(6);
t(17) = l(1)*lc(3)*COS23;
t(20) = l(2)*lc(3)*COS3;
t(28) = I(2) + I(3) + m(2)*(t(4)+t(7)) + ...
        m(3)*(t(10)+t(11)+t(13)+t(17)+2.0*t(20));
t(33) = I(3) + m(3)*(t(11)+t(17)+t(20));
t(36) = I(3) + m(3)*(t(11)+t(20));

M(1,1) = I(1) + I(2) + I(3) + m(1)*t(1) + ...
          m(2)*(t(3)+t(4)+2.0*t(7)) + ...
          m(3)*(t(3)+t(10)+t(11)+2.0*t(13)+2.0*t(17)+2.0*t(20));
M(1,2) = t(28);
M(1,3) = t(33);
M(2,1) = M(1,2);
M(2,2) = I(2) + I(3) + m(2)*t(4) + m(3)*(t(10)+t(11)+2.0*t(20));
M(2,3) = t(36);
M(3,1) = M(1,3);
M(3,2) = M(2,3);
M(3,3) = I(3) + m(3)*t(11);


t(2) = SIN2;
t(4) = m(2)*l(1)*lc(2)*t(2);
t(6) = l(1)*l(2)*t(2);
t(10) = l(1)*lc(3)*SIN23;
t(23) = l(2)*lc(3)*SIN3;

A = t(4)+ m(3)*t(6) + m(3)*t(10);
B = m(3)*t(10) + m(3)*t(23);

C(1,1) = -A*omega(2) - B*omega(3);
C(2,2) = -m(3)*t(23)*omega(3);
C(3,3) = 0;
C(1,2) = -A*omega(1) - A*omega(2) - B*omega(3);
C(2,1) =  A*omega(1) - m(3)*t(23)*omega(3);
C(1,3) = -B*omega(1) - B*omega(2) - B*omega(3);
C(3,1) =  B*omega(1) + m(3)*t(23)*omega(2);
C(2,3) = -m(3)*t(23)*(omega(1)+omega(2)+omega(3));
C(3,2) = m(3)*t(23)*(omega(1)+omega(2));

%F = zeros(3);
F(1,1)= atr(1); %.35   .45      .45 
F(2,2)= atr(2); %.2     .22      .1  
F(3,3)= atr(3); %.1     .22      .15

C = C + F;

return;
