% Simulation environment of a 2-link planar manipulator
% Copyright 2011 Adriano Siqueira
% File pr2l.m
% Computes the dynamic model of a 2-DOF planar manipulator 

function [M,C,G,b] = pr2l(theta,omega,m,l,lc,grav)

if (nargin ~= 6)
  error('pr2l.m requires 6 input arguments');
end;

COS1 = cos(theta(1));
SIN2 = sin(theta(2));
COS2 = cos(theta(2));
COS12 = cos(theta(1) + theta(2));


t(1) = lc(1)*lc(1);
t(3) = l(1)*l(1);
t(4) = lc(2)*lc(2);
t(5) = l(2)*l(2);
t(6) = COS2;
t(7) = l(1)*lc(2)*t(6);
t(8) = l(1)*l(2)*t(6);



M(1,1) = m(1)*t(1) + m(2)*(t(3)+t(4)+2.0*t(7));
M(1,2) = m(2)*(t(4)+t(7));
M(2,1) = M(1,2);
M(2,2) = m(2)*t(4);
t(2) = SIN2;
t(4) = m(2)*l(1)*lc(2)*t(2);

C(1,1) = -t(4)*omega(2);
C(1,2) = -t(4)*omega(1) - t(4)*omega(2);
C(2,1) = t(4)*omega(1);
C(2,2) = 0;


t(1) = m(2)*grav;
t(3) = COS12;
t(4) = lc(2)*t(3);
t(15) = COS1;
t(18) = l(1)*t(15);

G(1,1) = m(1)*grav*lc(1)*t(15) + t(1)*(t(18)+t(4));
G(2,1) = t(1)*t(4);



b = C*omega + G ;%+[0;0.5].*omega;

return;
