% Simulation environment of a 2-link planar manipulator
% Copyright 2011 Adriano Siqueira
% File pr2l_adap.m
% Computes the adapted dynamic model of a 2-DOF planar manipulator 

function [Y] = pr2l_adap(theta,omega,alfa,l,lc,grav)

if (nargin ~= 6)
  error('pr3l.m requires 6 input arguments');
end;

COS1 = cos(theta(1));
SIN2 = sin(theta(2));
COS2 = cos(theta(2));
COS12 = cos(theta(1) + theta(2));


t(1) = lc(1)*lc(1);
t(3) = l(1)*l(1);
t(4) = lc(2)*lc(2);
t(5) = l(2)*l(2);
t(6) = COS2;
t(7) = l(1)*lc(2)*t(6);
t(8) = l(1)*l(2)*t(6);

Y(1,1) = t(1)*alfa(1) + grav*lc(1)*COS1;

Y(1,2) = (t(3)+t(4)+2*t(7))*alfa(1) +(t(4)+t(7))*alfa(2)...
         - l(1)*lc(2)*SIN2*(2*omega(1)*omega(2) + omega(2)^2)...
         + grav*lc(1)*COS1 + grav*lc(2)*COS12;

Y(2,1) = 0;

Y(2,2) = (t(4)+t(7))*alfa(1) +(t(4))*alfa(2)...
         + l(1)*lc(2)*SIN2*(omega(1)^2)...
         + grav*lc(2)*COS12;


return;
