function nmh2 = normh2(varargin)
%NORMH2 Continuous H2 norm.
%
% [NMH2] = NORMH2(SYS) computes the H2 norm of LTI system SYS
%    If SYS is continuous-time and is not strictly proper, INF is returned.  
%    Otherwise, the H2 norm is computed as
%
%            NMH2 = trace[CPC'+DD']^0.5 = trace[B'QB+D'D]^0.5
%
%    where P is the controllability grammian, and Q is the observability
%    grammian.

% R. Y. Chiang & M. G. Safonov 8/91
% Revised by  M.G. Safonov 11/15/02

% Copyright 1988-2004 The MathWorks, Inc.
%       $Revision: 1.1.6.1 $
% All Rights Reserved.

nag1=nargin;
[emsg,nag1,xsflag,Ts,a,b,c,d]=mkargs5x('ss',varargin); error(emsg);

P=gram(ss(a,b,c,d,Ts),'c');
	
if isequal(Ts,0),
    if norm(d,'fro') > sqrt(eps)
       nmh2 = inf;
       return
	else
        d=0*d;
	end
end
	
nmh2 = sqrt(trace(c*P*c'+d*d'));
%
% ---- End of NORMH2.M % RYC/MGS %