function [bnds,info] = mussvcalc2(matin,blk,opt,mdata,gflag,SETUP)
%
% Issues
%  1) gflag is not used in mussvcalc.  It can be deleted 
%     from the input argument list.
%  2) New option: 
%       'g6'  Attempt lower bound based on wcgain multiple times 
%              (in this case 10+6*10 times), larger number typically
%              gives better lower bound. This alternative lower bound
%              code is run in addition to the power iteration.
%     Note: I modified goptvl.m so that the number following any option
%     can be more than one digit. Also, this new lower bound code
%     reverts to the standard power iteration for purely complex
%     block structures and hence is not run in this case.

%--------------------------------------------------
% Options Set-up
%--------------------------------------------------
if nargin == 2
    opt = '';
    mdata = [];
    SETUP = [];
 elseif nargin == 3
    mdata = [];
    SETUP = [];
 elseif nargin == 4
    SETUP = [];
 elseif nargin == 5
    SETUP = [];
end
if isempty(opt)
   opt = '';
end

% Map old options into new options
bigC = find(opt=='C'); opt(bigC) = 'a';
lowerr = find(opt=='r'); opt(lowerr) = 'i';
bigR = find(opt=='R'); opt(bigR) = 'm';
% Strip off 'c' and 'w'
lowerc = find(opt=='c'); opt(lowerc) = [];
lowerw = find(opt=='w'); opt(lowerw) = [];
ostruc.displaywarnings = any(opt=='d');
ostruc.fastupper = any(opt=='f');
ostruc.bestupper = any(opt=='a');
ostruc.bestuppernoscaling = any(opt=='n');
ostruc.initialize = any(opt=='i');
ostruc.multiplelower = any(opt=='m');
if ostruc.multiplelower
    ostruc.multiplelowertimes = goptvl(opt,'m',1,1); 
else
    ostruc.multiplelowertimes = 0;
end    
ostruc.silent = any(opt=='s');
ostruc.decreaselower = any(opt=='x');
ostruc.decreaselowerto2 = any(opt=='U');
ostruc.increaselower = any(opt=='t');
%ostruc.gui = any(opt=='g');
ostruc.backup = any(opt=='b');
ostruc.runmuwcglb = any(opt=='g'); 

if ~ostruc.displaywarnings
   SW = warning('off','all');
end

% Set max # of iterations in lower bound
if ostruc.decreaselowerto2
    mulbopt.CNTMAX = 2;    
elseif ostruc.decreaselower,  
    mulbopt.CNTMAX = 50;
else
   % XXX mussvcalc uses 200 iterations
    mulbopt.CNTMAX = 100;  
end

% Set # of tries for muwcglb
if ostruc.runmuwcglb
   % Always run wcglb at least 10x and add 10 iterations.  This improves 
   % algorithm performance due to the way it adaptively searches for the 
   % lower bound.
    wcglbopt.Ntry = 10+10*goptvl(opt,'g',1,1); 
end    

%--------------------------------------------------
% Get matrix dimensions and reshape data into 3-D
%--------------------------------------------------
if isempty(mdata)
   mdata = rand(1,2);
end
if isa(matin,'double')
   szm = size(matin);
   if length(szm)==2
      ostruc.silent = 1;
   end
   exd = szm(3:end);
   npts = prod(exd);
   % make N-D into 3-D
   mat = reshape(matin,[szm(1) szm(2) npts]);
elseif isa(matin,'frd')
   szm = size(matin.ResponseData);
   exd = szm(3:end);
   npts = prod(exd);
   Ts = matin.Ts;
   % make N-D into 3-D
   mat = reshape(matin.ResponseData,[szm(1) szm(2) npts]); % note RESPONSEDATA is DOUBLE
else
   error('Only DOUBLE and FRD data are allowed')
end
Mr = szm(1);
Mc = szm(2);

if isempty(mat) & length(blk)==0
   info.bnds = [];
   info.pvec = [];
   info.dvec = [];
   info.gvec = [];
   info.sens = [];
   info.blk = blk; 
   if ~ostruc.displaywarnings
      warning(SW);
   end
   return
end

%--------------------------------------------------
% Check blk structure and get block indices
%--------------------------------------------------

% blk must be Nx2 matrix of integers
if size(blk,2)~=2 | any(floor(blk(:))~=ceil(blk(:)))
    error('blk is invalid. blk must be Nx2 matrix of integers');
end

nblk = size(blk,1);
for ii=1:nblk
   if all( blk(ii,:) == [ 1 0] )
      blk(ii,:)  = [ 1 1] ;
   end
   if all( blk(ii,:) == [-1 1] )
      blk(ii,:)  = [-1 0] ;
   end
end
for ii=1:nblk
   if all(blk(ii,:) == [-1 -1])
      blk(ii,:) = [-1 0];
   end
end

% Get indices from blk2idx plus a few other indices/masks needed for mu
% Also grab indices for real blocks only and for complex blocks only
index = muindex(blk);
index.allreal.realidx = muindex(index.allreal.realblk);
index.allcomp.compidx = muindex(index.allcomp.compblk);
ridxm = index.ridxm; % col2 of tblkp
cidxm = index.cidxm; % col1 of tblkp

% Set lower bound opt structure for pure real problems 
% Use to increase # of power iters if matrix data is real
if index.allcomp.num==0 
   mulbopt_r = mulbopt;
   mulbopt_r.CNTMAX = 300;
end

% Check compatibility of block and matrix dimensions
if index.rdimm~=Mr || index.cdimm~=Mc
    error('Dimensions of matrix are incompatible with the blk dimensions.')
end

if index.allreal.num == nblk && ostruc.displaywarnings
    warning('Pure-real MUSSV problem, LowerBound convergence difficulties expected.');
end

%--------------------------------------------------
% Initialize outputs:
%  Scalings/Delta stored as row vectors in
%  the same format as the old mussv code
%--------------------------------------------------
wp=sum(sum(index.masks.DeltaFull_mask))+index.allrep.num;
wd=sum(sum(index.masks.Drep_mask))+index.full.num;
wg=sum(index.allreal.repeated);
bnds = zeros([1 2 exd]);
rowp = zeros([1 wp exd]);
rowd = zeros([1 wd exd]);
rowg = zeros([1 wg exd]);
sens = zeros([1 nblk exd]);
if ~isempty(SETUP)
    if isa(matin,'frd')
        bnds = frd(bnds,matin.Frequency,Ts,'Units',matin.Units);
        sens = frd(sens,matin.Frequency,Ts,'Units',matin.Units);
        rowd = frd(rowd,matin.Frequency,Ts,'Units',matin.Units);
        rowp = frd(rowp,matin.Frequency,Ts,'Units',matin.Units);
        rowg = frd(rowg,matin.Frequency,Ts,'Units',matin.Units);
    end
    info.dvec = rowd;
    info.pvec = rowp;
    info.gvec = rowg;
    info.sens = sens;
    info.blk = blk; 
    info.bnds = bnds;
    if ~ostruc.displaywarnings
        warning(SW);
    end
    return
end

%--------------------------------------------------
% Start pointwise computation of bounds 
%--------------------------------------------------
Deltaprev = [];
if ~ostruc.silent  
   strl = ['points completed (of ' int2str(npts) ') ... ' ];
   fprintf(strl)
end
density = mdata(2);
pcnt = 1;
lptxt = '';
fnpts = 10*floor(npts/10);
for ii = 1:npts
    M = mat(:,:,ii);
    Dr = eye(Mr); 
    Dc = eye(Mc); 
    Gcr = zeros(Mc,Mr);
                
    %--------------------------------------------------
    % Pre-scale: Osborne's
    %--------------------------------------------------
    if ~ostruc.bestuppernoscaling
        [dMd,Dr_os,Dci_os] = osbal(M,index);
        Dc_os = inv(Dci_os);
    else
        dMd = M;
        Dr_os = eye(Mr);
        Dc_os = eye(Mc);
    end
    
    %--------------------------------------------------
    % Scale Data and Compute Initial Upper Bound
    %--------------------------------------------------
    scale = max(max(abs(dMd)))/5;
    if scale<10*eps,
        scale = 1;
    else
        dMd=dMd/scale;
    end
    [uu,ss,vv]=svd(dMd);
    ub = max(diag(ss));
    
    %--------------------------------------------------
    % Lower Bound: Power iteration    
    %--------------------------------------------------
    if index.allcomp.num==0 & ostruc.runmuwcglb
        % Don't run power iteration for pure-real if user
        % sets option to call muwcglb
        lb = 0;        
        Delta = zeros(Mc,Mr);
        repc = index.allrep.allcols;
        repr = index.allrep.allrows;        
        Delta(repc,repr) = 1e50*eye(length(repc));
        fullc = index.full.allcols;
        fullr = index.full.allrows;        
        Delta(fullc,fullr) =  1e50*index.masks.DeltaFull_mask;
    else  
        if ostruc.initialize || ii==1    
            % initialize lower bound with no previous info
            bstart = vv(:,1);
            wstart = bstart;
        else        
            % initialize lower bound with with previous result
            bstart = bprev;
            wstart = wprev;
        end
        if index.allcomp.num==0 & isreal(dMd)
           % Increase # of power iterations if pure real unc/pure real data
           [lb,Delta,b,w]=mmupiter(dMd,index,bstart,wstart,mulbopt_r);
        else
           [lb,Delta,b,w]=mmupiter(dMd,index,bstart,wstart,mulbopt);
        end
        
        for jj=1:ostruc.multiplelowertimes
            % randomly initialize lower bound
            bstart = crand(Mc,1);    
            bstart = bstart/(norm(bstart) + 10*eps);
            wstart = crand(Mc,1);    
            wstart = wstart/(norm(wstart) + 10*eps);
            if index.allcomp.num==0 & isreal(dMd)
               % Increase # of power iterations if pure real unc/pure real data
               [lbtry,Deltatry,btry,wtry]=mmupiter(dMd,index,bstart,wstart,mulbopt_r);   
            else
               [lbtry,Deltatry,btry,wtry]=mmupiter(dMd,index,bstart,wstart,mulbopt);   
            end
            if lbtry>lb
                lb = lbtry; 
                Delta = Deltatry; 
                b = btry; 
                w = wtry; 
            end
        end
        bprev = b;
        wprev = w;
    end
    
    %--------------------------------------------------
    % Fast Upper Bound
    %--------------------------------------------------
    if index.allreal.num>0 && ( ub > 1.005*max([lb 10*eps]) )
        [ub,Dr,Dc,Gcr] = mufastub(dMd,index,[ub lb]);    
    else
        Dr = eye(Mr);
        Dc = eye(Mc);
        Gcr = zeros(Mc,Mr);
    end
        
    if (~ostruc.fastupper) && ( ub > 1.005*max([lb 10*eps]) )
        %--------------------------------------------------
        % Gradient Descent Upper Bound
        %--------------------------------------------------
        DGinit.Dr=Dr;  
        DGinit.Dc=Dc;  
        DGinit.Gcr=Gcr; 
        DGinit.ub=ub;
        [ub,Dr,Dc,Gcr] = mudescentub(dMd,index,DGinit);  
        
        %--------------------------------------------------
        % LMI Upper Bound: Use Hot start
        %--------------------------------------------------
        if ostruc.bestupper && ( ub > 1.005*max([lb 10*eps]) )
            DGinit.Dr=Dr;  
            DGinit.Dc=Dc;  
            DGinit.Gcr=Gcr;  
            DGinit.ub=ub;
            [ub,Dr,Dc,Gcr] = mulmiub(dMd,index,DGinit);
        end            
    end
    
    %--------------------------------------------------
    % Lower Bound: WCGAIN (WCLOWC) on real parts and
    %    power iteration on complex parts
    %--------------------------------------------------
    if ostruc.runmuwcglb && index.allreal.num>0 && ub>100*eps
        if ostruc.initialize || ii==1    
            % initialize lower bound with no previous info
            deltareal0 = [];
        else
            % initialize lower bound with with previous result
            deltareal0 = deltarealprev;
        end
        [lbtry,Deltatry,deltareal] = muwcglb(dMd,index,deltareal0,[ub lb],wcglbopt);
        deltarealprev = deltareal;
        if lbtry>lb
            lb = lbtry;
            Delta = Deltatry;
        end        
    end    
        
    %--------------------------------------------------
    % Unscale and prepare outputs:
    %--------------------------------------------------
    if ub<10*eps 
        % Need ub>0 b/c mussv outputs D/G scales in the balanced
        % form (ami2ynrow will cause problems if ub==0)
        ub = 10*eps; 
    end          
    ub = ub*scale;
    lb = lb*scale;
    Delta = Delta/scale;     
    normM=norm(M);
    
    if ub<normM
        Dr = Dr_os'*Dr*Dr_os;
        Dc = Dc_os'*Dc*Dc_os;
        Gcr = scale*(Dc_os'*Gcr*Dr_os);
        
        % Uncomment to test results: 
        % psdami should be psd (within numerical errors)
        %psdami = ub^2*Dc-M'*Dr*M-j*(Gcr*M-M'*Gcr');
        %min(real(eig(psdami)))

        % Store scalings as row vectors
        for i=1:nblk
            if blk(i,2)==0
                rptr = ridxm(i):(ridxm(i+1)-1);
                cptr = cidxm(i):(cidxm(i+1)-1);
            else
                rptr = ridxm(i);
                cptr = cidxm(i); 
            end
            Dcell{i} = Dr(rptr,rptr);
            Gcell{i} = Gcr(cptr,rptr);
        end
        [trowd,trowg] = ami2ynrow(Dcell,Gcell,blk,ub);
                
    else
        ub = normM;
        % sigmaub stores identity D-scales and a zero G-scales
        [trowd,trowg] = sigmaub(blk);
    end
    bnds(1,:,ii) = [ub lb];


    % Store Delta as rep reals, rep comp, complex full
    trowp = zeros(1,wp);
    cnt=1;
    for i=1:nblk
        if blk(i,1)<0 && blk(i,2)==0
            rptr = ridxm(i);
            cptr = cidxm(i);
            trowp(cnt) = Delta(cptr,rptr);
            cnt=cnt+1;
        end
    end
    for i=1:nblk
        if blk(i,1)>0 && blk(i,2)==0
            rptr = ridxm(i);
            cptr = cidxm(i);
            trowp(cnt) = Delta(cptr,rptr);
            cnt=cnt+1;
        end
    end    
    ridxm=index.full.allrows;
    cidxm=index.full.allcols;
    tmp = Delta(cidxm,ridxm);
    tmpidx = find(index.masks.DeltaFull_mask);
    trowp(index.allrep.num+1:end) = tmp(tmpidx);
        
    if wg,  rowg(1,:,ii) = trowg;  end
    rowd(1,:,ii) = trowd;
    rowp(1,:,ii) = trowp;
       
    %--------------------------------------------------
    % Sensitivity calculation
    %--------------------------------------------------
    [tdl,tdr] = mussvunwrap(trowd,blk);
    sm = tdl*M/tdr;
    ridxm = index.ridxm; % col2 of tblkp
    cidxm = index.cidxm; % col1 of tblkp
    if nblk == 1
      sens(1,1,ii) = 1;
    else
      for ib=1:nblk
         sensr = sm([ridxm(ib):ridxm(ib+1)-1],[1:cidxm(ib)-1 cidxm(ib+1):Mc]);
         sensc = sm([1:ridxm(ib)-1 ridxm(ib+1):Mr],[cidxm(ib):cidxm(ib+1)-1]);
         sens(1,ib,ii) = norm(sensr) + norm(sensc);
      end
    end
        
    %--------------------------------------------------
    % Display progress information
    %--------------------------------------------------    
%     if ostruc.gui && (ii/density >= pcnt)
%         set(mdata(1),'string',[int2str(ii) '/' int2str(npts)]);
%         drawnow;
%         pcnt = pcnt+1;
%     end

   if ~ostruc.silent
      fii = 10*floor(ii/10);
      if fii==ii || ii<=10 || ii>fnpts
         ptxt = int2str(ii);
         llptxt = length(lptxt);
         for count=1:llptxt
            fprintf('\b');
         end
         fprintf(ptxt);
         lptxt = ptxt;
      end
   end
            
end % for ii

if ~ostruc.silent && ostruc.backup
   nonewline = repmat('\b',1,length(strl)+length(lptxt));
   fprintf(nonewline)
elseif ~ostruc.silent
   fprintf('\n')
end

% Scale the upper bound by a small amount due to numerical rounding
bnds(1,1,:) = (1+5e-7)*bnds(1,1,:);
info.bnds = bnds;

% Store Outputs
if isa(matin,'frd')
   bnds = frd(bnds,matin.Frequency,Ts,'Units',matin.Units);
   sens = frd(sens,matin.Frequency,Ts,'Units',matin.Units);
   rowd = frd(rowd,matin.Frequency,Ts,'Units',matin.Units);
   rowp = frd(rowp,matin.Frequency,Ts,'Units',matin.Units);
   rowg = frd(rowg,matin.Frequency,Ts,'Units',matin.Units);
end

info.dvec = rowd;
info.pvec = rowp;
info.gvec = rowg;
info.sens = sens;
info.blk = blk; 
info.bnds = bnds;

% Test code
%[VDelta,VSigma,VLmi] = mussvextract(info);
%[norm(Delta-VDelta(:,:,end)), norm(Dr-VLmi.Dr(:,:,end)), ...
%    norm(Dc-VLmi.Dc(:,:,end)), norm(Gcr-VLmi.Gcr(:,:,end))]

if ~ostruc.displaywarnings
   warning(SW);
end


