function [cm,dm,mm,info]=scdloopmargin(sys,blocks,ports,varargin)
%See LOOPMARGIN for help
%

%   Copyright 2007 The MathWorks, Inc. and MUSYN Inc.

% Check for SCT
val = license('test','Simulink_Control_Design') && ~isempty(ver('slcontrol'));
if val==0
   error('Simulink LOOPMARGIN requires Simulink Control Design.');
end

% Initialize outputs
cm = []; 
dm = []; 
mm = [];
info = []; 

% Error checking on blocks/ports
if isa(blocks,'char')
   blocks = cellstr(blocks);
elseif ~iscellstr(blocks)
   error('Blocks should be a char array or a cell array of chars.');   
end

nb = size(blocks,1);
np = length(ports);
if nb~=np
   error('Number of Ports does not match the number of blocks.');
end

% Get Inputs
op =[];
opstr =[];

ni = nargin -3;
% Find the optional loopmargin argument
for i=ni:-1:1
   if isa(varargin{i},'char')
      opstr = varargin{i};
      varargin(i) = [];
   elseif isa(varargin{i},'opcond.OperatingPoint')
      op = varargin{i};
   end
end

% Blocks is a cell array of strings 
for i=1:nb
   lmio(i)=linio(blocks{i},ports(i),'outin','on');
end   

try
   [L,opx] = linearize(sys,lmio,varargin{:});
catch Ex
   throwAsCaller(MException('Robust:loopmargin:LinearizeError',['In the "loopmargin" command, the Simulink model %s couldn''t be'...
      ' linearized due to the following error: %s'],sys,Ex.message))
end
L = -L;

% Return operating point, linearization points 
if isa(opx,'opcond.OperatingPoint')
   info.OperatingPoint = opx;
elseif ~isempty(op)
   info.OperatingPoint = op;
else
   info.OperatingPoint = operpoint(sys);
end
info.LinearizationIO = lmio;
info.SignalNames = L.InputName;
info.L = L;

% Compute margins (classical, disk, multi-loop disk)
if size(L,1)==0 || size(L,2)==0
    % Should we error here?
    return;
else
    userin = lower(opstr( find(opstr~=','&opstr~=' ') ));
    if length(userin)==1
        cm = loopmargin(L,opstr);
        dm = info;
        mm = [];
        info = [];
    elseif length(userin)==2
        [cm,dm] = loopmargin(L,opstr);
        mm = info;
        info = [];
    else
        [cm,dm,mm] = loopmargin(L);
    end
end

