% STACK  Stack uncertain objects into UMAT array.
% 
%MATU = STACK(ARRAYDIM,MATU1,MATU2,...) produces an array of UMAT
%MATU by stacking the UMAT MATU1,MATU2,... along the array 
%dimension ARRAYDIM.  All uncertain atoms must have the same 
%number of inputs and outputs.
% 
%For example, if MATU1 and MATU2 are two uncertain attoms with the 
%same I/O dimensions,
%    * STACK(1,MATU1,MATU2) produces a 2-by-1 UMAT array
%    * STACK(2,MATU1,MATU2) produces a 1-by-2 UMAT array
%    * STACK(3,MATU1,MATU2) produces a 1-by-1-by-2 UMAT array.
% 
%You can also use STACK to concatenate UMAT arrays MATU1,MATU2,...
%along some array dimension ARRAYDIM.
% 
%See also APPEND, HORZCAT, VERTCAT

function C = stack(dim,varargin)
% Copyright 2003-2004 The MathWorks, Inc.

nin = nargin;
if nin>=3 && ~(isa(dim,'double') && ndims(dim)==2 && all(size(dim)==[1 1]) && ...
      dim>0 && floor(dim)==ceil(dim))
   error('Need at least 3 arguments, and DIM should be a positive integer.');
end
if nin==3
   C = stack(dim,umat(varargin{1}),umat(varargin{2}));
elseif nin==2
   C = varargin{1};
elseif nin>3
   C = stack(dim,stack(dim,umat(varargin{1}),umat(varargin{2})),varargin{3:end});
end
