%WCNORM  Worst-case norm of an uncertain element.
%   [MAXNORM,MAXNORMUNC,INFO] = WCNORM(M) calculates the worst-case 
%   norm of an uncertain element M.  "Worst-case" refers to maximum over
%   the uncertainty (and array dimensions), and "norm" is measured by
%   the maximum singular value of the matrix.
%
%   The outputs arguments for WCNORM are:
%     MAXNORM  Structure with the following fields:
%            LowerBound: Upper bound on worst-case gain. 
%            UpperBound: Lower bound on worst-case gain. 
%     MAXNORMUNC  Structure of worst-case uncertainty values. 
%     INFO  Structure with the following fields:
%           Sensitivity: Structure of percentages corresponding to MAXNORM 
%                        sensitivity to variations in each uncertainty
%                        level.
%            ArrayIndex: Array index (for arrays of uncertain matrices)
%                        where the maximum norm occurs.  Value is scalar,
%                        for single index use.
%
%   [MAXNORM,MAXNORMUNC,INFO] = WCNORM(M,OPTS) specifies options
%   described in OPTS.  See WCGOPT for more details on the options for
%   WCNORM. 
% 
%   See also NORM, SVD, WCGAIN, WCGOPT.

function varargout = wcnorm(varargin)
% Copyright 2003-2004 The MathWorks, Inc.

nout = nargout;
if nout >= 1
   varargout = cell(nout,1);
else
   varargout = cell(1,1);
end
varargin{1} = umat(varargin{1});
[varargout{:}] = wcnorm(umat(varargin{1}),varargin{2:end});