function [m,pflag,errstring] = pvset(m,property,value)
%PVSET  Set properties of uncertain objects.
%  
%MAT = PVSET(MAT,'Property1',Value1,'Property2',Value2,...)
%sets the values of the properties with exact names 'Property1',
%'Property2',...
%  
%See also SET.

% Copyright 2003-2006 The MathWorks, Inc.

% the only valid properties are particular ATOMNAME.
% TMW's requirements:
%   Same Class, Same Size, Same Name -> do it
%   Same Class, Same Size, Same Type, Same Data -> do it
pflag = 1;
errstring = '';
idx = gblkidx(m,property);
if length(idx)==1
   if isa(value,'atom')
        [idequal,nameeq,classeq,sizeeq,dataeq,nomeq] = ...
            isequal(m.AtomCell{idx},value);
      
        % Cache contains: UNames, UNamesCS, nblk, Sizes, Classes
        if classeq && sizeeq && nameeq
            % Same class, size, name but different data or nom
            % Cache Update: Nothing
            m.AtomCell{idx} = value;
            if dataeq && nomeq
                m.Info(idx) = 2;    % Same Name / Same Data 
            else                
                m.Info(idx) = 1;    % Same Name / New Data
            end
        elseif classeq && sizeeq && dataeq && nomeq
            % Same class, size, data, nom but different name
            % Cache Update: UNames, UNamesCS
            m.AtomCell{idx} = value;                  
            m.Info(idx) = 3;            % Name changed
            name = pvget(value,'Name');
            m.Cache.UNames{idx} = name;
            m.Cache.UNamesCS(idx) = sum(double(name));               
        else
            pflag = 0;
            errstring = 'Desired substitution is illegal or too complicated - use USUBS.';
        end
         
     
%       if sizeeq
%          m.AtomCell{idx} = value;                  
%          if nameeq 
%             % Cache.(UNames,UNamesCS,nblk,Sizes,Classes)
%             if dataeq && nomeq
%                m.Info(idx) = 2;  % Size/Name/Data/Nominal unchanged
%                % Cache Unchanged
%             else  
%                m.Info(idx) = 1;  % Size/Name OK, Data/Nominal (or class) different   
%                m.Cache.Classes{idx} = class(value);
%             end   
%          else
%              
%             m.Info(idx) = 3;  % SizeOk, but Name Changed
%             m.Cache.Classes{idx} = class(value);
%             name = pvget(value,'Name');
%             m.Cache.UNames{idx} = name;
%             m.Cache.UNamesCS(idx) = sum(double(name));
%          end
%       else
%          pflag = 0;
%          errstring = 'Substitution is disallowed due to size mismatch.';
%       end
      
%      [exact,cstd,csn] = isequal(m.AtomCell{idx},value);

%       if ~exact & cstd
%          m.AtomCell{idx} = value;
%          m.Info(idx) = 1;
%       elseif ~exact & csn
%          m.AtomCell{idx} = value;
%          m.Info(idx) = -1;
%       elseif exact
%          m.AtomCell{idx} = value;
%          m.Info(idx) = 0;
%          disp('ATOMDEPENDNECY PVSET')
%       elseif ~exact
%          %XXXMAJOR, see PVGET in the atomdependency list.
%          % also look into replacing an ATOM with a legal value (certain).
%          szv = size(value);
%          sza = size(m.AtomCell{idx});
%          if isequal(szv,sza)
%             m.AtomCell{idx} = value;
%             m.Info(idx) = 1;
%          else
%             error('Substitution is disallowed due to size mismatch');
%          end
%          %error('Desired substitution is illegal or too complicated (use SUBS)');
%       end
   else
      pflag = 0;
      errstring = 'Desired substitution is illegal or too complicated - use USUBS.';
      % note: this happens with:  >> mat.rho = 1;
   end
else
   pflag = 0;
   errstring = ['Invalid ULTIDYN property: ' property];
end
if nargout<=1 && pflag==0
   error(errstring);
end
