%DKITOPT    Create a DKSYN OPTIONS object.
%  OPTIONS = DKITOPT('NAME1',VALUE1,'NAME2',VALUE2,...) creates a
%  options object for the command DKSYN.
% 
%  OPTIONS = DKITOPT, by itself, creates an object, initialized
%  with all default values.
%
%  DKITOPT PROPERTIES
%
%   FrequencyVector:  Frequency vector used for analysis. Default 
%      is an empty matrix which results in the frequency range and number 
%      of points  used to be chosen automatically.
%   InitialController:  Controller used to initiate first iteration.
%      Default is an empty SS object.
%   AutoIter:  Automated mu-synthesis mode. Default is 'on'. 
%   DisplayWhileAutoIter: Displays iteration progress in AutoIter
%      mode.  Default is 'off'.
%   StartingIterationNumber:  Starting iteration number. Default is 1.
%   NumberOfAutoIterations:  Number of iterations to perform, Default
%      is 10.
%   AutoScalingOrder:  State order for fitting D-scaling data. 
%      Default is 5.
%   AutoIterSmartTerminate:  Automatic termination of iteration 
%      procedure based on progress of design iteration. Default
%      is 'on'.
%   AutoIterSmartTerminateTol: Tolerance used AutoIterSmartTerminate.
%      Default is 0.005.
%   Default: A 1-by-1 structure of robopt property default values.
%   Meaning: A 1-by-1 structure text description of each property. 
%
% See Also: DKSYN, H2SYN, HINFSYN, LOOPMARGIN, MUSSV, ROUSTSTAB, 
%           ROBUSTPERF, WCGAIN, WCSENS, WCMARGIN.
function out = dkitopt(varargin)

% Copyright 2003-2004 The MathWorks, Inc.

superiorto('char','double','optunit');
validate = 1;
nin = nargin;
if ceil(nin/2)~=floor(nin/2)
   validate = varargin{end};
   nin = nargin - 1;
end
hnin = nin/2;
mkatend = 1;
semp = ss;

OptCell = {...
optunit('FrequencyVector',[],[],'Optional Frequency vector',1);...
optunit('InitialController',semp,semp,'Optional controller used to initiate iteration',1);...
optunit('AutoIter','on','on','Automated MU-SYN mode  [{''off''} | ''on'']',1);...
optunit('DisplayWhileAutoIter','off','off','Displays iteration progress in AutoIter mode [{''off''} | ''on'']',1);...
optunit('StartingIterationNumber',1,1,'Starting iteration number',1);...
optunit('NumberOfAutoIterations',10,10,'not just peak',1);...
optunit('AutoScalingOrder',5,5,' ',1);...
optunit('AutoIterSmartTerminate','on','on','  [{''off''} | ''on'']',1);...
optunit('AutoIterSmartTerminateTol',0.005,0.005,'  [{''off''} | ''on'']',1);...
};

if mkatend
   out.OptCell = OptCell;
   out = class(out,'dkitopt',gsref(0));
end
if ceil(hnin)==floor(hnin)
   for i=1:hnin
      out = set(out,varargin{2*(i-1)+1},varargin{2*i});
   end
end

% Check validity of OUT.  Without any output arguments it will
% error out with appropriate error message if not valid.
%if isequal(validate,1)
if validate==1
   isvalid(out);
end