%PVGET  Get values of public DKITOPT properties.
%  
%VALUES = PVGET(MAT) returns all public values in a cell
%array VALUES.
%  
%VALUE = PVGET(SYS,PROPERTY) returns the value of the
%single property with name PROPERTY.
%  
%See also GET.

function [out,pflag,errstring] = pvget(m,property)
% Copyright 2003-2004 The MathWorks, Inc.

pflag = 1;
errstring = '';

ons = {};
for i=1:length(m.OptCell)
   ons = [ons;{pvget(m.OptCell{i},'Name')}];
end

switch property
case ons
   idx = find(strcmp(ons,property));
   out = m.OptCell{idx};
   out = pvget(out,'Value');
case 'PropNames'
   GP = {};
   SP = {};
   SPD = {};
   for i=1:length(m.OptCell)
      nm = pvget(m.OptCell{i},'Name');
      GP = [GP;{nm}];
      SP = [SP;{nm}];
      SPD = [SPD;{pvget(m.OptCell{i},'String')}];
   end
   GPend = {'Default';'Meaning'};
   out.GPropNames = [GP;GPend];
   out.SPropNames = SP;
   out.SPropDescription = SPD;
case 'OptNames'
   out = {};
   for i=1:length(m.OptCell)
      out = [out;{pvget(m.OptCell{i},'Name')}];
   end
case 'Default'
   cc = cell(0,1);
   for i=1:length(m.OptCell)
      cc = [cc;{pvget(m.OptCell{i},'Name')};{pvget(m.OptCell{i},'Default')}];
   end
   out = struct(cc{:});
case 'OptCell'
   cc = cell(0,1);
   for i=1:length(m.OptCell)
      cc = [cc;{pvget(m.OptCell{i},'Name')};{m.OptCell{i}}];
   end
   out = struct(cc{:});
case 'Meaning'
   cc = cell(0,1);
   for i=1:length(m.OptCell)
      cc = [cc;{pvget(m.OptCell{i},'Name')};{pvget(m.OptCell{i},'String')}];
   end
   out = struct(cc{:});
otherwise
   out = [];
   pflag = 0;
   errstring = ['Invalid DKITOPT property: ''' property '''.'];
end
if nargout<=1 && pflag==0
   error(errstring);
end
