%LFTDATAN LFT data for a DYNAMICSYS object

function [newdsdata] = lftdatan(dsdata,alist,copies,L)
% Copyright 2003-2004 The MathWorks, Inc.

% If DSDATA is the original DYNAMICSYS object, and ALIST/COPIES
% are full cell-array (of atoms) along with DOUBLE of copies,
% and L is a cell-array of names of atoms to be moved into the
% top of the IO, then this computes the new DYNAMICSYS object.

natoms = length(alist);
UNames = cell(natoms,1);
for i=1:natoms
   UNames{i} = pvget(alist{i},'Name');
end
Lidx = zeros(0,1);
cmat = zeros(0,1);
szmat = zeros(0,2);
for i=1:length(L)
   idx = strmatch(L{i},UNames,'exact');
   if length(idx)==1
      Lidx = [Lidx;idx];
      szmat = [szmat;size(alist{idx})];
      cmat = [cmat;copies(idx)];
   elseif length(idx)>1
      error('Too many matches.');
   end
end
blkdim = repmat(cmat,[1 2]).*szmat;
nblk = size(szmat,1);
cp = cumsum([1;blkdim(:,1)]);
rp = cumsum([1;blkdim(:,2)]);
tblksize = sum(blkdim,1);
CN = repmat({''},[tblksize(1) 1]);
RN = repmat({''},[tblksize(2) 1]);
RG = cell(nblk,2);
CG = cell(nblk,2);
for i = 1:nblk
   RG{i,1}  = (rp(i):rp(i+1)-1);
   RG{i,2}  = [UNames{Lidx(i)} '_NC'];
   CG{i,1} = (cp(i):cp(i+1)-1);
   CG{i,2} = [UNames{Lidx(i)} '_NC'];
end
inname = [CN ; pvget(dsdata,'InputName')];
oname = [RN ; pvget(dsdata,'OutputName')];
IG = pvget(dsdata,'InputGroup');
fn = fieldnames(IG);
for i=1:length(fn)
   IG.(fn{i}) = IG.(fn{i}) + tblksize(2);
end
OG = pvget(dsdata,'OutputGroup');
fn = fieldnames(OG);
for i=1:length(fn)
   OG.(fn{i}) = OG.(fn{i}) + tblksize(1);
end
for i=1:size(RG,1)
   OG.(RG{i,2}) = RG{i,1};
end
for i=1:size(CG,1)
   IG.(CG{i,2}) = CG{i,1};
end
newdsdata = dynamicsys(length(oname),length(inname));
newdsdata = pvset(newdsdata,'InputName',inname);
newdsdata = pvset(newdsdata,'OutputName',oname);
newdsdata = pvset(newdsdata,'InputGroup',IG);
newdsdata = pvset(newdsdata,'OutputGroup',OG);

   