function out = diag(m)
% DIAG Diagonal FRD and diagonals of FRD system.
%  If V is a vector of system models, DIAG(V) puts V on the main
%  diagonal. If X is a matrix of system models, DIAG(X) is the 
%  main diagonal of X.  DIAG(DIAG(X)) is a diagonal matrix of 
%  system models.
% 
%  Example
%     x = rss(3,4,1);
%     xg = frd(x,logspace(-2,2,80));
%     mxg = diag(xg);
%     size(mxg)
%  produces a diagonal system mxg of size 4-by-4. Given multivariable
%  system xx, a vector of the diagonal elements of xxg are found 
%  using DIAG.
%     xxg = [xg(1:2,1) xg(3:4,1)];
%     m = diag(xxg)
% 
%  See also APPEND, BLKDIAG.

% Copyright 2003-2005 The MathWorks, Inc.
szm =  size(m);
flg = 0;
if szm(1)~=1 && szm(2)~=1
   %error('DIAG expects an N-by-1 or 1-by-N frequency response.')
   flg = 1; %reduce to vector
end
% Data
[data,frequency,ts] = frdata(m);
szd = size(data);
if flg==0
   n = max(szm(1),szm(2));
   outdata = zeros([n n szd(3:end)]);
   for ct=1:prod(szd(3:end))
      outdata(:,:,ct) = diag(data(:,:,ct));
   end
else
   n = min(szm(1),szm(2));
   outdata = zeros([n 1 szd(3:end)]);
   for ct=1:prod(szd(3:end))
      outdata(:,:,ct) = diag(data(:,:,ct));
   end
end
out = frd(outdata,frequency,ts);

