function [num,den,tmin] = fitmagfrdeng(mag,theta,dd,nd,wt,ct,ttol,tmax)
% MAG: NPTS-by-1
% THETA: NPTS-by-1
% DD: scalar
% ND: scalar
% WT: NPTS-by-1
% CT: NPTS-by-1
% TTOL: scalar
% TMAX: scalar

%   Copyright 2003-2008 The MathWorks, Inc.
%   $Revision: 1.1.6.7 $ $Date: 2008/12/04 23:16:27 $

% Initialize outputs
num = [];
den = [];
tmin= [];

% Form matrices used in linear program. Acp, ccp
% are used to store cutting plane constraints
npts = length(theta);
z = exp(sqrt(-1)*theta);
wtinv=wt.^(-1);
magsq = mag.^2;
cosmat = cos(theta*[dd:-1:0]);
Anum = diag(1./magsq)*cosmat(:,(end-nd):end);
Aden = cosmat; 

% PJS 5/11/06--Add cutting planes to enforce pos. def.
Acp = [];
ccp = [];
cptol = 1e-10;
Ncp = 100;
cptheta = linspace(0,pi,Ncp); 
cptheta = cptheta(:);
cpcosmat = cos(cptheta*[dd:-1:0]);
Acp = [Acp; -cpcosmat(:,(end-nd):end) zeros(Ncp,dd+1)];
ccp = [ccp; repmat(-cptol,Ncp,1)];
Acp = [Acp; zeros(Ncp,nd+1) -cpcosmat];
ccp = [ccp; repmat(-cptol,Ncp,1)];               

% Initialize Bisection
tub = min(tmax,10); 
tlb = 0;
t = (tub+tlb)/2;
flg = -1;

% Check if linprog is in path
%lpflg = exist('linprog','file');
% GJB 17Sept08: pete linprog still gives some goofy  and SLOW
%                 results use LMI always

lpflg = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%     Solve via bisection and linear programming
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
goodpolya = []; goodpolyb = [];
if lpflg
	opts = optimset('Display','off');
else
   opts = zeros(1,5);
   opts(end) = 1;
end

infeascnt = 0;
feasible = false;
cpcnt = 0;

while tub-tlb > (ttol*t +ttol) & (tub <= tmax)
    % Set up Linear Program:  Find x such that A*x <= c
	% where x= [a_nd ... a0  b_{dd} b_{dd-1} ... b0]
    % [PJS 5/12/06: Since the optimal b_dd may be <0, you can't assume b_dd=1]
	R = diag(1+t*wtinv)*diag( ct~=-1 ) + diag( ct==-1 );
	L = inv( diag(1+t*wtinv)*diag( ct~=1 ) + diag( ct==1 ) );
    A = [Anum -R*Aden; -Anum L*Aden; Acp]; 
    c = zeros(2*npts,1);
	c = [c; ccp];

	if lpflg
      [x,fval,flg]=linprog([],A,c,[],[],[],[],[],opts);
    elseif 1
        % Call lmilab if linprog is not in path
        setlmis([]);
        x = lmivar(2,[size(A,2) 1]);
        for i1 = 1:size(A,1);         
            lmiterm([i1 1 1 x],A(i1,:),1/2,'s')
            lmiterm([-i1 1 1 0],c(i1))
        end
        lmisys = getlmis;
        opts = zeros(1,5);
        opts(end) = 1;
        [fval,x] = feasp(lmisys,opts);
        if isempty(fval) || fval > 0
            flg = -1;
        else
            flg = 1;
        end
	end

	% Check if LP was feasible: GJB changes 7Oct05
	if flg<=0  %LP infeasible   
        if ~feasible
            infeascnt = infeascnt+1;
            tlb = t;
            if tub>=tmax
                tub=2*tub;
            else
                tub = min(2*tub,tmax);
            end
        else
            tlb=t;
        end
        t = (tub+tlb)/2;     
	else  % LP feasible
     % Check that polynomials related to a and b have no roots 
     % on the unit circle
     a = x(1:(nd+1));
     polya = [ (a(1:end-1)/2)'   a(end)  fliplr((a(1:end-1)/2)')];
     rootsa = roots(polya);

     % GJB 2Nov04: pete says increase tolerance
     % aidx= find( abs(abs(rootsa)-1) < 1e3*nd*eps
     aidx= find( abs(abs(rootsa)-1) < 1e6*nd*eps );
     newtheta = [];
     if ~isempty(aidx)
       % Found root of a on unit disk. Store cutting-plane frequencies.
       temp  = angle(rootsa(aidx))';
       idx = find( (temp>=0) & (temp <=pi) );
       newtheta = [.99; 1; 1.01]*temp(idx);
       newtheta = reshape(newtheta, 3*length(idx) ,1);
     end

     b = x(nd+2:end);
     polyb = [ (b(1:end-1)/2)'   b(end)  fliplr((b(1:end-1)/2)')];
     rootsb = roots(polyb);
     % GJB 2Nov04: pete says increase tolerance
     % bidx= find( abs(abs(rootsb)-1) < 1e3*dd*eps );
     bidx= find( abs(abs(rootsb)-1) < 1e6*dd*eps );
     if ~isempty(bidx)
       % Found root of b on unit disk. Store cutting-plane frequencies.
       temp  = angle(rootsb(bidx))';
       idx = find( (temp>=0) & (temp <=pi) );
       newtheta2 = [.99; 1; 1.01]*temp(idx);
       newtheta2 = reshape(newtheta2, 3*length(idx) ,1);
     end

     if ( ~isempty(aidx) | ~isempty(bidx) )
       cpcnt = cpcnt+1;
       % Pete changed 7Oct05 to address overbounding issues
       if cpcnt < 50 %20
          % Form cutting planes:
          %    a(w0)>= tol for w0 where a(w0)=0
          %    and b(w1)>= tol for w1 where b(w1)=0       
          if ~isempty(aidx)
                nnt = length(newtheta);
                newcosmat = cos(newtheta*[nd:-1:0]); 
                Acp = [Acp; -newcosmat zeros(nnt,dd+1)];
                ccp = [ccp; repmat(-cptol,nnt,1)];
          end
          if ~isempty(bidx)
                nnt2 = length(newtheta2);
                newcosmat2 = cos(newtheta2*[dd:-1:0]); 
                Acp = [Acp; zeros(nnt2,nd+1) -newcosmat2];
                ccp = [ccp; repmat(-cptol,nnt2,1)];               
          end                 
       else           
        flg = -1;
        cpcnt = 0;
        if ~feasible
            infeascnt = infeascnt+1;
            tlb = t;
            if tub>=tmax
                tub=2*tub;
            else
                tub = min(2*tub,tmax);
            end           
            %tub = 2*tub;
        else
            tlb=t;
        end
        t = (tub+tlb)/2;     
       end
     else
       feasible = true;
       cpcnt = 0;
       % Found a solution that can be factorized
       %xgood = x; Agood=A; cgood = c;Rgood = R; tgood = t;
       goodpolya = polya; 
       goodpolyb = polyb;
       g=abs(polyval(goodpolya,z)./polyval(goodpolyb,z));
       fit=max([(g./magsq(1:npts)-1)./wtinv(1:npts);...
                (magsq(1:npts)./g-1)./wtinv(1:npts)]);
       % We should have: fit <= t 
       tub = fit;        
       t = (tub+tlb)/2;
     end
	end % if flg<=0 
%    fprintf('tub = %4.2f \t t = %4.2f \t tlb = %4.2f',tub,t,tlb);
%    fprintf('\t flg = %i \t feas = %i \n',flg,feasible);
end % while tub-tlb > ttol

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
%     Use spectral factorization to recover sys
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
if isempty(goodpolya)
  %disp('No feasible solution. Increase tmax, nd, or dd.');
  tmin = tmax;
  return;
else
  % Store stable roots and then factor goodpolya
  rootsa = roots(goodpolya); 
  idxa = find(abs(rootsa)<1-10*eps);
  stbrootsa = rootsa(idxa);

  num = poly(stbrootsa);
  num = real(num*sqrt( goodpolya(end)*prod(-1./stbrootsa) ));

  % Store stable roots and then factor goodpolya
  rootsb = roots(goodpolyb);
  idxb = find(abs(rootsb)<1-10*eps);  
  stbrootsb = rootsb(idxb);
  den = poly(stbrootsb);
  den = real(den*sqrt( goodpolyb(end)*prod(-1./stbrootsb) ));

  % Find tmin
  %g=abs( polyval(goodpolya,z)./polyval(goodpolyb,z) );
  %tmin=max([(g./magsq(1:npts)-1)./wtinv(1:npts);...
  %            (magsq(1:npts)./g-1)./wtinv(1:npts)]);
end

% keyboard
% 
% if 0
% bestmag = abs(frd(tf(num,den,1),theta));
% %fac=1; magtry = abs(frd(tf(num,den,1),theta));
% [zz,pp,kk]=zpkdata(tf(num,den,1));
% gg=kk*0.96*sqrt(0.5/0.11);
% ntry = gg*[1 -.5];
% dtry = [1 -.11]*sqrt(0.5/.11);
% tftry = tf(ntry,dtry,1);
% magtry = abs(frd(tftry,theta));
% semilogx(theta,mag,'r',bestmag,theta,squeeze(get(magtry,'Responsedata')));
% 
% polya = [ntry(1)*ntry(2) sum(ntry.^2) ntry(1)*ntry(2)];
% polyb = [dtry(1)*dtry(2) sum(dtry.^2) dtry(1)*dtry(2)];
% 
% xx=zeros(size(x));
% aa = polya(1:nd+1)*2;
% aa(end) = aa(end)/2;
% 
% bb = polyb(1:dd+1)*2;
% bb(end) = bb(end)/2;
% 
% if bb(1)<0
%     aa=-aa;
%     bb=-bb;
% end
% 
% xx(1:nd+1) = aa;
% xx(nd+2:end) = bb(2:end);
% 
% 
% % Store stable roots and then factor goodpolya
% rootsa = roots(polya); 
% idxa = find(abs(rootsa)<1-10*eps);
% stbrootsa = rootsa(idxa);
% 
% num2 = poly(stbrootsa);
% num2 = real(num2*sqrt( polya(end)*prod(-1./stbrootsa) ));
% 
% % Store stable roots and then factor polya
% rootsb = roots(polyb);
% idxb = find(abs(rootsb)<1-10*eps);  
% stbrootsb = rootsb(idxb);
% 
% den2 = poly(stbrootsb);
% den2 = real(den2*sqrt( polyb(end)*prod(-1./stbrootsb) ));
% 
% idx=find(Agood*xx>cgood);
% idx = idx(1);
% 
% aomega = cos( (nd:-1:0)*theta(idx))*aa(:);
% bomega = cos( (dd:-1:0)*theta(idx))*bb(:);
% %[c(idx)-A(idx,:)*xx,  aomega/magsq(idx) - bomega]
% [cgood(idx)-Agood(idx,:)*xx,  bomega*Rgood(idx,idx)-aomega/magsq(idx)]
% [bode(tf(num2,den2,1),theta(idx))^2  ,aomega/bomega]
% 
% end
