function [MAXGAIN,WCU,INFO] =  wcgain(m,varargin)
%WCGAIN   Worst-case gain of an FRD.
%   [MAXGAIN,MAXGAINUNC,INFO] = WCGAIN(SYS) calculates the worst-case gain
%   of an FRD.  "Worst-case" refers to maximum over array dimensions, and
%   "norm" is measured by the maximum over frequency of the maximum singular
%   value of the frequency response matrix (ie., the H-infinity norm).
% 
%   MAXGAIN  Structure with the following fields:
%            LowerBound: Worst-case gain (same as UpperBound, included for
%                        consistency with USS/WCGAIN and UFRD/WCGAIN). 
%            UpperBound: Worst-case gain (same as LowerBound, included for
%                        consistency). 
%     CriticalFrequency: Frequency at which the maximum gain occurs.
%   MAXGAINUNC  Structure with no fields (for consistency with USS & UFRD)
%   INFO  Structure with the following fields:
%           Sensitivity: Structure with no fields (for consistency) 
%             Frequency: Frequency vector used in analysis.
%            ArrayIndex: Array index (for arrays of systems) where the
%                        maximum gain occurs.  Value is scalar, for single
%                        index use.
%
%   [MAXGAIN,MAXGAINUNC,INFO] = WCGAIN(SYS,OPTS) specifies options
%   described in OPTS.  See WCGOPT for more details on the options for
%   WCGAIN. 
% 
%   See also LOOPMARGIN, MUSSV, LTI/NORM, ROBUSTSTAB, WCGOPT, WCSENS, WCMARGIN.

% Copyright 2003-2004 The MathWorks, Inc.

[MAXGAIN,WCU,INFO] = wcgain(ufrd(m),varargin{:});
