function sys=mktito(sys,nmeas,ncont)
% SYS=MKTITO(SYS,NMEAS,NCONT) adds TITO (two-input-two-output)
%    partitioning to an LTI system by setting its InputGroup 
%    and OutputGroup properties as follows:
%         [r,c]=size(SYS);
%         set(SYS,'InputGroup', struct('U1',1:c-NCONT,'U2',c-NCONT+1:c));
%         set(SYS,'OutputGroup',struct('Y1',1:r-NMEAS,'Y2',r-NMEAS+1:r));
%    Any pre-existing partition of SYS is overwritten.
% 
% See also AUGW HINFSYN H2SYN SDHFSYN

% R. Y. Chiang & M. G. Safonov

% Copyright 1988-2004 The MathWorks, Inc.
% $Revision: 1.1.6.1 $ $Date: 2004/07/31 23:57:19 $

% Based on Gahinet's 01/98 partitioning specification
% Rev 11/15/02 M. G. Safonov
% Rev 6/11/04 M. G. Safonov

[r,c]=size(sys);
if ~isa(sys,'lti'), error('SYS is not an LTI object'), end
if r<nmeas, error('NMEAS exceeds SYS output dimension'), end
if c<ncont, error('NCONT exceeds SYS input dimension'), end

% set(sys,'InputGroup',{ 1:c-ncont 'U1'; c-ncont+1:c 'U2' });
% set(sys,'OutputGroup',{ 1:r-nmeas 'Y1'; r-nmeas+1:r 'Y2'});
set(sys,'InputGroup', struct('U1', 1:c-ncont, 'U2', c-ncont+1:c));
set(sys,'OutputGroup',struct('Y1', 1:r-nmeas, 'Y2', r-nmeas+1:r));

% ----------- End of MKTITO.M --------RYC/MGS 1997

