function [Gm,aug,hhsv] = bstschml(G,mrtype,no,info)
%BSTSCHML Balanced stochastic truncation (BST) model reduction.
%
% [SS_H,AUG,HSV] = BSTSCHML(SS_,MRTYPE,NO,INFO) performs
%    relative error Schur model reduction on a SQUARE, STABLE G(s):=
%    (A,B,C,D). The infinity-norm of the relative error is bounded as
%    (Zhou, 1995):
%               -1                       n
%            |Gm (s)(Gm(s)-G(s))|    <= prod (1+2*si*(sqrt(1+si^2)+si))-1
%                                inf    k+1
%    where si denotes the i-th Hankel singular value of the all-pass
%    "phase matrix" of G(s).
%    The algorithm is based on the Balanced Stochastic Truncation (BST)
%    theory with Relative Error Method (REM).
%    Based on the "MRTYPE" selected, you have the following options:
%     1). MRTYPE = 1  --- no: size "k" of the reduced order model.
%     2). MRTYPE = 2  --- find k-th order reduced model that
%                         tolerance (db) <= "no".
%     3). MRTYPE = 3  --- display all the Hankel SV of phase matrix and
%                   prompt for "k" (in this case, no need to specify "no").
%    Input variable: "info" = 'left '(default is also 'left ').
%    Output variable "aug": aug(1,1) = no. of state removed
%                           aug(1,2) = relative error bound
%    Note that if D is not full rank, an error will result.

% R. Y. Chiang & M. G. Safonov 2/30/88
% Copyright 1988-2004 The MathWorks, Inc. 
%       $Revision: 1.1.6.1 $
% All Rights Reserved.
% ------------------------------------------------------------------------

nag1 = nargin;
%[emsg,nag1,xsflag,Ts,A,B,C,D,mrtype,no,info]=mkargs5x('ss',varargin); error(emsg);

[a,b,c,d,Ts] = ssdata(G);

if Ts, error('LTI inputs must be continuous time (Ts=0)'), end
nag1 = nargin;  % NARGIN may have been changed.

if nag1 <= 3
   info = 'left ';
end
if mrtype == 3
   no = NaN;
end
[Gm,aug,hhsv] = bstschmr(G,mrtype,no,info);
%
% ------- End of BSTSCHML.M --- RYC/MGS 9/13/87 %
