function [Gm,totbnd,hhsv] = ohklmr(G,mrtype,in)
%OHKLMR Optimal Hankel norm approximation (unstable plant).
%
% [SS_M,TOTBND,HSV] = OHKLMR(SS_,MRTYPE,IN) produces an optimal
%    Hankel model reduction of an given unstable plant G(s) such that
%    the inf-norm of the error (Ghed(s) - G(s)) <= sum of the k+1 to n
%    Hankel singular value of G(s) times 2. NO BALANCING IS REQUIRED.
%
%   mrtype = 1, in = order "k" of the reduced model.
%
%   mrtype = 2, find a reduced order model such that the total error is less
%             than "in".
%
%   mrtype = 3, display Hankel singular values, prompt for order "k" (in this
%             case, no need to specify "in").
%
%   TOTBND = Error bound, HSV = Hankel singular values.

% R. Y. Chiang & M. G. Safonov 4/12/87
% Copyright 1988-2004 The MathWorks, Inc.
%       $Revision: 1.1.6.1 $
% All Rights Reserved.
% ----------------------------------------------------------------------
%
%disp('  ')
%disp('        - - Working on Optimal Descriptor Hankel Model Reduction - -');

nag1=nargin;
%[emsg,nag1,xsflag,Ts,a,b,c,d,mrtype,in]=mkargs5x('ss',varargin); error(emsg);

[a,b,c,d,Ts] = ssdata(G);

if Ts, error('LTI inputs must be continuous time (Ts=0)'), end

if mrtype == 3
   in = 0;
end
[ra,ca] = size(a);
[ee,dd] = eig(a);
%
% ------ Find the no. of stable roots :
%
indstab = find(real(dd) < 0);
m = length(indstab);
%
% ------ If completely stable :
%
if m == ra
   [Glf,Grf,augl] = ohkapp(G,mrtype,in);
   Gm = Glf;
   augr = [0 0 0];
   totbnd = augl(1,3);
   hhsv = augl(4:4+ra-1)';
end
%
% ------ If completely unstable :
%
if m == 0
   [Glf,Grf,augr] = ohkapp(-G,mrtype,in);
   Gm = -Glf;
   augl = [0 0 0];
   totbnd = augr(1,3);
   hhsv = augr(4:4+ra-1)';
end
%
% ------ If having both stable & unstable parts :
%
if m > 0 & m < ra
  [Gl,Gr,msat] = stabproj(G);
  hsvl = hksv(Gl);
  hsvr = hksv(-Gr);
  hsv = [hsvl;hsvr];
  [hhsv,index] = sort(hhsv);
  hhsv = hhsv(ra:-1:1,:);
  index = index(ra:-1:1,:);
  if mrtype == 1
      nor = 0;
      for i = 1:in
          if index(i) > msat
             nor = nor + 1;
          end
      end
     nol = in-nor;
   end
   if mrtype == 2
      tol = in;
      tails = 0;
      for i = ra:-1:1
          tails = tails + hhsv(i);
          if 2*tails > tol
             no = i;
             break
          end
      end
      nor = 0;
      for i = 1:in
          if index(i) > msat
             nor = nor + 1;
          end
      end
      nol = in-nor;
      mrtype = 1;
   end
   if mrtype == 3
      hankelsv(ss(a,b,c,d));
      nol = in; nor = in;
   end
   if mrtype == 3
       disp('For the Stable Part:')
   end
   [Glf,Gllf,augl] = ohkapp(Gl,mrtype,nol);
   dlf = dl;
   if mrtype == 3
       disp('For the Unstable Part:')
   end
   [Grf,Grrf,augr] = ohkapp(-Gr,mrtype,nor);
   Grf = -Grf;
   totbnd = augl(1,3)+augr(1,3);
   hhsv = [augl(1,4:4+msat-1)';augr(1,4:4+(ra-msat)-1)'];
   Gm = Glf+Grf;
end
%
nn = augl(1,2) + augr(1,2);
if mrtype == 3
   disp(' ')
   disp(['               ' int2str(nn), '    states removed !!'])
end
%
% ------- End of OHKLMR.M --- RYC/MGS 4/12/87 %