%  SQRTM     Matrix square root.
%     X = SQRTM(A) is the principal square root of the matrix A, i.e. X*X = A.
%            
%     X is the unique square root for which every eigenvalue has nonnegative
%     real part.  If A has any real, negative eigenvalues then a complex
%     result is produced.  If A is singular then A may not have a
%     square root.  A warning is printed if exact singularity is detected.
%            
%     With two output arguments, [X, RESNORM] = SQRTM(A) does not print any
%     warning, and returns the residual, norm(A-X^2,'fro')/norm(A,'fro').
%  
%     With three output arguments, [X, ALPHA, CONDEST] = SQRTM(A) returns a
%     stability factor ALPHA and an estimate CONDEST of the matrix square root
%     condition number of X.  The residual NORM(A-X^2,'fro')/NORM(A,'fro') is
%     bounded approximately by N*ALPHA*EPS and the Frobenius norm relative
%     error in X is bounded approximately by N*ALPHA*CONDEST*EPS, where
%     N = MAX(SIZE(A)).
%  
%     See also EXPM, LOGM, FUNM.

% Copyright 2003-2004 The MathWorks, Inc.

function [x,a,c] = svd(m,flg);

szm = size(m);
ts = pvget(m,'Ts');
x = [];
a = [];
c = [];

if nargout == 3 
   x = zeros([szm(1) szm(1) length(m.Frequency) szm(3:end)]);
   a = zeros([1 1 length(m.Frequency) szm(3:end)]);
   c = zeros([1 1 length(m.Frequency) szm(3:end)]);
	for k = 1:prod(szm(3:end))
       for i = 1:length(m.Frequency)
          [x(:,:,i,k),a(1,1,i,k),c(1,1,i,k)] = sqrtm(m.ResponseData(:,:,i,k));
       end
	end
   x = frd(x,m.Frequency,ts);
   a = frd(a,m.Frequency,ts);
   c = frd(c,m.Frequency,ts);
elseif nargout == 2 
   x = zeros([szm(1) szm(1) length(m.Frequency) szm(3:end)]);
   a = zeros([1 1 length(m.Frequency) szm(3:end)]);
	for k = 1:prod(szm(3:end))
       for i = 1:length(m.Frequency)
          [x(:,:,i,k),a(1,1,i,k)] = sqrtm(m.ResponseData(:,:,i,k));
       end
	end
   x = frd(x,m.Frequency,ts);
   a = frd(a,m.Frequency,ts);
else 
   x = zeros([szm(1) szm(1) length(m.Frequency) szm(3:end)]);
	for k = 1:prod(szm(3:end))
       for i = 1:length(m.Frequency)
          x(:,:,i,k) = sqrtm(m.ResponseData(:,:,i,k));
       end
	end
   x = frd(x,m.Frequency,ts);
end
