function out = binop(A,B,op,n1,n2);
% Binary operation for NDLFT objects

% Copyright 2003-2006 The MathWorks, Inc.
% STATUS UNCLEAR.  must address the different
% levels of autoreduction that akes place.
A = ndlft(A); 
B = ndlft(B);
ioA = size(A);
ioB = size(B);
flg2d = length(ioA)==2 & length(ioB)==2;
switch op
case 'mldivide'
   % Relies on fact that every atom can be set to 0.  True
   % for Uncertain atoms
   A = setlitr(A,(1:length(A.DepData))',0);
case 'mrdivide'
   B = setlitr(B,(1:length(B.DepData))',0);
end

% Compute the new block structure, combining multiple
% blocks with the same name.
%
[uszA,uszB,rmsk,cmsk,newDepData,newCopies,isort,jsort] = lftmask(A,B);

% Any blocks to be combined must be in the same "literal"
% state, i.e. no normalizer, or the same normalizer.
%
tnblk = max(jsort);
blkidA = zeros(sum(A.Literal),1);
nblkA = length(A.Literal);
blkidB = zeros(sum(B.Literal),1);
cntA = 0;
cntB = 0;
for idx=1:tnblk
   loc = find(jsort==idx);
   if length(loc)==2
      loc(2) = loc(2) - nblkA;
      if A.Literal(loc(1))==1 & B.Literal(loc(2))==0
         cntA = cntA + 1;
         blkidA(cntA) = loc(1);
      elseif A.Literal(loc(1))==0 & B.Literal(loc(2))==1
         cntB = cntB + 1;
         blkidB(cntB) = loc(2);
      end
   end
end

if cntA>0
    % XXX relies on that every atom can be set to 0
    A = setlitr(A,blkidA(1:cntA),0);
end
if cntB>0
    B = setlitr(B,blkidB(1:cntB),0);
end

% Any blocks merged now have the same "literal" state.
%
newLiteral = [A.Literal; B.Literal];
newLiteral = newLiteral(isort);

[A11,A12,A21,A22] = LOCALpartn(A.MatrixData,uszA(2),uszA(1));
[B11,B12,B21,B22] = LOCALpartn(B.MatrixData,uszB(2),uszB(1));

%---------------------------------------------------------

% Binary OP here
% Builds M = [M11,M12;M21,M22]


switch op
case 'plus'
   switch flg2d
   case 1
      M = [blkdiag(A11,B11) [A12;B12]; A21 B21 A22+B22];
   case 0
      m11 = ndbdiagr(A11,B11);
      m12 = ndvertr(A12,B12);
      m21 = ndhorzr(A21,B21);
      m22 = ndplusr(A22,B22);
      M = ndvertr(ndhorzr(m11,m12),ndhorzr(m21,m22));
   end
case 'minus'
   switch flg2d
   case 1
      M = [blkdiag(A11,B11) [A12;B12]; A21 -B21 A22-B22];
   case 0
      m11 = ndbdiagr(A11,B11);
      m12 = ndvertr(A12,B12);
      m21 = ndhorzr(A21,-B21);
      m22 = ndminusr(A22,B22);
      M = ndvertr(ndhorzr(m11,m12),ndhorzr(m21,m22));
   end
case 'horzcat'
   switch flg2d
   case 1
      M = [blkdiag(A11,B11) blkdiag(A12,B12);A21 B21 A22 B22];
   case 0
      m11 = ndbdiagr(A11,B11);
      m12 = ndbdiagr(A12,B12);
      m21 = ndhorzr(A21,B21);
      m22 = ndhorzr(A22,B22);
      M = ndvertr(ndhorzr(m11,m12),ndhorzr(m21,m22));
   end
case 'blkdiag'
   switch flg2d
   case 1
      M = [blkdiag(A11,B11) blkdiag(A12,B12);blkdiag(A21,B21) blkdiag(A22,B22)];
   case 0
      m11 = ndbdiagr(A11,B11);
      m12 = ndbdiagr(A12,B12);
      m21 = ndbdiagr(A21,B21);
      m22 = ndbdiagr(A22,B22);
      M = ndvertr(ndhorzr(m11,m12),ndhorzr(m21,m22));
   end
case 'vertcat'
   switch flg2d
   case 1
      M = [blkdiag(A11,B11) [A12;B12];blkdiag(A21,B21) [A22;B22]];
   case 0
      m11 = ndbdiagr(A11,B11);
      m12 = ndvertr(A12,B12);
      m21 = ndbdiagr(A21,B21);
      m22 = ndvertr(A22,B22);
      M = ndvertr(ndhorzr(m11,m12),ndhorzr(m21,m22));
   end
case 'mtimes'
   switch flg2d
   case 1
      M = [A11 A12*B21 A12*B22;...
       zeros(uszB(2),uszA(1)) B11 B12;...
       A21 A22*B21 A22*B22];
   case 0
      m11 = ndvertr(ndhorzr(A11,ndtimesr(A12,B21)),ndhorzr(zeros(uszB(2),uszA(1)),B11));
      m12 = ndvertr(ndtimesr(A12,B22),B12);
      m21 = ndhorzr(A21,ndtimesr(A22,B21));
      m22 = ndtimesr(A22,B22);
      M = ndvertr(ndhorzr(m11,m12),ndhorzr(m21,m22));
   end
case 'starp'
   ca1 = uszA(1);
   ra1 = uszA(2);
   cb1 = uszB(1);
   rb1 = uszB(2);
   ca2 = ioA(2);
   ra2 = ioA(1);
   cb2 = ioB(2);
   rb2 = ioB(1);
   switch flg2d
   case 1
      top = [A11 A12;A21 A22];
      [nrt,nct] = size(top); 
      top11 = top(1:nrt-n1,1:nct-n2);
      top12 = top(1:nrt-n1,nct-n2+1:nct);
      top21 = top(nrt-n1+1:nrt,1:nct-n2);
      top22 = top(nrt-n1+1:nrt,nct-n2+1:nct);
      bot = [B22 B21;B12 B11];
      bot11 = bot(1:n2,1:n1);
      bot12 = bot(1:n2,n1+1:cb1+cb2);
      bot21 = bot(n2+1:rb1+rb2,1:n1);
      bot22 = bot(n2+1:rb1+rb2,n1+1:cb1+cb2);
      %now do the LFT
      tb = top22*bot11;
      [nrtb nctb] = size(tb);
      imtb = eye(nrtb) - tb;
      bt = bot11*top22;
      [nrbt,ncbt] = size(bt);
      imbt = eye(nrbt) - bt;
      errmsg1 = ['Unable to solve ill-conditioned linear relationship.'];
      errmsg2 = sprintf('\n');
      errmsg3 = ['Check that expression is well-defined at NominalValue.'];
      errmsg = [errmsg1 errmsg2 errmsg3];
      if rcond(imbt)<100*eps
         error(errmsg');
      end
      if rcond(imtb)<100*eps
         error(errmsg');
      end
      x = imbt\bot12;
      y = imtb\top21;
      checkX = imbt*x-bot12;
      checkY = imtb*y-top21;
      if any(any(isnan(checkX))) || norm(checkX)>1e-6*norm(bot12)
         error(errmsg');
      end
      if any(any(isnan(checkY))) || norm(checkY)>1e-6*norm(top21)
         error(errmsg');
      end
      upper = [ (top11+top12*bot11*y) top12*x ];
      lower = [ bot21*y (bot22+bot21*top22*x) ];
      omat = [upper ; lower];
      %order is [lftA I/O lftB], dims ...
      %[ra1 ra2+rb2-n1-n2 rb1]x[ca1 ca2+cb2-n1-n2 cb1]
      %switch to [lftA lftB I/O]
      r1 = ra1; r2 = ra2+rb2-n1-n2; r3 = rb1;
      c1 = ca1; c2 = ca2+cb2-n1-n2; c3 = cb1;
      M = omat([1:r1 r1+r2+1:r1+r2+r3 r1+1:r1+r2], ...
      [1:c1 c1+c2+1:c1+c2+c3 c1+1:c1+c2]);   
   case 0
      top = ndvertr(ndhorzr(A11,A12),ndhorzr(A21,A22));
      sztop = size(top);nrt=sztop(1);nct=sztop(2);
      top11 = reshape(top(1:nrt-n1,1:nct-n2,:),[nrt-n1 nct-n2 sztop(3:end)]);
      top12 = reshape(top(1:nrt-n1,nct-n2+1:nct,:),[nrt-n1 n2 sztop(3:end)]);
      top21 = reshape(top(nrt-n1+1:nrt,1:nct-n2,:),[n1 nct-n2 sztop(3:end)]);
      top22 = reshape(top(nrt-n1+1:nrt,nct-n2+1:nct,:),[n1 n2 sztop(3:end)]);
      bot = ndvertr(ndhorzr(B22,B21),ndhorzr(B12,B11));
      szbot = size(bot);nrb=szbot(1);ncb=szbot(2);
      bot11 = reshape(bot(1:n2,1:n1,:),[n2 n1 szbot(3:end)]);
      bot12 = reshape(bot(1:n2,n1+1:cb1+cb2,:),[n2 cb1+cb2-n1 szbot(3:end)]);
      bot21 = reshape(bot(n2+1:rb1+rb2,1:n1,:),[rb1+rb2-n2 n1 szbot(3:end)]);
      bot22 = reshape(bot(n2+1:rb1+rb2,n1+1:cb1+cb2,:),[rb1+rb2-n2 cb1+cb2-n1 szbot(3:end)]);
      %now do the LFT
      tb = ndtimesr(top22,bot11);
      sztb = size(tb);nrtb=sztb(1);nctb=sztb(2);
      imtb = ndminusr(eye(nrtb),tb);
      bt = ndtimesr(bot11,top22);
      szbt = size(bt);nrbt=szbt(1);ncbt=szbt(2);
      imbt = ndminusr(eye(nrbt),bt);
      x = ndmldivr(imbt,bot12);
      y = ndmldivr(imtb,top21);
      upper1 = ndplusr(top11,ndtimesr(ndtimesr(top12,bot11),y));
      upper2 = ndtimesr(top12,x);
      lower1 = ndtimesr(bot21,y);
      lower2 = ndplusr(bot22,ndtimesr(ndtimesr(bot21,top22),x));
      omat = ndvertr(ndhorzr(upper1,upper2),ndhorzr(lower1,lower2));
      szomat = size(omat);
      %order is [lftA I/O lftB], dims ...
      %[ra1 ra2+rb2-n1-n2 rb1]x[ca1 ca2+cb2-n1-n2 cb1]
      %switch to [lftA lftB I/O]
      r1 = ra1; r2 = ra2+rb2-n1-n2; r3 = rb1;
      c1 = ca1; c2 = ca2+cb2-n1-n2; c3 = cb1;
      M = omat([1:r1 r1+r2+1:r1+r2+r3 r1+1:r1+r2], ...
       [1:c1 c1+c2+1:c1+c2+c3 c1+1:c1+c2],:);
      szm = size(M);
      M = reshape(M,[szm(1) szm(2) szomat(3:end)]);
   end   
case 'mldivide'
   switch flg2d
   case 1
      tmp1 = [A11 zeros(uszA(2),uszB(1)+ioB(2));...
         zeros(uszB(2),uszA(1)) B11 B12;...
         zeros(ioA(2),uszA(1)+uszB(1)+ioB(2))];
      tmp2 = [A12; zeros(uszB(2),ioA(2)) ; eye(ioA(2))];
      tmp3 = A22\[-A21 B21 B22];
      M = tmp1 + tmp2*tmp3;
   case 0
      tmp1top = ndhorzr(A11,zeros(uszA(2),uszB(1)+ioB(2)));
      tmp1mid = ndhorzr(zeros(uszB(2),uszA(1)),ndhorzr(B11,B12));
      tmp1bot = zeros(ioA(2),uszA(1)+uszB(1)+ioB(2));
      tmp1 = ndvertr(ndvertr(tmp1top,tmp1mid),tmp1bot);
      tmp2 = ndvertr(A12,[zeros(uszB(2),ioA(2)) ; eye(ioA(2))]);
      tmp3 = ndmldivr(A22,ndhorzr(ndhorzr(-A21,B21),B22));
      M = ndplusr(tmp1,ndtimesr(tmp2,tmp3));
   end
case 'mrdivide'
   switch flg2d
   case 1
      tmp1 = [A11 zeros(uszA(2),uszB(1)+ioB(1));...
              zeros(uszB(2),uszA(1)) B11 zeros(uszB(2),ioB(1));...
              A21 zeros(ioA(1),uszB(1)+ioB(1))];
      errmsg1 = ['Unable to solve ill-conditioned linear relationship.'];
      errmsg2 = sprintf('\n');
      errmsg3 = ['Check that expression is well-defined at NominalValue.'];
      errmsg = [errmsg1 errmsg2 errmsg3];
      if rcond(B22)<100*eps
         error(errmsg');
      end
      tmp2 = [A12 ; B12 ; A22]/B22;
      checkQ = tmp2*B22-[A12 ; B12 ; A22];
      if any(any(isnan(checkQ))) || norm(checkQ)>1e-6*norm(B22)
         error(errmsg');
      end
      tmp3 = [zeros(ioB(1),uszA(1)) -B21 eye(ioB(1))];
      M = tmp1 + tmp2*tmp3;
   case 0
      tmp1top = ndhorzr(A11,zeros(uszA(2),uszB(1)+ioB(1)));
      tmp1mid = ndhorzr(ndhorzr(zeros(uszB(2),uszA(1)),B11),zeros(uszB(2),ioB(1)));
      tmp1bot = ndhorzr(A21,zeros(ioA(1),uszB(1)+ioB(1)));
      tmp1 = ndvertr(ndvertr(tmp1top,tmp1mid),tmp1bot);
      tmp2 = ndmrdivr(ndvertr(ndvertr(A12,B12),A22),B22);
      tmp3 = ndhorzr(ndhorzr(zeros(ioB(1),uszA(1)),-B21),eye(ioB(1)));
      M = ndplusr(tmp1,ndtimesr(tmp2,tmp3));
   end
otherwise
   error(['The operation  ''' op '''  is not supported.']);
end
%---------------------------------------------------------
% Shuffle LFT channels
M(1:length(rmsk),:,:) = M(rmsk,:,:);
M(:,1:length(cmsk),:) = M(:,cmsk,:);
out = ndlft(M,newDepData,newCopies,newLiteral);

function [a,b,c,d] = LOCALpartn(Q,nr,nc)
% PARTN  matrix partition
%
% [A,B,C,D] = PARTN(MAT,nr,nc) partitions the matrix MAT as
%
%          | A | B | 
%   MAT =  |---|---|  with size(A) = [nr nc]
%          | C | D |
%
% [A,B,C,D] = PARTN(MAT,nr) produces a square A matrix.

% Author(s): G. Wolodkin 2-19-98, A. Packard & G. Balas 5-16-06, 
% Copyright (c) 1986-98 by The MathWorks, Inc.
% $Revision: 1.1.6.3 $  $Date: 2006/11/17 13:59:40 $

if nargin < 3, nc = nr; end

szq = size(Q);
lastr = szq(1);
lastc = szq(2);
%  lastd = repmat({':'},[1 length(szq)-2]);
% % 
%  a = Q(1:nr,1:nc,lastd{:});
%  b = Q(1:nr,nc+1:lastc,lastd{:});
%  c = Q(nr+1:lastr,1:nc,lastd{:});
%  d = Q(nr+1:lastr,nc+1:lastc,lastd{:});

szq3e = szq(3:end);
nc2 = lastc-nc;
nr2 = lastr-nr;
a = reshape(Q(1:nr,1:nc,:),[nr nc szq3e]);
b = reshape(Q(1:nr,nc+1:lastc,:),[nr nc2 szq3e]);
c = reshape(Q(nr+1:lastr,1:nc,:),[nr2 nc szq3e]);
d = reshape(Q(nr+1:lastr,nc+1:lastc,:),[nr2 nc2 szq3e]);
