% function [out,idflg] = blk2ioch(mat,blkid,tb)
%
%  Opens up a LFT channel, moving the exposed
%  inputs/outputs into the top (or bottom) of
%  the I/O block.  Uses name in BLKID, or
%  numerical index.
%
%  Returns a DOUBLE or NDLFT.
%
%  Here, in moving a block to IO, the
%  BLOCK is no longer in the list (not just copies==0)
%  It is removed from the list.

% Copyright 2003-2004 The MathWorks, Inc.

% XXXOK 

function [out,idflg] = blk2ioch(mat,blkid,tb)

[rp,cp,nblk,iosize,blkiosize,szmd] = ndlftinfo(mat);
blknames = pvget(mat,'UNames');
if isstr(blkid) & min(size(blkid))==1                
   idx = strmatch(blkid,blknames,'exact');
elseif isa(blkid,'double') & floor(blkid)==ceil(blkid) & ...
 blkid>0 & blkid<=nblk
   idx = blkid;
else
   idx = [];
end
AtomCell = pvget(mat.DepData,'AtomCell');

idflg = 1;
if isempty(idx)
   idflg = 0;
   out = mat; % Important: if block doesn't exist, just pass back MAT
else
   szblk = size(AtomCell{idx});
   exin = mat.Copies(idx)*szblk(1);

   ior = (rp(nblk+1):szmd(1))';
   ioc = (cp(nblk+1):szmd(2))';
   rows = (1:rp(nblk+1)-1)';
   rows(rp(idx):rp(idx+1)-1) = [];
   cols = (1:cp(nblk+1)-1)';
   cols(cp(idx):cp(idx+1)-1) = [];

   if strcmp(tb,'t')
      ridx = [rows;(rp(idx):rp(idx+1)-1)';ior];
      cidx = [cols;(cp(idx):cp(idx+1)-1)';ioc];
   elseif strcmp(tb,'b')
      ridx = [rows;ior;(rp(idx):rp(idx+1)-1)'];
      cidx = [cols;ioc;(cp(idx):cp(idx+1)-1)'];
   else
      error(['Top/Bottom selection should be ''t'' or ''b''']);
   end
   outm = zeros([length(ridx) length(cidx) szmd(3:end)]);
   outm(:) = mat.MatrixData(ridx,cidx,:);
   outAtomCell = AtomCell;
   outLiteral = mat.Literal;
   outCopies = mat.Copies;
   % need no-block case to be 0x1, not 1x0 which is the Matlab default
   if nblk>1
      outAtomCell(idx) = [];
      outLiteral(idx) = [];
      outCopies(idx) = [];
      outDepData = atomlist(outAtomCell,zeros(nblk-1,1));
   else
      outLiteral = zeros(0,1);
      outCopies = zeros(0,1);
      outDepData = atomlist;
   end

   if nblk==1 & length(szmd)==2
      out = outm;
   else
      out = ndlft(outm,outDepData,outCopies,outLiteral);
   end
end
