function b = imp2exp(a,yidx,uidx)
% IMP2EXP Converts an implicit LFT relationship to Explicit I/O
%        B = IMP2EXP(A,YIDX,UIDX) transforms the LFT constraint
%        A*[Y;U]=0 into Y=B*U.  The index vectors refer to the
%        columns (inputs) of A as referenced by the explicit
%        relationship for B.  So, if Y=B*U, then ...

% final call to NDLFT/GENREDU means that NDLFT/IMP2EXP
% can return DOUBLE, ATOM or NDLFT.

% Copyright 2003-2004 The MathWorks, Inc.

% STATUS UNCLEAR totally, might be fine

if ~isa(a,'ndlft')
	a = ndlft(a);
end

% b = parm([2 5],3,'b');
% m = ic;
% u = icsignal(1);
% y = icsignal(1);
% m.eq{1} = b*y==u;
% m.Input = u;
% m.Output = y;
% m.System (gives algebraic error, as we ultimately are trying to get 1/b)
% XXX: do we need to do this in most cases?  See above for an example that does
a = setlitr(a,find(a.Literal==1),0);
[rp,cp,nblk,iosize,blkiosize,szmd] = ndlftinfo(a);
blkr = (rp(1):rp(end)-1)';
blkc = (cp(1):cp(end)-1)';
ior = (rp(end):szmd(1))';
ioc = (cp(end):szmd(2))';
%[blkr,blkc,xr,xc,ior,ioc,rp,cp,idm] = glftidx(a);
r1 = length(ior);
r2 = length(blkr);
c1 = length(yidx);
c2 = length(uidx);
c3 = length(blkc);
if r1==c1
   amat = pvget(a,'MatrixData');
	if length(szmd)==2
      m11m13 = amat([blkr],[blkc;ioc(uidx)]);
      m21m23 = amat([ior],[blkc;ioc(uidx)]);
      m12 = amat([blkr],[ioc(yidx)]);
      m22 = amat([ior],[ioc(yidx)]);
      if rcond(m22)<1e-12
      	error('Equations are algebraically incompatible');
         return
      end
      tmp = [m12;eye(c1)]/m22;
      resid = tmp*m22 - [m12;eye(c1)];
      relfac = 1e-8;
      if any(any(isnan(resid))) | any(any(isinf(resid)))
      	error('Equations are algebraically incompatible');
         return
      elseif norm(resid,'fro')>relfac*norm([m12;eye(c1)],'fro')
      	error('Equations are numerically incompatible');
         return
      else
         aa = [m11m13;zeros(c1,c3+c2)] - tmp*m21m23;
         ddata = pvget(a,'DepData');
         cop = pvget(a,'Copies');
         lit = pvget(a,'Literal');
         b = ndlft(aa,ddata,cop,lit);
      end
	else
   	npts = prod(szmd(3:end));
   	outdim = r2+c1;
   	indim = c3+c2;
   	%m11m13 = amat([blkr],[blkc;ioc(uidx)],:);
   	%m21m23 = amat([ior],[blkc;ioc(uidx)],:);
   	m12 = amat([blkr],[ioc(yidx)],:);
   	%m22 = amat([ior],[ioc(yidx)],:);
   	aa = zeros([outdim indim szmd(3:end)]);
   	relfac = 1.0e-8;
   	for i=1:npts
   		%tmp = [m12(:,:,i);eye(c1)]/m22(:,:,i);
   		%resid = tmp*m22(:,:,i)-[m12(:,:,i);eye(c1)];
   		tmp = [amat([blkr],[ioc(yidx)],i);eye(c1)]/amat([ior],[ioc(yidx)],i);
   		resid = tmp*amat([ior],[ioc(yidx)],i)-[amat([blkr],[ioc(yidx)],i);eye(c1)];
   		if any(any(isnan(resid))) | any(any(isinf(resid)))
   			errstr('I/O choice yields algebraically incompatible equations');
   			error(errstr);
   		elseif max(max(abs(resid)))>relfac*max(max(abs([m12(:,:,i);eye(c1)])))
   			errstr('I/O choice yields numerically incompatible equations');
   			error(errstr);
   		else
            aa(:,:,i) = [amat([blkr],[blkc;ioc(uidx)],i);zeros(c1,c3+c2)] - ...
               tmp*amat([ior],[blkc;ioc(uidx)],i);
   		end
   	end
      ddata = pvget(a,'DepData');
      cop = pvget(a,'Copies');
      lit = pvget(a,'Literal');
      b = ndlft(aa,ddata,cop,lit);
	end
   % can return DOUBLE, ATOM or NDLFT.
   b = simplify(b,'nodepromote','noremove');
else
	error('Invalid IMP2EXP dimensions: OUTPUT should match LENGTH(YIDX)');
   return
end

