function [b,nelim] = ndlftredv(m,blkid,level)

% Copyright 2004-2005 The MathWorks, Inc.

if nargin<3
   error('Invalid # of input arguments: need (MATIN,BLKID,LEVEL)');
end

if level==0
   b = m;
   nelim = 0;
   return
end

[rp,cp,nblk,iosize,blkiosize,szmd] = ndlftinfo(m);
ior = blkiosize(2) + (1:iosize(1));
ioc = blkiosize(1) + (1:iosize(2));
npts = prod(szmd(3:end));
idx = gblkidx(m,blkid);
nelim = 0;
if ~isempty(idx) && m.Copies(idx)>0
   newCopies = m.Copies;
   newLiteral = m.Literal;
   bcopies = m.Copies(idx);
   atomcell = pvget(m.DepData,'AtomCell');
   sinfo = pvget(m.DepData,'Info');
   
   blksize = size(atomcell{idx,1});
   traits = pvget(atomcell{idx,1},'Traits');
   %TheName = pvget(atomcell{idx,1},'Name');
   brow = blksize(1);
   bcol = blksize(2);
   nrows = 1:szmd(1);    % everything
   nrows(rp(idx):rp(idx+1)-1) = [];
   ncols = 1:szmd(2);
   ncols(cp(idx):cp(idx+1)-1) = [];
   Bmat = m.MatrixData(rp(idx):rp(idx+1)-1,ncols,:); % immediately 3-d at most
   Cmat = m.MatrixData(nrows,cp(idx):cp(idx+1)-1,:);
   Amat = m.MatrixData(rp(idx):rp(idx+1)-1,cp(idx):cp(idx+1)-1,:);
   r1 = 1:rp(idx)-1;
   c1 = 1:cp(idx)-1;
   r2 = [rp(idx+1):rp(nblk+1)-1 ior];
   c2 = [cp(idx+1):cp(nblk+1)-1 ioc];
   lr1 = length(r1);
   lc1 = length(c1);
   lr2 = length(r2);
   lc2 = length(c2);
   Dmat11 = m.MatrixData(r1,c1,:);
   Dmat12 = m.MatrixData(r1,c2,:);
   Dmat21 = m.MatrixData(r2,c1,:);
   Dmat22 = m.MatrixData(r2,c2,:);
   Dmat = m.MatrixData([r1 r2],[c1 c2],:);
   % at matrix level, the partitioning is
   %  [D11 C1  D12]
   %  [B1  A   B2 ]
   %  [D21 C2  D22],
   % since the block might be anywhere.
   %ny = length(nrows);
   %nu = length(ncols);
   % delete = 0;
   doreduce = 0;
   %naz = 0; % currently not used, but there is: if norm(Amat(:))==0
   %asid = 0; % currently not used, but there is: isndssid()
   % Loop through until no progress
   go = 1;
   while go==1
      go = 0;
      if level>=1
         [Amat,Bmat,Cmat,keep] = zizoelim(Amat,Bmat,Cmat,bcol,brow,bcopies,traits.zizo);
         nelim = nelim + (bcopies-keep);
         if keep<bcopies && keep>0
            doreduce = 1;
            newCopies(idx) = keep;
            bcopies = keep;
            go = 1;
         elseif keep==0
            b = zerocopies(m,blkid);
            return % only working on a single block, so here we are done.
         end
      end
      if level>=1 && norm(Amat(:))==0
         [Bmat,Cmat,keep] = idrowcol(Bmat,Cmat,bcopies,brow,bcol,traits.linear);
         nelim = nelim + (bcopies-keep);
         if keep<bcopies && keep>0
            doreduce = 1;
            newCopies(idx) = keep;
            bcopies = keep;
            go = 1;
            Amat = Amat(1:bcopies*bcol,1:bcopies*brow,:);
         elseif keep==0
            b = zerocopies(m,blkid);
            return
         end
      end
      % XXX: Misses on this (though scalars work)
      % >> f = ultidyn('f',[4 3]);
      % >> ff = 2*f - f*2
      % but gets
      % >> ff = f - f;

      if level>=2
         if isequal(traits.linear,1)
            if isequal(traits.scalar,1)
               keep = bcopies;
               % XXX, check conditions here
% 13 May 2005: Replace separate dynamic/constant
%                if ~isequal(traits.dynamic,1) & isndssid(Amat)
%                   [Cnew,Bnew,keep] = lrmoda(Cmat,Bmat);
%                elseif isndzero(Amat) & isequal(traits.dynamic,1)
%                   bcons = isndcons(Bmat);
%                   ccons = isndcons(Cmat);
%                   if bcons & ~ccons
%                      [Cnew,Bnew,keep] = lfac3d(ndtimesr(Cmat,Bmat(:,:,1)));
%                   elseif ~bcons & ccons 
%                      [Cnew,Bnew,keep] = rfac3d(ndtimesr(Cmat(:,:,1),Bmat));
%                   elseif bcons & ccons
%                      [Cnew,Bnew,keep] = lrmod(Cmat,Bmat);
%                   end
               if isndssid(Amat)
                  [Cnew,Bnew,keep] = lrmoda(Cmat,Bmat);
% 13 May 2005: End Mod
               end
               nelim = nelim + (bcopies-keep);
               if keep<bcopies && keep>0
                  doreduce = 1;
                  newCopies(idx) = keep;
                  bcopies = keep;
                  Amat = Amat(1:bcopies*bcol,1:bcopies*brow,:);
                  Bmat = Bnew;
                  Cmat = Cnew;
                  go = 1;
               elseif keep==0
                  b = zerocopies(m,blkid);
                  return
               end
            else
               %XXX: linear, but not scalar, Anything to do?
            end
         else
            %XXX:  not linear, anything to do?
         end
      end
   end % while go
   if isequal(traits.linear,1) && isequal(traits.scalar,1) && npts==1
      % APPLY SMINREAL to ss(Amat,Bmat,Cmat,Dmat),
      % using direct call to SMREAL
      [tmpa,tmpb,tmpc] = smreal(Amat,Bmat,Cmat,[]);
      keep = size(tmpa,1);
      if keep<bcopies && keep>0
          doreduce = 1;
          newCopies(idx) = keep;
          bcopies = keep;
          Amat = tmpa;
          Bmat = tmpb;
          Cmat = tmpc;
          % go = 1;
       elseif keep==0
          b = zerocopies(m,blkid);
          return
       end
   end
   
   % at this point: Amat, Bmat, Cmat, Dmat, doreduce, bcopies, newCopies
   %                newLiteral
   % are all "correct" and consistent.  In the reduction involving
   % balanced realizations, care must be used to preserve correctness.      
   % switchedtoZero = 0; % why?  a literal==1 might get switched to 0
   if bcopies>1 && isequal(traits.linear,1) && ...
         isequal(traits.scalar,1) && npts==1 && level>=3
      NX = bcopies;
      % go = 0;
      AmatNoN = [];
      if m.Literal(idx)==1
         % there is only 1 point (see IF statement)
         % could be a signle frequency point, or just no frequency
         % 3 lines below seems to rely on DOUBLE, as TMP would otherwise be screwed
         N = pvget(atomcell{idx},'Normalizer');
         S = N.S;
         AmatNoN = Amat;
         ABCD = [Amat Bmat;Cmat [Dmat11 Dmat12;Dmat21 Dmat22]];
         ABCDN = lft(kron(S,eye(NX)),ABCD,NX,NX);
         % switchedtoZero = 1;
      else
         ABCDN = [Amat Bmat;Cmat Dmat];
      end
      keepvec = repmat(NX+1,[1 4]);
      eAmatN = eig(ABCDN(1:NX,1:NX));
      ABCDred = cell(1,4);
      keeplit = [1 1 0 0];
      errc = ones(1,4);
      SLOPE = 0.0001;
      etol = NX*NX*100*eps;
      % First try both Literal==0 cases
      if max(abs(eAmatN))<1-etol
         [keepvec(4),ABCDred{4},errc(4)] = LOCALreduce(ABCDN,NX,1);
      end
      %if max(real(eAmatN))<-1000*eps
      if all(real(eAmatN)<-SLOPE*abs(imag(eAmatN))-etol) 
         [keepvec(3),ABCDred{3},errc(3)] = LOCALreduce(ABCDN,NX);
      end
      if ~isempty(AmatNoN)  % Literal is 1 for these
         eAmatNoN = eig(AmatNoN);
         if max(abs(eAmatNoN))<1-etol
            [keepvec(2),ABCDred{2},errc(2)] = LOCALreduce(ABCD,NX,1);
         end
         if all(real(eAmatNoN)<-SLOPE*abs(imag(eAmatNoN))-etol) 
            [keepvec(1),ABCDred{1},errc(1)] = LOCALreduce(ABCD,NX);
         end
      end
      rkeep = find(keepvec<NX & errc==0);
      if ~isempty(rkeep)
         usecase = min(rkeep); % prioritize from 1 to 4, as above
         keep = keepvec(usecase);
      else
         keep = NX;
      end
      if keep>0 && keep<bcopies
         % Overwrite the state-space data, update LITERAL flag
         nelim = nelim + (bcopies-keep);
         % bcopies = keep;
         Amat = ABCDred{usecase}(1:keep,1:keep);
         Bmat = ABCDred{usecase}(1:keep,keep+1:end);
         Cmat = ABCDred{usecase}(keep+1:end,1:keep);
         Dmat = ABCDred{usecase}(keep+1:end,keep+1:end);
         doreduce = 1;
         newCopies(idx) = keep;
         newLiteral(idx) = keeplit(usecase);
      elseif keep==0
         b = zerocopies(m,blkid);
         return
      end
   end
   % Note: if balanced reduction not allowed, or did not reduce representation,
   % then do not overwrite state-space data, so do not change LITERAL flag, so
   % that even if DOREDUCE==1, from earlier, everything is still ok.
   if doreduce==1
      M11 = Dmat(1:lr1,1:lc1,:);
      M12 = Dmat(1:lr1,lc1+1:lc1+lc2,:);
      M21 = Dmat(lr1+1:lr1+lr2,1:lc1,:);
      M22 = Dmat(lr1+1:lr1+lr2,lc1+1:lc1+lc2,:);
      rdim = size(M11,1) + size(Bmat,1) + size(M21,1);
      cdim = size(M11,2) + size(Cmat,2) + size(M12,2);
      mmtop = ndhorzr(ndhorzr(M11,Cmat(1:rp(idx)-1,:,:)),M12);
      mmmid = ndhorzr(ndhorzr(Bmat(:,1:cp(idx)-1,:),Amat),Bmat(:,cp(idx):length(ncols),:));
      mmbot = ndhorzr(ndhorzr(M21,Cmat(rp(idx):length(nrows),:,:)),M22);
      mm = reshape(ndvertr(ndvertr(mmtop,mmmid),mmbot),[rdim cdim szmd(3:end)]);
      b = ndlft(mm,atomlist(atomcell,sinfo),newCopies,newLiteral);
   else
      nelim = 0;
      b = m;
   end
else
   nelim = 0;
   b = m;
end

function [keep,ABCDred,errc] = LOCALreduce(ABCD,NX,alpha)

sbmudlhand = sbmudl(lti,'char');
if nargin==2
   [Abal,Bbal,Cbal,Dbal,hsv,errc] = sbmudlhand(...
      ABCD(1:NX,1:NX),ABCD(1:NX,NX+1:end),...
      ABCD(NX+1:end,1:NX),ABCD(NX+1:end,NX+1:end),[]);
else
   BLT = kron([-alpha -sqrt(2*alpha);-sqrt(2*alpha) -1],eye(NX));
   ABCDmod = lft(BLT,ABCD,NX,NX);
   [Abal,Bbal,Cbal,Dbal,hsv,errc] = sbmudlhand(...
      ABCDmod(1:NX,1:NX),ABCDmod(1:NX,NX+1:end),...
      ABCDmod(NX+1:end,1:NX),ABCDmod(NX+1:end,NX+1:end),[]);
end
if ~errc
   if isempty(hsv)
      keep = 0;
      ABCDred = Dbal;
   else
      keep = min([length(find(hsv>(hsv(1)*1e-8))) length(find(hsv>1e-10))]);
      if keep<NX
         ABCDred = [Abal(1:keep,1:keep) Bbal(1:keep,:);Cbal(:,1:keep) Dbal];
         if nargin==3
            BLTI = kron([1 sqrt(2*alpha);sqrt(2*alpha) alpha],eye(keep));
            ABCDred = lft(BLTI,ABCDred,keep,keep);
         end
      else
         ABCDred = ABCD;
      end
   end
else
   keep = NX;
   ABCDred = ABCD;
end


% Notes

% AUTOMATIC REDUCTION WITH REGARD TO DYNAMIC ORDER
% In CSTB, we have
%sys1 = ss(-1,2,3,4)
%sys2 = ss(-2,3,4,5)
%sys = [sys1 sys2]
%h=sys(1,1);
% yields a 2-state system (no automatic reduction), whereas
%tf1 = tf(sys1);
%tf2 = tf(sys2);
%tff = [tf1 tf2];
%h=ss(tff(1,1))
%yields a 1-state system.

