function [Lnew,Rnew,rk] = lrmod(L,R)
% Given L and R (2-d or 3-d), it returns Lnew and Rnew
% such that Lnew*Rnew = L*R, with possibly lower rank
% 3-d are interpreted as simply functions over a single
% variable.  This routine simply factors them pointwise.

% Copyright 2003-2004 The MathWorks, Inc.

    szL = size(L);
    szR = size(R);
    if szL(1)<szL(2) | szR(2)<szR(1)
        % inner dimension is not the smallest
        if szL(1)<szR(2)
            rk = szL(1);
            Lnew = eye(rk);
            Rnew = ndtimesr(L,R);
        else
            rk = szR(2);
            Rnew = eye(rk);
            Lnew = ndtimesr(L,R);
        end
    end
            
    if isa(L,'double') & isa(R,'double') & szL(2)==szR(1)
        if ndims(L)==2
            [Lfac,cL,rkL] = lzfac(L);
            if ndims(R)==2
                [cR,Rfac,rkR] = lzfac(R);
                if rkL<rkR
                    Lnew = Lfac;
                    Rnew = cL*cR*Rfac;
                    rk = rkL;
                else
                    Lnew = Lfac*cL*cR;
                    Rnew = Rfac;
                    rk = rkR;
                end
            elseif ndims(R)==3
                [cR,Rfac,rkR] = rfac3d(R);
                if rkL<rkR
                    Lnew = Lfac;
                    Rnew = ndtimesr(cL*cR,Rfac);
                    rk = rkL;
                else
                    Lnew = Lfac*cL*cR;
                    Rnew = Rfac;
                    rk = rkR;
                end
            else
                error('Invalid R dimension')
                return
            end
        elseif ndims(L)==3
            [Lfac,cL,rkL] = lfac3d(L);
            if ndims(R)==2
                [cR,Rfac,rkR] = lzfac(R);
                if rkL<rkR
                    Lnew = Lfac;
                    Rnew = cL*cR,Rfac;
                    rk = rkL;
                else
                    Rnew = Rfac;
                    Lnew = ndtimesr(Lfac,cL*cR);
                    rk = rkR;
                end
            elseif ndims(R)==3 & szL(3)==szR(3)
                [cR,Rfac,rkR] = rfac3d(R);
                if rkL<rkR
                    Lnew = Lfac;
                    Rnew = ndtimesr(cL*cR,Rfac);
                    rk = rkL;
                else
                    Rnew = Rfac;
                    Lnew = ndtimesr(Lfac,cL*cR);
                    rk = rkR;
                end
            else
                error('Invalid R dimension')
                return
            end
        end
    end
    if rk<min([szL(1) szL(2)]) & rk<min([szR(1) szR(2)]) & rk>0
        [Lnew,Rnew,rk] = lrmod(Lnew,Rnew);
    end
            
        
