function [Lnew,Rnew,rk] = lrmoda(L,R)

% Copyright 2003-2005 The MathWorks, Inc.

szl = size(L);
szr = size(R);
ndl = length(szl);
ndr = length(szr);
% XXX: exact properties of this is unclear.  Appears to
% factor L*R into Lnew*Rnew.  Either and/or both can
% be 3-d, in general the results Lnew and Rnew are both 3-d
% even if just one (not both) of the arguments (L/R) is.
% This functionality is consistent with its use in NDLFTREDV.

maxdim = max([szl(1) szr(2)]);

if szl(2)==szr(1)
   amaxrk = szl(2);
   maxdim = min([szl(1) szl(2) szr(2)]);
   if ndl==2
      Lnew = zeros([szl(1) maxdim szr(3:end)]);
      Rnew = zeros([maxdim szr(2) szr(3:end)]);
      npts = prod(szr(3:end));
      maxrk = 0;
      for i=1:npts
         [u,s,v] = svd(L*R(:,:,i));
         Lnew(:,:,i) = u*sqrt(s(1:szl(1),1:maxdim));
         Rnew(:,:,i) = sqrt(s(1:maxdim,1:szr(2)))*v';
         if min(size(s))>1
            ds = diag(s);
         else
            ds = s;
         end
         tmprk = LOCALrank(maxdim,ds,L,R(:,:,i));
         if tmprk>maxrk
            maxrk = tmprk;
         end
      end
   elseif ndr==2
      Lnew = zeros([szl(1) maxdim szl(3:end)]);
      Rnew = zeros([maxdim szr(2) szl(3:end)]);
      npts = prod(szl(3:end));
      maxrk = 0;
      for i=1:npts
         [u,s,v] = svd(L(:,:,i)*R);
         Lnew(:,:,i) = u*sqrt(s(1:szl(1),1:maxdim));
         Rnew(:,:,i) = sqrt(s(1:maxdim,1:szr(2)))*v';
         if min(size(s))>1
            ds = diag(s);
         else
            ds = s;
         end
         tmprk = LOCALrank(maxdim,ds,L(:,:,i),R);
         if tmprk>maxrk
            maxrk = tmprk;
         end
      end
   elseif isequal(szl(3:end),szr(3:end))
      Lnew = zeros([szl(1) maxdim szl(3:end)]);
      Rnew = zeros([maxdim szr(2) szr(3:end)]);
      npts = prod(szr(3:end));
      maxrk = 0;
      for i=1:npts
         [u,s,v] = svd(L(:,:,i)*R(:,:,i));
         Lnew(:,:,i) = u*sqrt(s(1:szl(1),1:maxdim));
         Rnew(:,:,i) = sqrt(s(1:maxdim,1:szr(2)))*v';
         if min(size(s))>1
            ds = diag(s);
         else
            ds = s;
         end
         % size(ds)
         tmprk = LOCALrank(maxdim,ds,L(:,:,i),R(:,:,i));
         if tmprk>maxrk
            maxrk = tmprk;
         end
      end
   end
   maxrk = min([maxrk amaxrk]);
   Lnew = Lnew(:,1:maxrk,:);
   Rnew = Rnew(1:maxrk,:,:);
   rk = maxrk;
else
   error('Invalid inner dimensions on L/R');
end

function rk = LOCALrank(maxdim,ds,L,R)

ztol = maxdim*eps*norm(L)*norm(R);
rk = sum(ds>ztol);

