function [Lfac,Rfac,rk] = lzfac(m)
% Main computational routine for LFAC3D and RFAC3D
% Factors a 2-d matrix M = Lfac*Rfac.  Inner dimension
% is equal to rank of M.  Specifically for the case when
% M is very large, but contains lots of identically zero
% rows and columns.

% Copyright 2003-2005 The MathWorks, Inc.

    szm = size(m);
    if ndims(m)==2
        if all(szm>[1 1])
            am= abs(m);
            summ = sum(am,1);
            sumt = sum(am,2);
            ridx = find(sumt>0);
            cidx = find(summ>0);
            if length(ridx)>=length(cidx)
                [q,r] = qr(m(ridx,cidx),0);
                [u,s,v] = svd(r); %s is a matrix
                ds = diag(s);
                tol = max(size(r)')*max(ds)*eps; %used to be 100*
                rk = sum(ds > tol);
                Lfac = zeros(szm(1),rk);
                Rfac = zeros(rk,szm(2));
                Lfac(ridx,:) = q*u(:,1:rk)*sqrt(s(1:rk,1:rk));
                Rfac(:,cidx) = sqrt(s(1:rk,1:rk))*v(:,1:rk)';
            else
                [q,r] = qr(m(ridx,cidx)',0);
                [u,s,v] = svd(r);
                ds = diag(s);
                tol = max(size(r)')*max(ds)*eps;%used to be 100*
                rk = sum(ds > tol);
                Lfac = zeros(szm(1),rk);
                Rfac = zeros(rk,szm(2));
                Lfac(ridx,:) = v(:,1:rk)*sqrt(s(1:rk,1:rk));
                Rfac(:,cidx) = sqrt(s(1:rk,1:rk))*u(:,1:rk)'*q';
            end
        elseif szm(1)==1
            if norm(m)==0
                Lfac = zeros(1,0);
                Rfac = zeros(0,szm(2));
                rk = 0;
            else
                Lfac = 1;
                Rfac = m;
                rk = 1;
            end
        else
            if norm(m)==0
                Lfac = zeros(szm(1),0);
                Rfac = zeros(0,1);
                rk = 0;
            else
                Lfac = m;
                Rfac = 1;
                rk = 1;
            end
        end 
    else
        error('LZFAC only works on 2-d matrices')
        return
    end
        

