function [out,pflag,errstring] = pvget(m,property,varargin)

% Copyright 2003-2006 The MathWorks, Inc.

pflag = 1;
errstring = '';
% STATUS UNFINISHED

switch property
case 'MatrixData'
   out = m.MatrixData;
case 'Copies'
   out = m.Copies;
case 'DepData'
   out = m.DepData;
case 'Literal'
   out = m.Literal;
case 'AtomCell'
   out = pvget(m.DepData,'AtomCell');
case 'PropNames'
   out.GPropNames = [{'NominalValue';'UNames';'DepData'}];
   out.GPropNames = [{'NominalValue';'DepData'}];
   out.SPropNames = [{'DepData'}];
   out.SPropDescription = [{'LFT Dependency object'}];
case 'NominalValue'
   % XXXPH, XXXDELAY
   % Only do this for Uncertain Atoms.  But, some uncertain
   % atoms have NominalValues that are PlaceHolderATOMs.
   % All Uncertain atoms need to have a NominalClass which tells what
   % class NominalValue is.  For instance, NominalClass(udelay)
   % woule be PHDelay
   [rp,cp,nblk,iosize,blkiosize,szmd] = ndlftinfo(m);
   if nargin==2
      if isempty(m.Literal)
         tmp = m;
      else
         % XXX relies on that every atom can be set to 0
         tmp = setlitr(m,find(m.Literal==1),0);
      end
      out = zeros([iosize szmd(3:end)]);
      out(:,:,:) = tmp.MatrixData(rp(end):end,cp(end):end,:);
   else
      % pvget(M,'NominalValue',blkid1,blkid2,blkid3,...)
      %   Sets the blocks identified by the list equal to
      %   their nominal values.  List should be CHARs
      % XXX should also take a single Cell array.
      out = m;
      for i=1:length(varargin)
         tmp = gblkidx(out,varargin{i});
         if out.Literal(tmp)==1
            out = setlitr(out,tmp,0);
         end
         out = delblk(out,tmp);
      end
      out = depromote(out);
   end
case 'LFTData'
   if nargin==2
      out.MofMDelta = pvget(m,'MofMDelta');
      out.DELTAofMDelta = pvget(m,'DELTAofMDelta');
      out.BlkInfoStruct = pvget(m,'BlkInfoStruct');
   elseif nargin==3
      L = varargin{1};
      if isa(L,'char')
         L = cellstr(L);
      end
      if isa(L,'cell') && ndims(L)==2 && min(size(L))==1
         Mtmp = m;
         Delta = [];
         for i=length(L):-1:1
            idx = gblkidx(Mtmp,L{i});
            if ~isempty(idx)
               acm = pvget(Mtmp.DepData,'AtomCell');
               Latom = acm{idx};
               copies = Mtmp.Copies;
               Mtmp = setlitr(Mtmp,idx,0); % always NDLFT/SETLITR
               Mtmp = blk2ioch(Mtmp,L{i},'t'); % always NDLFT/BLK2IOCH
               newDelta = LOCALatom2ndlft(Latom,copies(idx));
               Delta = blkdiag(newDelta,Delta);
            end
         end
         out.MofMDelta = Mtmp;
         out.DELTAofMDelta = Delta;
         out.BlkInfoStruct = pvget(Delta,'BlkInfoStruct');
      else
         error('3rd argument should be a cell array or character array');
      end
   else
      error('Invalid syntax.');
   end
case 'MofMDelta'
   % SETLITR preserves the block ordering
   tmp = setlitr(m,find(m.Literal==1),0);
   out = tmp.MatrixData;
case 'DELTAofMDelta'
   % in case {'M'} above, SETLITR preserves the block
   % ordering, so the same order is used here too.
   [rp,cp,nblk,iosize,blkiosize,szmd] = ndlftinfo(m);
   nr = blkiosize(1);
   nc = blkiosize(2);
   out = ndlft([zeros(nc,nr) eye(nc);eye(nr) zeros(nr,nc)],...
      m.DepData,m.Copies,0*m.Literal);
case 'BlkInfoStruct'
   [rp,cp,nblk,iosize,blkiosize,szmd] = ndlftinfo(m);
   atomcell = pvget(m.DepData,'AtomCell');
   NormalizedDeltaCell = cell(sum(m.Copies),1);
   N4 = zeros(nblk,4);
   fnames = {'Name' 'Size' 'Type' 'Occurrences'};
   vals = repmat({cell(nblk,1)},[1 4]);
   svinfo = [fnames;vals];
   N1 = struct(svinfo{:});
   pt = 1;
   for i=1:nblk
      aname = pvget(atomcell{i},'Name');
      sz = size(atomcell{i});
      inttype = pvget(atomcell{i},'IntType');
      catc = class(atomcell{i});
      switch catc
         case 'ureal'
            natom = ureal([aname 'Normalized'],0);
         case 'ucomplex'
            natom = ucomplex([aname 'Normalized'],0);
         case 'ultidyn'
            natom = ultidyn([aname 'Normalized'],size(atomcell{i}));
         case 'ucomplexm'
            natom = ucomplexm([aname 'Normalized'],zeros(size(atomcell{i})));
         case 'udyn'
            natom = udyn([aname 'Normalized'],size(atomcell{i}));
      end
      NormalizedDeltaCell(pt:pt+m.Copies(i)-1) = repmat({natom},[m.Copies(i) 1]);
      N4(i,:) = [sz m.Copies(i) inttype];
      N1(i).Name = aname;
      N1(i).Size = sz;
      N1(i).Type = catc;
      N1(i).Occurrences = m.Copies(i);
      pt = pt + m.Copies(i);
   end
   out.NormalizedDeltaCell = NormalizedDeltaCell;
   out.N1 = N1;
   out.N4 = N4;
case 'N23BlkStructure'
   [rp,cp,nblk,iosize,blkiosize,szmd] = ndlftinfo(m);
   atomcell = pvget(m.DepData,'AtomCell');
   if nargin==2
      allowsubs = 0;
   else
      allowsubs = isequal(varargin{1},1);
   end
   N3 = zeros(0,3);
   N2 = zeros(0,2);
   for i=1:nblk
      szatom = size(atomcell{i});
      szatom = szatom(1:2);
      switch class(atomcell{i})
         case 'ureal'
            N3 = [N3;1 1 -m.Copies(i)];
            N2 = [N2;-m.Copies(i) 0];
         case 'ucomplex'
            N3 = [N3;1 1 m.Copies(i)];
            N2 = [N2;m.Copies(i) 0];
         case {'udyn' ; 'ultidyn' ; 'ucomplexm'}
            if m.Copies(i)==1 || isequal(szatom,[1 1])
               N3 = [N3;szatom m.Copies(i)];
               if m.Copies(i)==1
                  N2 = [N2;szatom];
               else
                  N2 = [N2;m.Copies(i) 0];
               end
            elseif allowsubs
               % repeated, non-scalar, make them all separate elements.
               % upper bound will be conservative, but still an upper
               % bound.
               N3 = [N3;repmat([szatom 1],[m.Copies(i) 1])];
               N2 = [N2;repmat([szatom],[m.Copies(i) 1])];
            else
               msg1 = ['Uncertain element ' pvget(atomcell{i},'Name') ' is a '];
               msg2 = ['repeated, non-scalar ' upper(class(atomcell{i})) '.\n'];
               msg3 = ['Cannot extract muBlkStruct or muToolsBlkStruct in this case.'];
               msg = sprintf([msg1 msg2 msg3]);
               error(msg);
            end
         otherwise
            error('Unrecognizable uncertain element.');
      end
   end
   out.N2 = N2;
   out.N3 = N3;
case {'UNames' 'AtomList'}
%    [rp,cp,nblk,iosize,blkiosize,szmd] = ndlftinfo(m);
%    atomcell = pvget(m.DepData,'AtomCell');
%    out = cell(nblk,1);
%    for i=1:nblk
%       out{i} = pvget(atomcell{i,1},'Name');
%    end
    out = pvget(m.DepData,'UNames');
otherwise
   out = [];
   pflag = 0;
   errstring = ['Invalid NDLFT property: ''' property '''.'];
end
if nargout<=1 && pflag==0
   error(errstring);
end

function B = LOCALatom2ndlft(A,copies)
szA = size(A);
nr = szA(1)*copies; % rows of DELTA
nc = szA(2)*copies; % columns of DELTA
B = ndlft([zeros(nc,nr) eye(nc);eye(nr) zeros(nr,nc)],...
   atomlist({A},0),copies,0);

% case 'NormalizedDeltaCell'
%    atomcell = pvget(m.DepData,'AtomCell');
%    out = cell(sum(m.Copies),1);
%    pt = 1;
%    for i=1:nblk
%       aname = pvget(atomcell{i},'Name');
%       switch class(atomcell{i})
%          case 'ureal'
%             natom = ureal([aname 'Normalized'],0);
%          case 'ucomplex'
%             natom = ucomplex([aname 'Normalized'],0);
%          case 'ultidyn'
%             natom = ultidyn([aname 'Normalized'],size(atomcell{i}));
%          case 'ucomplexm'
%             natom = ucomplexm([aname 'Normalized'],zeros(size(atomcell{i})));
%          case 'udyn'
%             natom = udyn([aname 'Normalized'],size(atomcell{i}));
%       end
%       out(pt:pt+m.Copies(i)-1) = repmat({natom},[m.Copies(i) 1]);
%       pt = pt + m.Copies(i);
%    end
% case 'BlkStructure'
%    out = zeros(nblk,4);
%    atomcell = pvget(m.DepData,'AtomCell');
%    for i=1:nblk
%       if isa(atomcell{i},'ucomplex')
%          sz = pvget(atomcell{i},'BlockSize');
%       else
%          sz = size(atomcell{i});
%       end
%       out(i,:) = [sz m.Copies(i) pvget(atomcell{i},'IntType')];
%    end   
% case 'N4BlkStructure'
%    out = zeros(nblk,4);
%    atomcell = pvget(m.DepData,'AtomCell');
%    for i=1:nblk
%       if isa(atomcell{i},'ucomplex')
%          sz = pvget(atomcell{i},'BlockSize');
%       else
%          sz = size(atomcell{i});
%       end
%       out(i,:) = [sz m.Copies(i) pvget(atomcell{i},'IntType')];
%    end   
% case 'N1BlkStructure'
%    fnames = {'Name' 'Size' 'Type' 'Occurrences'};
%    vals = repmat({cell(nblk,1)},[1 4]);
%    svinfo = [fnames;vals];
%    out = struct(svinfo{:});
%    atomcell = pvget(m.DepData,'AtomCell');
%    for i=1:nblk
%       if isa(atomcell{i},'ucomplex')
%          sz = pvget(atomcell{i},'BlockSize');
%       else
%          sz = size(atomcell{i});
%       end
%       out(i).Name = pvget(atomcell{i},'Name');
%       out(i).Size = sz;
%       out(i).Type = class(atomcell{i});
%       out(i).Occurrences = m.Copies(i);
%    end   
% case 'N3BlkStructure'
%    if nargin==2
%       allowsubs = 0;
%    else
%       allowsubs = isequal(varargin{1},1);
%    end
%    out = zeros(0,3);
%    for i=1:nblk
%       szatom = size(atomcell{i});
%       szatom = szatom(1:2);
%       switch class(atomcell{i})
%          case 'ureal'
%             out = [out;1 1 -m.Copies(i)];
%          case 'ucomplex'
%             out = [out;1 1 m.Copies(i)];
%          case {'udyn' ; 'ultidyn' ; 'ucomplexm'}
%             if m.Copies(i)==1 || isequal(szatom,[1 1])
%                out = [out;szatom m.Copies(i)];
%             elseif allowsubs
%                % repeated, non-scalar, make them all separate elements.
%                % upper bound will be conservative, but still an upper
%                % bound.
%                out = [out;repmat([szatom 1],[m.Copies(i) 1])];
%             else
%                msg1 = ['Uncertain element ' pvget(atomcell{i},'Name') ' is a '];
%                msg2 = ['repeated, non-scalar ' upper(class(atomcell{i})) '.\n'];
%                msg3 = ['Cannot extract muBlkStruct in this case.'];
%                msg = sprintf([msg1 msg2 msg3]);
%                error(msg);
%             end
%          otherwise
%             error('Unrecognizable uncertain element.');
%       end
%    end
% case 'muToolsBlkStructure'
%    if nargin==2
%       allowsubs = 0;
%    else
%       allowsubs = isequal(varargin{1},1);
%    end
%    out = zeros(0,2);
%    for i=1:nblk
%       szatom = size(atomcell{i});
%       szatom = szatom(1:2);
%       switch class(atomcell{i})
%          case 'ureal'
%             out = [out;-m.Copies(i) 0];
%          case 'ucomplex'
%             out = [out;m.Copies(i) 0];
%          case {'udyn' ; 'ultidyn' ; 'ucomplexm'}
%             if m.Copies(i)==1
%                out = [out;szatom];
%             elseif isequal(szatom,[1 1])
%                out = [out;m.Copies(i) 0];
%             elseif allowsubs
%                % repeated, non-scalar, make them all separate elements.
%                % upper bound will be conservative, but still an upper
%                % bound.
%                out = [out;repmat(szatom,[m.Copies(i) 1])];
%             else
%                msg1 = ['Uncertain element ' pvget(atomcell{i},'Name') ' is a '];
%                msg2 = ['repeated, non-scalar ' upper(class(atomcell{i})) '.\n'];
%                msg3 = ['Cannot extract muToolsBlkStruct in this case.'];
%                msg = sprintf([msg1 msg2 msg3]);
%                error(msg);
%             end
%          otherwise
%             error('Unrecognizable uncertain element.');
%       end
%    end
% 
