function b = repmat(a,M,N)
% REPMAT Replicate and tile an array.
%
%  B = REPMAT(A,M,N) creates a large matrix B consisting of an M-by-N
%  tiling of copies of A.
% 
%  B = REPMAT(A,[M N]) accomplishes the same result as REPMAT(A,M,N).
% 
%  B = REPMAT(A,[M N P ...]) tiles the array A to produce a
%  M-by-N-by-P-by-... block array.  A can be N-D.
% 
%  REPMAT(A,M,N) when A is a scalar is commonly used to produce
%  an M-by-N matrix filled with A's value.
% 
%  See also BLKDIAG.

% Copyright 2003-2004 The MathWorks, Inc.

if nargin < 2
   error('Requires at least 2 inputs.')
elseif nargin == 2
  if length(M)==1
     repdata = [M M];
  else
     repdata = M;
  end
else
  repdata = [M N];
end
repdata = repdata(:).';
N = length(repdata);

b = a;
% STATUS MATRIXOP
[rp,cp,nblk,iosize,blkiosize,szmd] = ndlftinfo(a);
if N>length(szmd)
   szmd = [szmd ones(1,N-length(szmd))];
elseif N<length(szmd)
   repdata = [repdata ones(1,length(szmd)-N)];
   N = length(repdata);
end
   
if all(repdata>=0) && all(floor(repdata)==ceil(repdata))
   L.subs = cell(1,N);
   L.subs{1} = [1:rp(end)-1 repmat(rp(end):szmd(1),[1 repdata(1)])];
   L.subs{2} = [1:cp(end)-1 repmat(cp(end):szmd(2),[1 repdata(2)])];
   for i=3:N
      L.subs{i} = repmat(1:szmd(i),[1 repdata(i)]);
   end
   L.type = '()';
   b.MatrixData = builtin('subsref',a.MatrixData,L);
else
   error('Size vector should be a row vector with integer elements.');
end
