function  out = stack(dim,A,B)
% STACK for NDLFT

% Copyright 2003-2004 The MathWorks, Inc.

% STATUS UNCLEAR.  I believe this is correct, including 0-copy blocks.

[Arp,Acp,nblkA,ioA,blkioA,szma] = ndlftinfo(A); % still contains ZERO-copies info
[Brp,Bcp,nblkB,ioB,blkioB,szmb] = ndlftinfo(B); % still contains ZERO-copies info
if ~isequal(ioA,ioB)
   error('Input/Output Dimensions should be equal.');
end

if length(szma)<dim+2
   szma = [szma ones(1,dim+2-length(szma))];
end
if length(szmb)<dim+2
   szmb = [szmb ones(1,dim+2-length(szmb))];
end
tmpa = szma;
tmpa([1 2 dim+2]) = [];
tmpb = szmb;
tmpb([1 2 dim+2]) = [];
if ~isequal(tmpa,tmpb)
   error('Incompatible Array Dimensions.');
end

% BLKIO is [Output Input] dimensions for OPERATOR
Aatomcell = pvget(A.DepData,'AtomCell');
Batomcell = pvget(B.DepData,'AtomCell'); % these are cell arrays of atoms
bcell = [Aatomcell;Batomcell];
copies = [A.Copies;B.Copies];
UNames = [pvget(A.DepData,'UNames'); pvget(B.DepData,'UNames')];
[b,isort,jsort] = unique(double(char(UNames)),'rows');

if isempty(isort)
   isort = zeros(0,1); % forces Literal in ndlft/binop to be 0x1 in [] case
end

% Any blocks to be combined must be in the same "literal"
% state, i.e. no normalizer, or the same normalizer.
%
tnblk = max(jsort);
blkidA = zeros(sum(A.Literal),1);
nblkA = length(A.Literal);
blkidB = zeros(sum(B.Literal),1);
cntA = 0;
cntB = 0;
for idx=1:tnblk
   loc = find(jsort==idx);
   if length(loc)==2
      loc(2) = loc(2) - nblkA;
      if A.Literal(loc(1))==1 & B.Literal(loc(2))==0
         cntA = cntA + 1;
         blkidA(cntA) = loc(1);
      elseif A.Literal(loc(1))==0 & B.Literal(loc(2))==1
         cntB = cntB + 1;
         blkidB(cntB) = loc(2);
      end
   end
end

if cntA>0
    % XXX relies on that every atom can be set to 0
    A = setlitr(A,blkidA(1:cntA),0);
end
if cntB>0
    B = setlitr(B,blkidB(1:cntB),0);
end

% Any blocks merged now have the same "literal" state.
%
newLiteral = [A.Literal; B.Literal];
newLiteral = newLiteral(isort);

rna = [];
cna = [];
rnb = [];
cnb = [];
ca = [];
ra = [];
cb = [];
rb = [];
rpta = 1;
cpta = 1;
rptb = 1;
cptb = 1;

% right size, right order, need merge check
bcell = atomlist(bcell(isort,1),zeros(length(isort),1));
copies = copies(isort,1);

for idx=1:tnblk
   loc = find(jsort==idx);
   if length(loc)==1
      if loc<=nblkA
         ra = [ra Arp(loc):Arp(loc+1)-1];
         ca = [ca Acp(loc):Acp(loc+1)-1];
         rna = [rna rpta:rpta+(Arp(loc+1)-Arp(loc)-1)];
         cna = [cna cpta:cpta+(Acp(loc+1)-Acp(loc)-1)];
         rpta = rpta+(Arp(loc+1)-Arp(loc));
         cpta = cpta+(Acp(loc+1)-Acp(loc));
         rptb = rptb+(Arp(loc+1)-Arp(loc));
         cptb = cptb+(Acp(loc+1)-Acp(loc));
      else
         loc = loc - nblkA;
         rb = [rb Brp(loc):Brp(loc+1)-1];
         cb = [cb Bcp(loc):Bcp(loc+1)-1];
         rnb = [rnb rptb:rptb+(Brp(loc+1)-Brp(loc)-1)];
         cnb = [cnb cptb:cptb+(Bcp(loc+1)-Bcp(loc)-1)];
         rptb = rptb+(Brp(loc+1)-Brp(loc));
         cptb = cptb+(Bcp(loc+1)-Bcp(loc));
         rpta = rpta+(Brp(loc+1)-Brp(loc));
         cpta = cpta+(Bcp(loc+1)-Bcp(loc));
      end
   else
      % IT must be that
      % length(loc)==2 & loc(1)<=nblkA & loc(2)>nblkA
      loc(2) = loc(2) - nblkA;
      ra = [ra Arp(loc(1)):Arp(loc(1)+1)-1];
      ca = [ca Acp(loc(1)):Acp(loc(1)+1)-1];
      rna = [rna rpta:rpta+(Arp(loc(1)+1)-Arp(loc(1))-1)];
      cna = [cna cpta:cpta+(Acp(loc(1)+1)-Acp(loc(1))-1)];
      rb = [rb Brp(loc(2)):Brp(loc(2)+1)-1];
      cb = [cb Bcp(loc(2)):Bcp(loc(2)+1)-1];
      rnb = [rnb rptb:rptb+(Brp(loc(2)+1)-Brp(loc(2))-1)];
      cnb = [cnb cptb:cptb+(Bcp(loc(2)+1)-Bcp(loc(2))-1)];
      maxr = max(Arp(loc(1)+1)-Arp(loc(1)),Brp(loc(2)+1)-Brp(loc(2)));
      maxc = max(Acp(loc(1)+1)-Acp(loc(1)),Bcp(loc(2)+1)-Bcp(loc(2)));
      rpta = rpta+maxr;
      cpta = cpta+maxc;
      rptb = rptb+maxr;
      cptb = cptb+maxc;
      if isequal(Aatomcell{loc(1)},Batomcell{loc(2)})
         copies(idx) = max(A.Copies(loc(1)), B.Copies(loc(2)));
      else
         % XXX: should we provide more details about the differences?
         errStr = sprintf('Atoms named ''%s'' have different properties.',...
                     UNames{loc(1)});
         error(errStr);
      end
   end
end
Anew = zeros([(rpta+ioA(1)-1) (cpta+ioA(2)-1) szma(3:end)]);
Bnew = zeros([(rptb+ioB(1)-1) (cptb+ioB(2)-1) szmb(3:end)]);
Anew([rna rpta:rpta+ioA(1)-1],[cna cpta:cpta+ioA(2)-1],:) = ...
   A.MatrixData([ra Arp(nblkA+1):szma(1)],[ca Acp(nblkA+1):szma(2)],:);
Bnew([rnb rptb:rptb+ioB(1)-1],[cnb cptb:cptb+ioB(2)-1],:) = ...
   B.MatrixData([rb Brp(nblkB+1):szmb(1)],[cb Bcp(nblkB+1):szmb(2)],:);
Rmat = cat(dim+2,Anew,Bnew);
out = ndlft(Rmat,bcell,copies,newLiteral);
