function out = robopt(varargin)
%ROBOPT    Create a ROBUSTSTAB/ROBUSTPERF OPTIONS object.
%  OPTIONS = ROBOPT('NAME1',VALUE1,'NAME2',VALUE2,...) creates a
%  options object for the commands ROBUSTSTAB and ROBUSTPERF.
% 
%  OPTIONS = ROBOPT, by itself, creates an object, initialized
%  with all default values.
%
%  ROBOPT PROPERTIES
%
%  Display - Displays progress of computations. Default is 'off'. 
%  Sensitivity - Computes margin sensitivity to individual 
%            uncertainties.  Default is 'on'.
%  VaryUncertainty - Percentage variation of uncertainty for sensitivity
%           calculations (finite-difference estimate). Default is 25.
%  Mussv - Option string used by MUSSV. Default is 'sm9'
%  Default - A 1-by-1 structure of ROBOPT property default values.
%  Meaning - A 1-by-1 structure text description of each property. 
%
%
% See Also: DKITOPT, ROUSTSTAB, ROBUSTPERF, WCGAIN, WCGOPT, WCSENS, WCMARGIN.

% Copyright 2003-2004 The MathWorks, Inc.

superiorto('char','double','optunit');
validate = 1;
nin = nargin;
if ceil(nin/2)~=floor(nin/2)
   validate = varargin{end};
   nin = nargin - 1;
end
hnin = nin/2;
mkatend = 1;

OptCell = {...
optunit('Display','off','off','Displays progress of computations [{''off''} | ''on'']',1);...
optunit('Sensitivity','on','on',...
 'Computes margin sensitivity to individual uncertainties [''off'' | {''on''}]',1);...
optunit('VaryUncertainty',25,25,...
   'Percentage variation of uncertainty for sensitivity calculations',1);...
optunit('Mussv','sm9','sm9','Option string used by MUSSV',1);...
};

if mkatend
   out.OptCell = OptCell;
   out = class(out,'robopt',gsref(0));
end
if ceil(hnin)==floor(hnin)
   for i=1:hnin
      out = set(out,varargin{2*(i-1)+1},varargin{2*i});
   end
end

% Check validity of OUT.  Without any output arguments it will
% error out with appropriate error message if not valid.
%if isequal(validate,1)
if validate==1
   isvalid(out);
end
