%PVGET  Get values of public UCOMPLEX object properties.
%  
%VALUES = PVGET(MAT) returns all public values in a cell
%array VALUES.
%  
%VALUE = PVGET(SYS,PROPERTY) returns the value of the
%single property with name PROPERTY.
%  
%See also GET.

% Copyright 2003-2004 The MathWorks, Inc.
function [out,pflag,errstring] = pvget(p,property,varargin)

pflag = 1;
errstring = '';
switch property
case {'Name','NominalValue','AutoSimplify'}
   out = pvget(p.atom,property);
case 'PropNames'
   if isempty(p.Data)
      TypeDepSStr = {};
      TypeDepGStr = {};
      TypeDepSDesc = {};
   else
      anom = abs(pvget(p.atom,'NominalValue'));
      if anom~=0
         TypeDepSStr = {'Mode';'Radius';'Percentage'};
         TypeDepGStr = {'Mode';'Radius';'Percentage'};
         TypeDepSDesc = {'Radius | Percentage';'scalar DOUBLE';...
                 'scalar DOUBLE'};
      else
         TypeDepSStr = {'Mode';'Radius';'Percentage'};
         TypeDepGStr = {'Mode';'Radius';'Percentage'};
         TypeDepSDesc = {'Radius | Percentage';'scalar DOUBLE';...
                 'Not settable since Nominal==0'};
      end
   end
   aGS = pvget(p.atom,'PropNames');
   nidx = find(strcmpi(aGS.SPropNames,'NominalValue'));
   if length(nidx)>0
      aGS.SPropDescription{nidx} = '1x1 DOUBLE';
   end
   ng = length(aGS.GPropNames);
   ns = length(aGS.SPropNames);
   asing = find(strcmpi(aGS.GPropNames,'AutoSimplify'));
   asins = find(strcmpi(aGS.SPropNames,'AutoSimplify'));
   if length(asing)>0
      use = 1:ng;
      use(asing) = [];
      out.GPropNames = [aGS.GPropNames(use);TypeDepGStr;aGS.GPropNames(asing)];
   end
   if length(asins)>0
      use = 1:ns;
      use(asins) = [];
      out.SPropNames = [aGS.SPropNames(use);TypeDepSStr;aGS.SPropNames(asins)];
      out.SPropDescription = [aGS.SPropDescription(use);TypeDepSDesc;aGS.SPropDescription(asins)];
   end
case 'atom'
   out = p.atom;
case 'Data'
   out = p.Data;
case 'Traits'
   out.scalar = 1;
   out.linear = 1;
   out.zizo = 1;
   out.dynamic = 0;
   out.uncertain = 1;
case 'Radius'
   if isempty(p.Data)
      out = zeros(0,1);
   else
      out = p.Data(1);
   end
case 'Percentage'
   if isempty(p.Data)
      out = zeros(0,1);
   else
      nom = pvget(p.atom,'NominalValue');
      if nom==0
         out = [inf];
      else
         out = 100*p.Data(1)/abs(nom);
      end
   end
case 'Instance'
   if nargin==2
      N = [];
   else
      N = varargin{1};
   end
   if isempty(N)
      N = 1;
   end
   r = randn(N,1)+sqrt(-1)*randn(N,1);
   r = r./abs(r);
   rdisc = sqrt(rand(N,1)).*r;
   out = pvget(p.atom,'NominalValue') + p.Data(1)*rdisc;
case 'Normalizer'
   cen = pvget(p.atom,'NominalValue');
   rad = p.Data(1);
   S = [0 sqrt(rad);sqrt(rad) cen];
   T = flipud(fliplr(inv(S)));
   out.S = S;
   out.T = T;
   out.Tarb = [];
case 'IntType'
   out = 3;
case 'Mode'
   if ~isempty(p.Data)
      pv = {'Radius';'Percentage'};
      out = pv{p.Data(2)};
   else
      out = [];
   end
case 'BlockSize'
   out = size(pvget(p.atom,'NominalValue'));
otherwise
   out = [];
   pflag = 0;
   errstring = ['Invalid UCOMPLEX property: ''' property '''.'];
end
if nargout<=1 && pflag==0
   error(errstring);
end

