%PVSET  Set properties of UCOMPLEX objects.
%  
%MAT = PVSET(MAT,'Property1',Value1,'Property2',Value2,...)
%sets the values of the properties with exact names 'Property1',
%'Property2',...
%  
%See also SET.

% Copyright 2003-2005 The MathWorks, Inc.
function [p,pflag,errstring] = pvset(p,property,value)

Types = {'Radius';'Percentage'};
thistype = Types{p.Data(2)};

pflag = 1;
errstring = '';
switch property
case {'Name','AutoSimplify'}
   [A,pflag,errstring] = pvset(p.atom,property,value);
   if pflag==1
      p.atom = A;
   end
case {'NominalValue'}
   if isa(value,'double') & isequal(size(value),[1 1])
      if p.Data(2)==2 % Percentage Mode
         perc = p.Data(1)/abs(pvget(p.atom,'NominalValue'));
         newradius = abs(value)*perc;
         p.Data = [newradius p.Data(2)];
      end
      [A,pflag,errstring] = pvset(p.atom,property,value);
      if pflag==1
         p.atom = A;
      end
   else
      pflag = 0;
      errstring = ['Invalid NominalValue, NominalValue must be scalar. double.'];
   end
case 'Radius'
   if isa(value,'double') & isequal(size(value),[1 1]) & ...
    imag(value)==0 & value>0
      mode = p.Data(2);
      nom = pvget(p.atom,'NominalValue');
      p.Data = [value mode];
   else
      pflag = 0;
      errstring = ['Invalid Radius, Radius must be positive scalar.'];
   end
case 'Percentage'
   if isa(value,'double') & isequal(size(value),[1 1]) & ...
    imag(value)==0 & value>0
      mode = p.Data(2);
      radius = value*abs(pvget(p.atom,'NominalValue'))*.01;
      p.Data = [radius mode];
   else
      pflag = 0;
      errstring = ['Invalid Percentage, Percentage must be positive scalar.'];
   end
case 'Mode'
   pv = {'Radius','Percentage'};
   idx = find(strncmpi(pv,value,length(value)));
   if idx==1
      p.Data(2) = 1;
   elseif idx==2
      anom = abs(pvget(p.atom,'NominalValue'));
      if anom>0
         if p.Data(2)==1
            p.Data(2) = 2;
         end
      else
         pflag = 0;
         errstring = ['Cannot change Mode to Percentage when NominalValue equals 0.'];
      end
   else
      pflag = 0;
      errstring = ['Invalid Mode choice for UCOMPLEX'];
   end
end
if nargout<=1 && pflag==0
   error(errstring);
end

