function nvalue = actual2normalized(atom,value)
%ACTUAL2NORMALIZED   Computes the normalized-distance between
%  an uncertain atom's nominal value and a given value.
%
%  NDIST = ACTUAL2NORMALIZED(A,V) is the normalized-distance
%  between the nominal value of the uncertain atom A and the
%  given value V.  If A is a UREAL, then NDIST may be positive
%  or negative, reflecting that V is greater than, or less than
%  the nominal value.  If A is any other class of uncertain atom,
%  then NDIST is nonnegative.
%
%  If V is an array of values, then NDIST will be an array of
%  normalized distances.
%
%  Normalized-Distance from Nominal is used in interpreting the
%  robustness margins computed by the ROBUSTSTAB and ROBUSTPERF
%  commands.
%
%  See also: NORMALIZED2ACTUAL, ROBUSTSTAB, ROBUSTPERF

% Copyright 2003-2004 The MathWorks, Inc.

if nargin<2
    error('nvalue = actual2normalized(atom,avalue)');
end

if ~isa(value,'double')
   error('For UCOMPLEXM, the actual value must be a double.');
end

szv = size(value);
if isequal(szv(1:2),size(atom))
   nvalue = zeros([1 1 szv(3:end)]);
   N = pvget(atom,'Normalizer');
   for i=1:prod(szv(3:end))
      nvalue(:,:,i) = norm(lft(value(:,:,i),N.T)); 
   end
else
   error('Value is not the correct dimension for ATOM');
end

