%PVGET  Get values of public UCOMPLEXM object properties.
%  
%VALUES = PVGET(MAT) returns all public values in a cell
%array VALUES.
%  
%VALUE = PVGET(SYS,PROPERTY) returns the value of the
%single property with name PROPERTY.
%  
%See also GET.

% Copyright 2003-2004 The MathWorks, Inc.
function [out,pflag,errstring] = pvget(p,property,varargin)

pflag = 1;
errstring = '';
switch property
case 'WL'
   out = p.WL;
case 'WR'
   out = p.WR;
case {'Name','NominalValue','AutoSimplify'}
   out = pvget(p.atom,property);
case 'PropNames'   
   aGS = pvget(p.atom,'PropNames');
   nidx = find(strcmpi(aGS.SPropNames,'NominalValue'));
   szm = size(pvget(p.atom,'NominalValue'));
   ird = int2str(szm(1));
   icd = int2str(szm(2));
   ird = 'Nr';
   icd = 'Nc';
   if length(nidx)>0
      aGS.SPropDescription{nidx} = [ird ' x ' icd ' complex DOUBLE'];
   end
   TypeDepGStr = {'WL';'WR'};
   TypeDepSStr = {'WL';'WR'};
   TypeDepSDesc = {[ird ' x ' ird ' complex DOUBLE'];[icd ' x ' icd ' complex DOUBLE'];};
   ng = length(aGS.GPropNames);
   ns = length(aGS.SPropNames);
   asing = find(strcmpi(aGS.GPropNames,'AutoSimplify'));
   asins = find(strcmpi(aGS.SPropNames,'AutoSimplify'));
   if length(asing)>0
      use = 1:ng;
      use(asing) = [];
      out.GPropNames = [aGS.GPropNames(use);TypeDepGStr;aGS.GPropNames(asing)];
   else
      out.GPropNames = [aGS.GPropNames;TypeDepGStr];
   end
   if length(asins)>0
      use = 1:ns;
      use(asins) = [];
      out.SPropNames = [aGS.SPropNames(use);TypeDepSStr;aGS.SPropNames(asins)];
      out.SPropDescription = [aGS.SPropDescription(use);TypeDepSDesc;aGS.SPropDescription(asins)];
   else
      out.SPropNames = [aGS.SPropNames;TypeDepSStr];
      out.SPropDescription = [aGS.SPropDescription;TypeDepSDesc];
   end
case 'BlockSize'
   out = [size(p.WL,2) size(p.WR,1)];
case 'atom'
   out = p.atom;
case 'Instance'
   if nargin==2
      Npts = [];
   else
      Npts = varargin{1};
   end
   if isempty(Npts)
      Npts = 1;
   end
   blksize = pvget(p.atom,'Size');
   delta = (rand([blksize Npts])-.5) + sqrt(-1)*(rand([blksize Npts])-.5);
   out = zeros([blksize Npts]);
   nv = pvget(p.atom,'NominalValue');
   for i=1:Npts
      sf = sqrt(rand(1,1));  % check...
      out(:,:,i) = nv + p.WL*(sf*delta(:,:,i)/norm(delta(:,:,i)))*p.WR;
   end
case 'Normalizer'
   blksize = pvget(p.atom,'Size');
   nom = pvget(p.atom,'NominalValue');
   snom = size(nom);
   S = [zeros(blksize(2),blksize(1)) p.WR;p.WL nom];
   WLi = pinv(p.WL);
   WRi = pinv(p.WR);
   T = [zeros(snom(2),snom(1)) WRi;WLi -WLi*nom*WRi];
   %Si = inv(S);
   %T = [Si(szn(2)+1:end,szn(1)+1:end) Si(szn(2)+1:end,1:szn(1));...
   %      Si(1:szn(2),szn(1)+1:end) Si(1:szn(2),1:szn(1))];
   out.S = S;
   out.T = T;
   out.Tarb = [];
case 'Traits'
   szn = size(pvget(p.atom,'NominalValue'));
   out.scalar = szn(1)==1 & szn(2)==1;
   out.linear = 1;
   out.zizo = 1;
   out.dynamic = 0;
   out.uncertain = 1;
case 'IntType'
   out = 4;
otherwise
   out = [];
   pflag = 0;
   errstring = ['Invalid UCOMPLEXM property: ''' property '''.'];
end
if nargout<=1 && pflag==0
   error(errstring);
end
