%PVGET  Get values of public UDYN object properties.
%  
%VALUES = PVGET(MAT) returns all public values in a cell
%array VALUES.
%  
%VALUE = PVGET(SYS,PROPERTY) returns the value of the
%single property with name PROPERTY.
%  
%See also GET.

% Copyright 2003-2004 The MathWorks, Inc.
function [out,pflag,errstring] = pvget(p,property)

pflag = 1;
errstring = '';
switch property
case {'Name','NominalValue','AutoSimplify'}
   out = pvget(p.atom,property);
case 'atom'
   out = p.atom;
case 'Traits'
   [ny,nu] = size(pvget(p.atom,'NominalValue'));
   out.scalar = (ny==1)&(nu==1);
   out.linear = 0;
   out.zizo = 0;
   out.dynamic = 1;
   out.uncertain = 1;
case 'PropNames'
   aGS = pvget(p.atom,'PropNames');
   TypeDepGStr = {'Lip'};
   TypeDepSStr = {'Lip';'ioSize'};
   TypeDepSDesc = {'scalar DOUBLE';'1x2 DOUBLE'};
   TypeDepGStr = cell(0,1);
   TypeDepSStr = {'ioSize'};
   TypeDepSDesc = {'1x2 DOUBLE'};
   out.GPropNames = [aGS.GPropNames;TypeDepGStr];
   nidx = find(strcmpi(aGS.SPropNames,'NominalValue'));
   if length(nidx)>0
      n = length(aGS.SPropNames);
      keep = 1:n;
      keep(nidx) = [];
      out.SPropNames = [aGS.SPropNames(keep);TypeDepSStr];
      out.SPropDescription = [aGS.SPropDescription(keep);TypeDepSDesc];
   else
      out.SPropNames = [aGS.SPropNames;TypeDepSStr];
      out.SPropDescription = [aGS.SPropDescription;TypeDepSDesc];
   end
case 'Lip'
   out = p.Lip;
case 'Normalizer'
   szp = size(p.atom);
   S = [zeros(szp(2),szp(1)) eye(szp(2));eye(szp(1)) zeros(szp(1),szp(2))];
   T = S;
   out.S = S;
   out.T = T;
   out.Tarb = [];
case 'Instance'
   szp = size(p.atom);
   out = zeros(szp);
case 'IntType'
   out = 5;
case 'BlockSize'
   out = size(pvget(p.atom,'NominalValue'));
otherwise
   out = [];
   pflag = 0;
   errstring = ['Invalid UDYN property: ''' property '''.'];
end
if nargout<=1 && pflag==0
   error(errstring);
end
