%BLK2IOCH Converts an uncertainty to an input/output block.
%
%[OUT,IDFLG] = BLK2IOCH(MAT,BLKID,TB) opens up a LFT channel, moving the 
%exposed inputs/outputs into the top (or bottom) of the I/O block. MAT is
%an uncertain matrix, BLKID is the name or numerical index of the uncertain 
%that is present in MAT and TB is a string that determines if the block 
%should be moved to the top, 't' or the bottom, 'b' of the inputs and outputs
%of OUT. OUT is an uncertain matrix with BLKID at either the top, if
%TB='t', or bottom, TB='b', of the inputs/outputs of the original uncertain
%matrix MAT. IDFLG is 1 if the BLKID was an uncertain atom in MAT otherwise
%it is 0.
%
%BLK2IOCH returns a UFRD object.
%
%See also: NORMALIZE


% function [out,idflg] = blk2ioch(mat,blkid,tb)
%
%  Opens up a LFT channel, moving the exposed
%  inputs/outputs into the top (or bottom) of
%  the I/O block.  Uses name in BLKID, or
%  numerical index.
%
%  Always returns a UFRD

% Copyright 2003-2004 The MathWorks, Inc.
function [out,idflg] = blk2ioch(mat,blkid,tb,litreq)

if nargin<3
   estr = 'Invalid Syntax.';
   error(estr);
elseif nargin==3
   litreq = 'literalZERO';
end

[outtmp,idflg] = blk2ioch(mat.ResponseData,blkid,tb,litreq);

szo = size(outtmp); % new O/I size
szm = size(mat.ResponseData); % old O/I size
szdiff = szo(1:2)-szm(1:2); % O/I dimension increase

% puts blanks for names in these channels
% XXXMINOR - could use the atom's name...
tmpds = dynamicsys(szdiff(1),szdiff(2));
if strcmp(tb,'t')
   tmpds = utAppend(tmpds,mat.dynamicsys);
elseif strcmp(tb,'b')
   tmpds = utAppend(mat.dynamicsys,tmpds);
end
out = ufrd(outtmp,mat.Frequency,'UNITS',mat.Units,mat.Ts,tmpds);
