%  BLKDIAG  Group uncertain models by appending their inputs and outputs.
%  
%     SYS = BLKDIAG(SYS1,SYS2, ...) produces the aggregate system
%   
%                   [ SYS1  0       ]
%             SYS = [  0   SYS2     ]
%                   [           .   ]
%                   [             . ]
%  
%     BLKDIAG concatenates the input and output vectors of the uncertain
%     models SYS1, SYS2,... to produce the resulting model SYS.
%  
%     If SYS1,SYS2,... are arrays of uncertain models, BLKDIAG returns an 
%     uncertain array SYS of the same size where 
%        SYS(:,:,k) = BLKDIAG(SYS1(:,:,k),SYS2(:,:,k),...) .
%  
%     See also SERIES, PARALLEL, FEEDBACK.
function out = blkdiag(varargin)

% Copyright 2003-2004 The MathWorks, Inc.

if nargin==1
   out = varargin{1};
elseif nargin==2
   A = varargin{1};
   B = varargin{2};
   if ~isa(A,'ufrd') 
      A = lift2ufrd(A,B.Frequency,B.Units);
   elseif ~isa(B,'ufrd')
      B = lift2ufrd(B,A.Frequency,A.Units);
   else   
   % Check frequency vectors
      try
         % give priority to rad/s if both FRD's
         [freq,units] = freqcheck(A.Frequency,A.Units,B.Frequency,B.Units);
      catch einfo
         throw(einfo);
      end
   end
   sza = size(A);
   szb = size(B);
   if ~isexdok(sza(4:end),szb(4:end))
         error('Invalid extra dimensions')
   end
   try
        out = binop(A,B,'blkdiag');
   catch einfo
        throw(einfo);
   end
else
   out = blkdiag(blkdiag(varargin{1},varargin{2}),varargin{3:end});
end