% INV  Inverse UFRD model.
% 
%  ISYS = INV(SYS) computes the inverse model ISYS such that
% 
%      y = SYS * u   <---->   u = ISYS * y 
% 
%  The UFRD model SYS must have the same number of inputs and
%  outputs.
% 
%  For arrays of UFRD models, INV is performed on each individual
%  model.
% 
%  See also mldivide, mrdivide, ltimodels.

function out = inv(a)
% Copyright 2003-2004 The MathWorks, Inc.

sza = size(a);
if sza(1)==sza(2)
   % cheat with mrdivide for now...
   % XXXMINOR dynamicsys handled in mrdivide
   newI = lift2ufrd(eye(sza(1)),a.Frequency,a.Units);
   out = binop(newI,a,'mrdivide');
else
   estr = 'Non-square models cannot be inverted.';
   error(estr)
end