%IPERMUTE  Inverse permute array of UFRD models.
%  A = IPERMUTE(B,ORDER) is the inverse of permute. IPERMUTE
%  rearranges the dimensions of B so that PERMUTE(A,ORDER) will
%  produce B.  The array produced has the same values of A but the
%  order of the subscripts needed to access any particular element
%  are rearranged as specified by ORDER.  The elements of ORDER 
%  must be a rearrangement of the numbers from 1 to N.
% 
%  PERMUTE and IPERMUTE are a generalization of transpose (.') 
%  for N-D UFRD arrays.
% 
%  Example
%    a = rss(3,2,2);
%    del = ultidyn('del',[2 2]);
%    aa = (a+del)*rand(2,2,3,4,5);
%    ag = frd(aa,logspace(-2,2,12));
%    b = permute(ag,[3 2 1]);
%    c = ipermute(b,[3 2 1]); % ag and c are equal
%  
%  See also PERMUTE.

function m = ipermute(m, index)
% Copyright 2003-2004 The MathWorks, Inc.

index = index(:).';
szm = size(m);
if (length(index)>=length(szm)-2 || (length(szm)==4 && szm(4)==1)) && ...
      isa(index,'double') && isequal(sort(index),1:length(index))
   % don't change OUTPUT/INPUT/FREQUENCY
   modindex = [1 2 3 3+index];
   try
      m.ResponseData = ipermute(m.ResponseData,modindex);
   catch einfo
      throw(einfo);
   end
else
   msg = 'ORDER contains an invalid permutation index.';
   error(msg);
end

