%ISVALID  True if input is an UFRD.
%  
%[PFLG,A] = ISVALID(SYS) returns PFLG==1 (true) if the uncertain 
%frequency response. SYS is an uncertain frequency response, and 0 
%otherwise. A  is the input argument SYS.

% Copyright 2003-2004 The MathWorks, Inc.
function [pflag,A] = isvalid(A)

pflag = true;

% XXXMINOR - errstr is not passed back.  Only UFRD calls ISVALID, and
% does so with two arguments, then generating a non-informitive error.
% We should fix this.
szA = size(A.ResponseData);
NY = szA(1);
NU = szA(2);
if isa(A.ResponseData,'umat')
   if isa(A.Frequency,'double') && ndims(A.Frequency==2) && (size(A.Frequency,2))==1 ...
         && isreal(A.Frequency) && all(sort(A.Frequency)==A.Frequency)
      if size(A.ResponseData,3)~=length(A.Frequency)
         if nargout<=1
            error('ResponseData has inconsistent number of Frequency points')
         else
            errstr = 'ResponseData has inconsistent number of Frequency points';
            pflag = 0;
         end
      end
      if ~isequal(A.Units,'rad/s') && ~isequal(A.Units,'Hz')
         if nargout<=1
            error('Units should be ''rad/s'' or ''Hz''')
         else
            errstr = 'Units should be ''rad/s'' or ''Hz''';
            pflag = 0;
         end
      end
   else
      if nargout<=1
         error('Frequency should be real, sorted, double vector')
      else
         errstr = 'Frequency should be real, sorted, double vector';
         pflag = 0;
      end
   end
   if ~(isa(A.Ts,'double') && ndims(A.Ts)==2 && all(size(A.Ts)==[1 1]) && isreal(A.Ts) &&...
         (A.Ts>=0 || A.Ts == -1 || A.Ts == -2))
      pflag = 0;
      errstr = 'Invalid sample time.';
   end
else
   if nargout<=1
      error('ResponseData is inappropriate Size');
   else
      errstr = 'ResponseData is inappropriate Size';
      pflag = 0;
   end
end
if pflag==0
   pflag = false;
end

% XXXMAJOR
A.dynamicsys = utCheckSystem(A.dynamicsys,NY,NU);