%MLDIVIDE  Left division for UFRD objects.
%  
%SYS = MLDIVIDE(SYS1,SYS2) is invoked by SYS=SYS1\SYS2.
%and is equivalent to SYS = INV(SYS1)*SYS2.
%  
%See also MRDIVIDE, INV, MTIMES.

% Copyright 2003-2004 The MathWorks, Inc.
function out = mldivide(A,B)

if ~isa(A,'ufrd') 
   A = lift2ufrd(A,B.Frequency,B.Units);
elseif ~isa(B,'ufrd')
   B = lift2ufrd(B,A.Frequency,A.Units);
else   
   % Check frequency vectors
   try
      % give priority to rad/s if both FRD's
      [freq,units] = freqcheck(A.Frequency,A.Units,B.Frequency,B.Units);
   catch einfo
      throw(einfo);
   end
end

sza = size(A);
szb = size(B);
if ~isexdok(sza(4:end),szb(4:end))
   estr = 'Array dimensions are incompatible.';
   error(estr)
end
if sza(1)==szb(1) && sza(1)==sza(2)
   try
      out = binop(A,B,'mldivide');
   catch einfo
      throw(einfo);
   end
elseif sza(1)==1 & sza(2)==1 & szb(1)>0 & szb(2)>0
   Aext = A;
   for i=2:szb(1)
      Aext = blkdiag(Aext,A);
   end
   out = Aext\B;
else
   estr = 'In A\B, A must be square, and A and B have the same output dimension.';
   error(estr);
end
