%MRDIVIDE  Right division for UFRD objects.
%  
%SYS = MRDIVIDE(SYS1,SYS2) is invoked by SYS=SYS1/SYS2.
%and is equivalent to SYS = SYS1*INV(SYS1).
%  
%See also MLDIVIDE, INV, MTIMES.

% Copyright 2003-2004 The MathWorks, Inc.
function out = mrdivide(A,B)
%XXXOK, XXXHELP

if ~isa(A,'ufrd') 
   A = lift2ufrd(A,B.Frequency,B.Units);
elseif ~isa(B,'ufrd')
   B = lift2ufrd(B,A.Frequency,A.Units);
else   
   % Check frequency vectors
   try
      % give priority to rad/s if both FRD's
      [freq,units] = freqcheck(A.Frequency,A.Units,B.Frequency,B.Units);
   catch einfo
      throw(einfo);
   end
end

sza = size(A);
szb = size(B);
if ~isexdok(sza(4:end),szb(4:end))
   estr = 'Array dimensions are incompatible.';
   error(estr)
end
if sza(2)==szb(2) && szb(1)==szb(2)
   try
      out = binop(A,B,'mrdivide');
   catch einfo
      throw(einfo);
   end
elseif szb(1)==1 & szb(2)==1 & sza(1)>0 & sza(2)>0
   Bext = B;
   for i=2:sza(2)
      Bext = blkdiag(Bext,B);
   end
   out = A/Bext;
else
   estr = 'In A/B, B must be square, and A and B have the same input dimension.';
   error(estr);
end
