function [freqs,units] = freqcheck(freqsLHS,unitsLHS,freqsRHS,unitsRHS)
%FREQCHECK  Check compatibility of frequency vectors, considering units.
%           If either system has units of rad/s, return frequencies in
%           rad/s, otherwise, return frequencies in Hz.

% Pulled directly from CTB private directories

%   Author: S. Almy
%   Copyright 1986-2004 The MathWorks, Inc.
%   $Revision: 1.1.6.1 $  $Date: 2004/08/01 00:02:17 $


freqs = freqsLHS;
units = unitsLHS;
reltol = 1e3*eps;

% Unit alignment
if ~strcmpi(unitsLHS,unitsRHS)
   % Convert units to rad/sec
   if strncmpi(unitsRHS,'h',1)
      freqsRHS = freqsRHS*2*pi;
   else
      freqsLHS = freqsLHS*2*pi;
      freqs = freqsRHS;
      units = unitsRHS;
   end
end

% The frequency points in FREQLHS and FREQRHS should now match
% up to round-off errors of level RELTOL
if length(freqsLHS) ~= length(freqsRHS)
   error('Size mismatch. Length of frequency vectors must match.')
elseif any(abs(freqsLHS-freqsRHS)>reltol*(1+freqsLHS))
   error('Frequency points don''t match (taking units into account).');
end
