function [A,B,C,D] = utfit1omega(w,Ts)
% UTFIT1OMEGA creates system whose frequency response is
%   exactly 1 at a given frequency.  The frequency response
%   is 0 at low and high frequency, and the peak magnitude
%   is 1.

% Copyright 2004 The MathWorks, Inc.

ni = nargin;
if ni==1
   Ts = 0;
elseif ni==0
   error('Need at least one input argument');
end

if isa(w,'double') && ndims(w)==2 && all(size(w)==[1 1]) && ...
      imag(w)==0 && w>0
   xi = 1/sqrt(2);
   if Ts==0
      wn = w;
      % sys = ss(tf([2*xi*wn 0],[1 2*xi*wn wn^2])); % too slow
      lterm = 2*xi*wn;
      sterm = sqrt(lterm);
      A = [0 wn;-wn -lterm];
      B = [0;sterm];
      C = [0 sterm];
      D = 0;
   elseif Ts>0
      theta = w*Ts;
      ejt = exp(sqrt(-1)*theta);
      wn = imag((ejt-1)/(ejt+1));
      tmp = 2*xi*wn;
      ws = wn^2;
      a = tmp;
      b = 1+tmp+ws;
      c = -2 + 2*ws;
      d = 1-tmp+ws;
      [T,M] = balance([0 1 0;-d/b -c/b 1;-a*(b+d)/b^2 -a*c/b^2 a/b]);
      A = M(1:2,1:2);
      B = M(1:2,3);
      C = M(3,1:2);
      D = M(3,3);
%       sys = tf([tmp 0 -tmp],[1+tmp+ws -2+2*ws 1-tmp+ws],Ts);
   elseif Ts==-1
      % Treat as TS=1, then put back with Ts = -1;
      theta = w;
      ejt = exp(sqrt(-1)*theta);
      wn = imag((ejt-1)/(ejt+1));
      tmp = 2*xi*wn;
      ws = wn^2;
      a = tmp;
      b = 1+tmp+ws;
      c = -2 + 2*ws;
      d = 1-tmp+ws;
      [T,M] = balance([0 1 0;-d/b -c/b 1;-a*(b+d)/b^2 -a*c/b^2 a/b]);
      A = M(1:2,1:2);
      B = M(1:2,3);
      C = M(3,1:2);
      D = M(3,3);
%       sys = tf([tmp 0 -tmp],[1+tmp+ws -2+2*ws 1-tmp+ws],-1);
   else
      error('Invalid Sample-Time.');
   end
else
   error('W must be real, positive scalar.');
end

