function [A,B,C,D] = vecorat(frequency,gainvec,ts,topleftreal,fs)

% Copyright 2004 The MathWorks, Inc.

nin = nargin;
exr = 0;
if nargin==3
   fs = 0;
elseif nin==5 && ~isempty(topleftreal)
   exr = 1;
end
if isempty(gainvec)
   if exr==1
      A = zeros(0,0);
      B = zeros(0,1);
      C = zeros(1,0);
      D = topleftreal;
   else
      out = gainvec;
   end
   return
end

if isa(frequency,'double') & isa(gainvec,'double')
   szg = size(gainvec); % not empty, see above
   if szg(2)>1 || fs==2 % row vector
      A = zeros(szg(2),szg(2));
      C = zeros(1,szg(2));
      B = zeros(szg(2),exr+szg(2));
      D = zeros(1,exr+szg(2));
      for i=1:length(gainvec)
         [A(i,i),B(i,exr+i),C(i),D(exr+i)] = LOCALsisorat(frequency,gainvec(i),ts);
      end
      if exr==1
         D(1) = topleftreal;
      end
      %out = ss(A,B,C,D,ts);
   elseif szg(1)>1 || fs==1 % column
      A = zeros(szg(1),szg(1));
      B = zeros(szg(1),1);
      C = zeros(exr+szg(1),szg(1));
      D = zeros(exr+szg(1),1);
      for i=1:length(gainvec)
         [A(i,i),B(i),C(exr+i,i),D(exr+i)] = LOCALsisorat(frequency,gainvec(i),ts);
      end
      if exr==1
         D(1) = topleftreal;
      end
      %out = ss(A,B,C,D,ts);
   elseif szg(1)==1 & szg(2)==1
      [A,B,C,D] = LOCALsisorat(frequency,gainvec(1),ts);
      %out = ss(A,B,C,D,ts);
   else
      error('GAINVEC should be a row or column');
   end
else
   error('Input arguments should be DOUBLES');
end
    
% function out = LOCALsisorat(frequency,gain)
%    or
% function out = LOCALsisorat(data_g)
%
%   Finds a first order, stable transfer function that has a
%   complex gain of GAIN at a frequency FREQUENCY.

% In the case of one
%   input argument, DATA_G must be a 1x1 VARYING matrix, with
%   1 data point. The INDEPENDENT VARIABLE is interpreted as
%   FREQ, and the value of the matrix is interpreted as GAIN.
%   The transfer function produced has constant magnitude
%   across frequencies.
%
%   See also: DYPERT, MU, RANDEL, UNWRAPP.


function [a,b,c,d] = LOCALsisorat(freq,gain,ts)


% if ~isa(freq,'double') | ~isa(gain,'double') | ndims(freq)~=2 ...
%   | ndims(gain)~=2 | size(gain,1)~=1 | size(gain,2)~=1 ... 
%   | size(freq,1)~=1 | size(freq,2)~=1 
%    error('SISORAT needs non-empty, scalar-valued frequency and gain')
% end
if freq < 0
   error('Frequency should be non-negative')
end
if freq == 0 & imag(gain) ~= 0
   msg = 'Nonzero frequency required to achieve a complex gain.\n';
   msg = [msg 'Resetting frequency to 1e-6.'];
   msg = sprintf(msg);
   warning(msg);
   freq = 1e-6;
end
if abs(gain)==0
   %warning('WARNING_IN_SISORAT: gain is zero')
   gain = 1e-8;
end
   disk = gain/abs(gain);
   if ts == 0 % continuous-time
      %if abs(imag(disk)) < abs(real(disk))*1000*eps
      
      %XXXMINOR
      if abs(imag(disk)) < abs(real(disk))*10*sqrt(eps)
         disk = real(disk);
         gain = real(gain);
      end
      sw = 1;
      if imag(disk) == 0
         b=1e-6*sqrt(abs(gain));
         c=b;
         a = -1;
         d = gain;
         %out = ss(-1,b,c,gain);
      else
         if imag(disk) < 0
            sw = -1;
            disk = -disk;
         end
         pscal = sqrt(abs(gain));
         beta = (imag(disk)*freq)/(1+real(disk));
         bc = sqrt(2*beta);
         % Use realization with abs(b)==abs(c)
         %out = ss(-beta,-pscal*bc,pscal*sw*bc,pscal*pscal*sw);
         a = -beta;
         b = -pscal*bc;
         c = -sw*b;
         d = pscal*pscal*sw;
      end
   else % discrete-time
      if abs(imag(disk)) < abs(real(disk))*1000*eps
         disk = real(disk);
         gain = real(gain);
      end
      if imag(disk) == 0
         a = -0.9 + 1.8*rand;
         b = 1e-6*sqrt(abs(gain));
         c = b;
         d = gain;
         %out = ss(a,b,c,d,ts);
      else 
         ats = abs(ts);
         if abs(pi-freq*ats)<1000*eps
            freq = freq*0.999;
         end
         etheta = exp(sqrt(-1)*freq*ats);
         betap = real((disk*etheta-1)/(disk-etheta)); 
         betam = real((disk*etheta+1)/(disk+etheta));
         beta = betap;
         pm = 1;
         if abs(betam)<1
            beta = betam;
            pm = -1;
         end
         b = sqrt(abs(gain*(1-beta^2)));
         c = pm*b;
         d = -pm*beta*abs(gain);
         a = beta;
         %out = ss(a,b,c,d,ts);
      end
   end

