%PVGET  Get values of public UFRD properties.
%  
%VALUES = PVGET(SYS) returns all public values in a cell
%array VALUES.
%  
%VALUE = PVGET(SYS,PROPERTY) returns the value of the
%single property with name PROPERTY.
%  
%See also GET.

% Copyright 2003-2006 The MathWorks, Inc.
function [out,pflag,errstring] = pvget(m,property,varargin)

pflag = 1;
errstring = '';
switch property
case 'ResponseData'
   out = m.ResponseData;
case {'dynamicsys' 'dynamicsys'}
   out = m.dynamicsys;
case 'Frequency'
   out = m.Frequency;
case 'Units'
   out = m.Units;
case 'Ts'
   out = m.Ts;
case 'Uncertainty'
   out = pvget(m.ResponseData,'Uncertainty');
case 'NominalValue'
   ResponseData = pvget(m.ResponseData,'NominalValue');
   out = frd(ResponseData,m.Frequency,'Units',m.Units);
%    Old code, eliminated Nov05, as LTI changed
%    tmp = get(ds2lti(m.dynamicsys,0));
%    fn = {'InputName';'OutputName';'InputGroup';'OutputGroup';'Notes';'UserData'};
%    out = pvset(out,'Ts',utarbts(m.Ts));
%    for i=1:length(fn)
%       out = pvset(out,fn{i},tmp.(fn{i}));
%    end
   namevalues = [pnames(m.dynamicsys)';pvget(m.dynamicsys)'];
   out = pvset(out,namevalues{:});
   out = setTs(out,utarbts(m.Ts));  % or out = pvset(out,'Ts',utarbts(m.Ts)) 11/05
case 'PropNames'
   com1 = {'Frequency';'ResponseData';'Units';'Ts'};
   s1d = {'vector of frequency points( Nf frequency points)';...
      'NY-by-NU-by-NF UMAT array of complex responses';...       
      '[ ''rad/s'' | ''Hz'' ]';...                            
      'Scalar (sample time in seconds)'};
   com4 = {'InputName';'OutputName';'InputGroup';'OutputGroup'};
   com4d = {'Nu-by-1 cell array of strings';...        
    'Ny-by-1 cell array of strings';...        
    'STRUCT of Input groups';... 
    'STRUCT of Output groups'};
   g5 = [{'NominalValue';'Uncertainty'}];
   s5 = [{'Uncertainty'}];
   s5d = {'not explicitly set by user, gateway to uncertain atoms'};
   g6 = [{'Notes';'UserData'}];
   s6 = [{'Notes';'UserData'}];
   s6d = {'Array or cell array of strings';'Arbitrary'};
   out.GPropNames = [com1;com4;g5;g6];
   out.SPropNames = [com1;com4;s5;s6];
   out.SPropDescription = [s1d;com4d;s5d;s6d];
case 'Instance'
   out = usample(m);
case {'Copies' 'AtomCell' 'Literal'}
   out = pvget(m.ResponseData,property);
% case {'DELTAofMDelta' 'BlkStructure' 'muToolsBlkStructure' 'N3BlkStructure' 'theDELTAasNDLFT'}
%    out = pvget(m.ResponseData,property);
case {'DELTAofMDelta' 'BlkInfoStructure' 'N23BlkStructure' 'theDELTAasNDLFT'}
   % N23BlkStructure allows for 3rd ALLOWSUBS argument,  that is what the
   % varargin is for.  
   out = pvget(m.ResponseData,property,varargin{:});
case 'UNames'
   out = pvget(m.ResponseData,'UNames');
case {'InputName','OutputName','InputGroup','OutputGroup',...
        'Notes','UserData','Version','Name'}
   out = pvget(m.dynamicsys,property);
otherwise
   out = [];
   pflag = 0;
   errstring = ['Invalid UFRD property: ''' property '''.'];
end
if nargout<=1 && pflag==0
   error(errstring);
end

function [RN,CN,RG,CG,RGs,CGs] = LOCALrcdata(blksz,nblk,blkn,rp,cp)
CN = repmat({''},[blksz(1) 1]);
RN = repmat({''},[blksz(2) 1]);
RG = cell(nblk,2);
CG = cell(nblk,2);
for i = 1:nblk
   RG{i,1}  = (rp(i):rp(i+1)-1);
   RG{i,2}  = [blkn{i} '_NC'];
   CG{i,1} = (cp(i):cp(i+1)-1);
   CG{i,2} = [blkn{i} '_NC'];
end
% Update to NEW GROUP
CGtmp = CG(:,[2 1])';
CGs = struct(CGtmp{:});
RGtmp = RG(:,[2 1])';
RGs = struct(RGtmp{:});

