%PVSET  Set properties of UFRD objects.
%  
%SYS = PVSET(SYS,'Property1',Value1,'Property2',Value2,...)
%sets the values of the properties with exact names 'Property1',
%'Property2',...
%  
%See also SET.

% Copyright 2003-2004 The MathWorks, Inc.
function [m,pflag,errstring] = pvset(m,property,value)

pflag = 1;
errstring = '';
switch property
case 'ResponseData'
   m.ResponseData = value;
case 'Units'
   m.Units = value;
case {'dynamicsys' 'dynamicsys'}
   m.dynamicsys = value;
case 'Frequency'
   m.Frequency = value;
case 'Ts'
   if isa(value,'double') && ndims(value)==2 && isreal(value) && all(size(value)==[1 1]) ...
         && (value>=0 | value==-1)
      m.Ts = value;
   else
      pflag = 0;
      errstring  = 'Ts should be -1, 0 or positive scalar';
   end
case {'InputName','OutputName','InputGroup','OutputGroup',...
        'Notes','UserData','Name'}
   m.dynamicsys = pvset(m.dynamicsys,property,value);
case 'Uncertainty'
   [tmp,pflag,errstring] = pvset(m.ResponseData,property,value);
   if pflag==1
      m.ResponseData = tmp;
   end
otherwise
   pflag = 0;
   errstring = ['Invalid UFRD property: ' property];
end
if nargout<=1 && pflag==0
   error(errstring);
end
      
