function B = reshape(A,varargin)
%RESHAPE  Reshape UFRD array
%  
%   B = RESHAPE(A,S1,S2,...,Sk) reshapes the UFRD array A
%   into a S1-by-S2-by...-Sk UFRD array.  There must
%   be S1*S2*...*Sk array elements to begin with.
%   
%   B = RESHAPE(A,[S1 S2 ... Sk]) is the same thing.
%  
%   See also SIZE, NDIMS.

% Copyright 2003-2004 The MathWorks, Inc.

newshape = [varargin{:}];

if length(newshape)<2
   error('At least two dimensions must be specified.');
end

sza = size(A);
npts = length(A.Frequency);
if prod(sza(3:end))==prod(newshape)
   B = A;
   B.ResponseData = reshape(A.ResponseData,[sza(1:2) npts newshape]);
else
   error('To RESHAPE the number of elements must not change.');
end