function out = simplify(a,varargin)
%SIMPLIFY Simplify representation of UFRD
%   B = SIMPLIFY(A) performs model-reduction-like techniques to
%   detect and eliminate redundant copies of uncertain elements.
%   Depending on the result, the CLASS of B may be lower than A
%   (which is UFRD).  Hence B may be an FRD or UFRD.
%   The AutoSimplify property of each uncertain element in
%   A determines what reduction methods are used.  After reduction,
%   any uncertain element which does not actually affect the result
%   is deleted from the representation.
%
%   B = SIMPLIFY(A,'full') overrides all uncertain element's
%   AutoSimplify property, and uses 'full' reduction techniques.
%
%   B = SIMPLIFY(A,'basic') overrides all uncertain element's
%   AutoSimplify property, and uses 'basic' reduction techniques.
%
%   B = SIMPLIFY(A,'class') does no reduction.  However, any uncertain
%   elements in A with 0 occurances are eliminated, and the class of
%   B may be lower than class of A.

% Copyright 2004 The MathWorks, Inc.

out = a;
tmp = umat(simplify(a.ResponseData,varargin{:}));
szt = size(tmp);
if length(szt)==2 || szt(3)==1
   tmp = repmat(tmp,[1 1 length(a.Frequency) ones(1,length(szt)-3)]);
end
out.ResponseData =tmp;

% XXXCONSOLIDATE - all code below is common to all 3 versions
depromotechoices = {'nodepromote'};
vin = nargin-1;

nodepflg = 0;
for i=1:vin
   idx = find(strncmpi(depromotechoices,varargin{i},length(varargin{i})));
   if length(idx)==1
      nodepflg = 1;
   end
end

if nodepflg==0
   out = depromote(out);
end
