%VERTCAT  Vertical concatenation of UFRD objects.
%  
%SYS = VERTCAT(SYS1,SYS2,...) performs a concatenation operation of 
%  SYS = [SYS1; SYS2; ...].
%
%See also: HORZCAT

% Copyright 2003-2004 The MathWorks, Inc.
function out = vertcat(varargin)

if nargin==1
   out = varargin{1};
elseif nargin==2
   A = varargin{1};
   B = varargin{2};
   if ~isa(A,'ufrd') 
      A = lift2ufrd(A,B.Frequency,B.Units);
   elseif ~isa(B,'ufrd')
      B = lift2ufrd(B,A.Frequency,A.Units);
   else   
   % Check frequency vectors
      try
         % give priority to rad/s if both FRD's
         [freq,units] = freqcheck(A.Frequency,A.Units,B.Frequency,B.Units);
      catch einfo
         throw(einfo);
      end
   end
   sza = size(A);
   szb = size(B);
   
   if any(sza==0)
         out = B;
   elseif any(szb==0)
         out = A;
   elseif sza(2)==szb(2)
      if ~isexdok(sza(4:end),szb(4:end))
            error('Invalid extra dimensions')
      end
      try
            out = binop(A,B,'vertcat');
      catch einfo
           throw(einfo);
      end
   else
      msg = 'All rows in the bracketed expression must have the same\nnumber of columns.';
      msg = 'In [SYS1 ; SYS2], SYS1 and SYS2 must have the same number of inputs.';
      msg = sprintf(msg);
      error(msg);
   end
else
   out = vertcat(vertcat(varargin{1},varargin{2}),varargin{3:end});
end



