function [out,pflag,errstring] = pvget(p,property,varargin)

% Copyright 2006 The MathWorks, Inc.

pflag = 1;
errstring = '';
switch property
case {'Name','NominalValue','AutoSimplify'}
   out = pvget(p.atom,property);
case 'atom'
   out = p.atom;
case 'Traits'
   [ny,nu] = size(pvget(p.atom,'NominalValue'));
   out.scalar = (ny==1)&(nu==1);
   out.linear = 1;
   out.zizo = 1;
   out.dynamic = 1;
   out.uncertain = 1;
case 'Type'
   out = p.Type;
case 'Bound'
   out = p.Bound;
case 'SampleStateDim'
   out = p.SampleStateDim;
case 'SampleStateNaming'
   out = p.SampleStateNaming;
case 'PropNames'
   aGS = pvget(p.atom,'PropNames');
%    TypeDepGStr = {'Type';'Bound';'SampleStateDim';'SampleStateNaming'};
%    TypeDepSStr = {'Type';'Bound';'SampleStateDim';'ioSize';'SampleStateNaming'};
%    TypeDepSDesc = {'[ GainBounded | PositiveReal ]';'scalar DOUBLE';...
%          'Positive Integer';'1x2 DOUBLE';'[ on | off ]'};
%    TypeDepSStr = {'Type';'Bound';'SampleStateDim';'SampleStateNaming'};
%    TypeDepSDesc = {'[ GainBounded | PositiveReal ]';'scalar DOUBLE';...
%          'Positive Integer';'[ on | off ]'};
   TypeDepGStr = {'Type';'Bound';'SampleStateDim'};
   TypeDepSStr = {'Type';'Bound';'SampleStateDim'};
   TypeDepSDesc = {'[ GainBounded | PositiveReal ]';'scalar DOUBLE';...
         'Positive Integer'};
   nidx = find(strcmpi(aGS.SPropNames,'NominalValue'));
   asins = find(strcmpi(aGS.SPropNames,'AutoSimplify'));
   ns = length(aGS.SPropNames);
   asing = find(strcmpi(aGS.GPropNames,'AutoSimplify'));
   ng = length(aGS.GPropNames);
   if ~isempty(nidx) | ~isempty(asins)
      keep = 1:ns;
      keep([nidx asins]) = []; % Get rid of NominalValue and AsutSimplify
      out.SPropNames = [aGS.SPropNames(keep);TypeDepSStr;aGS.SPropNames(asins)];
      out.SPropDescription = [aGS.SPropDescription(keep);TypeDepSDesc;aGS.SPropDescription(asins)];
   else
      out.SPropNames = [aGS.SPropNames;TypeDepSStr];
      out.SPropDescription = [aGS.SPropDescription;TypeDepSDesc];
   end
   if length(asing)>0
      use = 1:ng;
      use(asing) = [];
      out.GPropNames = [aGS.GPropNames(use);TypeDepGStr;aGS.GPropNames(asing)];
   else
      out.GPropNames = [aGS.GPropNames;TypeDepGStr];
   end
case 'Instance'
   if nargin==6
      % pvget(ATOM,'Instance',NX,TS,Npts,Bw)
      Nx = varargin{1};
      Ts = varargin{2};
      Npts = varargin{3};
      Bw = varargin{4};
   elseif nargin==5
      % pvget(ATOM,'Instance',NX,TS,Npts)
      Nx = varargin{1};
      Ts = varargin{2};
      Npts = varargin{3};
      Bw = [];
   elseif nargin==4
      Nx = varargin{1};
      Ts = varargin{2};
      Npts = [];
      Bw = [];
   elseif nargin==3
      Nx = varargin{1};
      Ts = [];
      Npts = [];
      Bw = [];
   elseif nargin==2
      Nx = [];
      Ts = [];
      Npts = [];
      Bw = [];
   else
      % If/Else will probably never get here, based on defaults
      error('Instance requires Nx and Ts as arguments');
   end
   
   if isempty(Nx)
      % max statedim==10;
      %NxMax = 10;
      %Nx = ceil(rand(1)*NxMax);
      Nx = p.SampleStateDim;
   end
   if isempty(Ts)
      Ts = 0;
   end
   if isempty(Npts)
      Npts = 1;
   end
   tol = 0.001;
   szp = size(pvget(p.atom,'NominalValue'));
   if Ts>0 || Ts==-1
      tmp = drssnb(Bw,Ts,Nx,szp(1),szp(2),Npts);
      tmp.Ts = Ts;
   elseif Ts==0
      tmp = rssnb(Bw,Nx,szp(1),szp(2),Npts);
   else
      error('Sample-Time must be -1, 0 or positive.');
   end
   switch p.Type
   case 'GainBounded'
      rdistr = 1-rand(Npts,1).^2;
      sf = reshape(p.Bound*rdistr,[1 1 Npts]); % 1x1xNpts DOUBLE
      out = sf*tmp;
   case 'PositiveReal'
      sf = reshape(rand(Npts,1),[1 1 Npts]); % 1x1xNpts DOUBLE
      N = pvget(p,'Normalizer');
      out = lft(sf*tmp,N.S);
   otherwise
      error('Invalid type');
   end
   tmpStateName = cell(Nx,1);
   for i=1:Nx
      tmpStateName{i} = [pvget(p.atom,'Name') '_ultidyn' int2str(i)];
   end
   if isequal(p.SampleStateNaming,'on')
      out.StateName = tmpStateName;
   end
case 'Normalizer'
   szp = size(pvget(p.atom,'NominalValue'));
   switch p.Type
   case 'GainBounded'
      snb = sqrt(p.Bound);
      snbi = 1/snb;
      S = [zeros(szp(2),szp(1)) snb*eye(szp(2));snb*eye(szp(1)) zeros(szp(1),szp(2))];
      T = [zeros(szp(2),szp(1)) snbi*eye(szp(2));snbi*eye(szp(1)) zeros(szp(1),szp(2))];
   case 'PositiveReal'
      alpha = 1/(2*abs(p.Bound) + 1);
      S = [-1 sqrt(2/alpha);-sqrt(2/alpha) (alpha*p.Bound+1)/alpha];
      SNV = scalarnomval(p.Bound);
      T = inv(S);
      T = fliplr(flipud(T));
      S = kron(S,eye(szp(1)));
      T = kron(T,eye(szp(1)));
   otherwise
      error('Invalid Type')
   end
   out.S = S;
   out.T = T;
   out.Tarb = [];
case 'IntType'
   out = 2;
case 'BlockSize'
   out = size(pvget(p.atom,'NominalValue'));
case 'SSNominalValue'
   % pvget(ATOM,'SSNominalValue',TS)
   [ny,nu] = size(pvget(p.atom,'NominalValue'));
   nx = ny + nu + 1;  % choice for Destabilizing Perts (dyad)
   switch p.Type
   case 'GainBounded'
      sqbnd = sqrt(p.Bound);
      B = 1e-3*sqbnd*rand(nx,nu);
      C = 1e-3*sqbnd*rand(ny,nx);
      D = zeros(ny,nu);
      if varargin{1}==0 % Continuous
         A = -eye(nx);
         out = ss(A,B,C,D);
      else
         A = zeros(nx);
         out = ss(A,B,C,D,varargin{1});
      end
   case 'PositiveReal'
      N = pvget(p,'Normalizer');
      %SNV = scalarnomval(p.Bound);
      B = 1e-3*rand(nx,nu);
      C = 1e-3*rand(ny,nx);
      %D = (SNV)*eye(ny);
      D = zeros(ny,nu);
      if varargin{1}==0 % Continuous
         A = -eye(nx);
         out = lft(ss(A,B,C,D),N.S);
      else
         A = zeros(nx);
         out = lft(ss(A,B,C,D,varargin{1}),N.S);
      end
   end
otherwise
   out = [];
   pflag = 0;
   errstring = ['Invalid ULTIDYN property: ''' property '''.'];
end
if nargout<=1 && pflag==0
   error(errstring);
end
