%PVSET  Set properties of ULTIDYN objects.
%  
%MAT = PVSET(MAT,'Property1',Value1,'Property2',Value2,...)
%sets the values of the properties with exact names 'Property1',
%'Property2',...
%  
%See also SET.

% Copyright 2003-2004 The MathWorks, Inc.
function [p,pflag,errstring] = pvset(p,property,value)

pflag = 1;
errstring = '';
switch property
case {'Name','AutoSimplify'}
   [A,pflag,errstring] = pvset(p.atom,property,value);
   if pflag==1
      p.atom = A;
   end
case 'Bound'
   if ndims(value)==2 & all(size(value)==[1 1]) & isa(value,'double')
      if isequal(p.Type,'PositiveReal')
         sza = size(p.atom);
         SNV = scalarnomval(value);
         p.atom = pvset(p.atom,'NominalValue',SNV*eye(sza(1)));
         p.Bound = value;
      elseif isequal(p.Type,'GainBounded')
         if value>0
            p.Bound = value;
         else
            pflag = 0;
            errstring = 'For GainBounded ULTIDYN, Bound must be positive.';
         end
      end
   else
      pflag = 0;
      errstring = 'Bound must be a scalar.';
   end
case 'SampleStateDim'
   if ndims(value)==2 & all(size(value)==[1 1]) & isa(value,'double') ...
    & value>0 & ceil(value)==floor(value)
      p.SampleStateDim = value;
   else
      pflag = 0;
      errstring = 'SampleStateDim should be a positive integer';
   end
case 'SampleStateNaming'
   if ndims(value)==2 & size(value,1)==1 & isa(value,'char') ...
    & (strcmpi(value,'on') | strcmpi(value,'off'))
      p.SampleStateNaming = lower(value);
   else
      pflag = 0;
      errstring = 'SampleStateNaming should be ''on'' or ''off''';
   end
case 'Type'
   if isa(value,'char') & size(value,1)==1 & ndims(value)==2
      choices = {'GainBounded';'PositiveReal'};
      idx = find(strncmpi(choices,value,length(value)));
      if length(idx)==1
         if idx==1 & isequal(p.Type,'PositiveReal')
            p.Type = choices{1};
            p.Bound = 1;  % default for ULTIDYN
            sza = size(p.atom);
            p.atom = pvset(p.atom,'NominalValue',zeros(sza));
         elseif idx==2 & isequal(p.Type,'GainBounded');
            if size(p,1)~=size(p,2)
               pflag = 0;
               errstring = 'PositiveReal ULTIDYN must be square';
            else
               p.Type = choices{2};
               p.Bound = 0;
               SNV = scalarnomval(p.Bound);
               sza = size(p.atom);
               p.atom = pvset(p.atom,'NominalValue',SNV*eye(sza(1)));
            end
         end
      else
         pflag = 0;
         errstring = 'Invalid TYPE, TYPE must be ''GainBounded'' or ''PositiveReal''';
      end
   else
      pflag = 0;
      errstring = 'TYPE should be a string';
   end
otherwise
   pflag = 0;
   errstring = ['Invalid ULTIDYN property: ' property];
end
if nargout<=1 && pflag==0
   error(errstring);
end
