%BLK2IOCH Converts an uncertainty to an input/output block.
%
%[OUT,IDFLG] = BLK2IOCH(MAT,BLKID,TB) opens up a LFT channel, moving the 
%exposed inputs/outputs into the top (or bottom) of the I/O block. MAT is
%an uncertain matrix, BLKID is the name or numerical index of the uncertain 
%that is present in MAT and TB is a string that determines if the block 
%should be moved to the top, 't' or the bottom, 'b' of the inputs and outputs
%of OUT. OUT is an uncertain matrix with BLKID at either the top, if
%TB='t', or bottom, TB='b', of the inputs/outputs of the original uncertain
%matrix MAT. IDFLG is 1 if the BLKID was an uncertain atom in MAT otherwise
%it is 0.
%
%  Always returns a UMAT

% Copyright 2003-2004 The MathWorks, Inc.

function [out,idflg] = blk2ioch(mat,blkid,tb,litreq)

if nargin<3
   estr = 'Invalid Syntax.';
   error(estr);
elseif nargin==3
   litreq = 'literalZERO';
end

switch litreq
   case 'literalZERO'
      if isa(blkid,'char')
         mat = setlitr(mat,{blkid},0);
      else
         mat = setlitr(mat,blkid,0);
      end
   case 'literalONE'
      if isa(blkid,'char')
         mat = setlitr(mat,{blkid},1);
      else
         mat = setlitr(mat,blkid,1);
      end
end

[outtmp,idflg] = blk2ioch(mat.NDlftData,blkid,tb);
% since NDLFT/BLK2IOCH may return a DOUBLE, need to
% promote outtmp back to NDLFT
outtmp = ndlft(outtmp);
out = umat(outtmp);