%BLKDIAG  Group uncertain matrices by appending their inputs and outputs.
%  
%MAT = BLKDIAG(MAT1,MAT2, ...) produces the aggregate matrix
%   
%                 [ MAT1  0       ]
%           MAT = [  0   MAT2     ]
%                 [           .   ]
%                 [             . ]
%  
%BLKDIAG concatenates the input and output vectors of the uncertain
%matrices MAT1, MAT2,... to produce the resulting matrix MAT.
%  
%If MAT1,MAT2,... are arrays of uncertain matrices, BLKDIAG returns an 
%uncertain array MAT of the same size where 
%  MAT(:,:,k) = BLKDIAG(MAT1(:,:,k),MAT2(:,:,k),...) .
%  
%See also SERIES, PARALLEL, FEEDBACK.
function out = blkdiag(varargin)

% Copyright 2003-2004 The MathWorks, Inc.

if nargin==1
   out = varargin{1};
elseif nargin==2
   if isa(varargin{1},'umat') & isa(varargin{2},'umat')
      sza = size(varargin{1});
      szb = size(varargin{2});
      if ~isexdok(sza(3:end),szb(3:end))
         error('Array dimensions are incompatible.')
      end      
      if sza(1)==0 || sza(2)==0
         out = varargin{2};      
      elseif szb(1)==0 || szb(2)==0
         out = varargin{1};      
      else
         try
            out = binop(varargin{1},varargin{2},'blkdiag');
         catch
            rethrow(lasterror)
         end
      end
   else
      out = blkdiag(umat(varargin{1}),umat(varargin{2}));
   end
elseif nargin>2
   out = blkdiag(blkdiag(umat(varargin{1}),umat(varargin{2})),varargin{3:end});
end