function sys = cat(dim,varargin)
%CAT  Concatenation of uncertain matrices.
%
%UMAT = CAT(DIM,UMAT1,UMAT2,...) concatenates the uncertain
%matrices along the dimension DIM.  The values DIM=1,2 
%correspond to the output and input dimensions, and the values 
%DIM=3,4,... correspond to the UMAT array dimensions 1,2,...
%
%For example,
%   * CAT(1,UMAT1,UMAT2) is equivalent to [UMAT1 ; UMAT2]
%   * CAT(2,UMAT1,UMAT2) is equivalent to [UMAT1 , UMAT2]
%   * CAT(4,UMAT1,UMAT2) is equivalent to STACK(2,UMAT1,UMAT2).
%
%See also HORZCAT, VERTCAT, STACK, APPEND.

% Copyright 2003-2004 The MathWorks, Inc.

switch dim
case 1
   sys = vertcat(varargin{:});
case 2
   sys = horzcat(varargin{:});
otherwise
   sys = stack(dim-2,varargin{:});
end
