%HORZCAT  Horizontal concatenation of UMAT objects.
%  
%MAT = HORZCAT(MAT1,MAT2,...) performs a concatenation operation of 
%  MAT = [MAT1 , MAT2, ...].
%
%See also: VERTCAT, UMAT

function out = horzcat(varargin)
%

% Copyright 2003-2004 The MathWorks, Inc.

if nargin==1
   out = varargin{1};
elseif nargin==2
   if isa(varargin{1},'umat') & isa(varargin{2},'umat')
      sza = size(varargin{1});
      szb = size(varargin{2});
      if ~isexdok(sza(3:end),szb(3:end))
         error('Array dimensions are incompatible.')
      end
      if sza(2)==0
         out = varargin{2};
         if sza(1)>0 & sza(1)~=szb(1)
            warning('Concatenation involves an empty array with incorrect number of rows.');
         end
      elseif szb(2)==0
         out = varargin{1};
         if szb(1)>0 & szb(1)~=sza(1)
            warning('Concatenation involves an empty array with incorrect number of rows.');
         end
      elseif sza(1)==szb(1)
         try
            out = binop(varargin{1},varargin{2},'horzcat');
         catch
            rethrow(lasterror)
         end
      else
         msg = 'All matrices on a row in the bracketed expression must have the\nsame number of rows.';
         msg = sprintf(msg);
         error(msg);
      end
   else
      out = horzcat(umat(varargin{1}),umat(varargin{2}));
   end
elseif nargin>2
   out = horzcat(horzcat(umat(varargin{1}),umat(varargin{2})),varargin{3:end});
end
