function B = imp2exp(A,yidx,uidx)
%IMP2EXP Converts an implicit linear relationship to explicit
%   input-output relation.
%

% Copyright 2003-2004 The MathWorks, Inc.
%   B = IMP2EXP(A,YIDX,UIDX) transforms a linear constraint on
%   variables Y and U of the form A(:,[YIDX;UIDX])*[Y;U] = 0 into
%   an explicit input/output relationship Y = B*U.  The index
%   vectors YIDX and UIDX refer to the columns (inputs) of A as
%   referenced by the explicit relationship for B.
%
%   See also INV.

if isa(yidx,'double') && isa(uidx,'double') && ...
      ndims(yidx)==2 && ndims(uidx)==2 && ...
      min(size(yidx))==1 && min(size(uidx))==1 && ...
      all(ceil(yidx)==floor(yidx)) && all(ceil(uidx)==floor(uidx)) && ...
      min(yidx)>0 && min(uidx)>0 && ...
      length(yidx)>0 && length(uidx)>0
   yidx = yidx(:);
   uidx = uidx(:);
   if isempty(intersect(yidx,uidx))
      szA = size(A);
      if length(uidx)==1 || min(diff(sort(uidx)))>0
         unyidx = unique(yidx);
         missingyidx = 1:szA(2);  % wGiven U, assume you can solve for anything not in U
         missingyidx([unyidx;uidx]) = [];
         yidxext = [yidx;missingyidx];
         % Operate on ExtendedY
         B = umat(imp2exp(A.NDlftData,yidxext,uidx));
         NY = length(yidx);
         % Select appropriate output channels
         L.type = '()';
         L.subs = {1:NY ':'};
         B = subsref(B,L);
      else
         error('Elements of UIDX must be unique.');
      end
   else
      error('Indices UIDX and YIDX must not have any elements in common.');
   end
else
   error('UIDX and YIDX must be vectors of positive integers.');
end
