%INV  Inverse UMAT object.
% 
%IMAT = INV(MAT) computes the inverse matrix IMAT such that
% 
%    y = MAT * u   <---->   u = IMAT * y 
% 
%The UMAT matrix MAT must have the same number of inputs and
%outputs.
% 
%For arrays of UMAT matrices, INV is performed on each individual
%matrix.
% 
%See also MLDIVIDE, MRDIVIDE, UMAT.

% Copyright 2003-2004 The MathWorks, Inc.

function out = inv(a)

%XXXOK, XXXCOMMENT
[rp,cp,nblk,iosize,blkiosize,szmd] = umatinfo(a);
if iosize(1)==iosize(2)
   anom = pvget(a,'NominalValue');
   for i=1:prod(szmd(3:end))
      [l,u,p] = lu(anom(:,:,i));
      if rcond(u)<10*eps,
         error('Cannot invert uncertain matrix with singular NominalValue.');
      end
   end
   % cheat with mrdivide for now...
   out = binop(umat(eye(iosize(1))),a,'mrdivide');
else
   error('Matrix must be square.')
end