%IPERMUTE  Inverse permute array of UMAT objects.
%  A = IPERMUTE(B,ORDER) is the inverse of permute. IPERMUTE
%  rearranges the dimensions of B so that PERMUTE(A,ORDER) will
%  produce B.  The array produced has the same values of A but the
%  order of the subscripts needed to access any particular element
%  are rearranged as specified by ORDER.  The elements of ORDER
%  must be a rearrangement of the numbers from 1 to N.
% 
% PERMUTE and IPERMUTE are a generalization of transpose (.') 
% for UMAT arrays.
% 
% Example
%  a = randumat(2,2);
%  aa = a*rand(2,2,3,4,5);
%  b = permute(aa,[3 2 1]);
%  c = ipermute(b,[3 2 1]); % aa and c are equal
% 
%  See also IPERMUTE.

function m = permute(m, index)
% Copyright 2003-2004 The MathWorks, Inc.

index = index(:).';
szm = size(m);
if (length(index)>=length(szm)-2 || (length(szm)==4 && szm(4)==1)) && ...
      isa(index,'double') && isequal(sort(index),1:length(index))
   modindex = [index];
   try
      m.NDlftData = permute(m.NDlftData,modindex);
   catch fred
      throw(fred);
   end
else
    msg = 'ORDER contains an invalid permutation index.';
    error(msg);
end


