%PERMUTE  Permute array of UMAT objects.
% B = PERMUTE(A,ORDER) rearranges the array dimensions of A so that they
% are in the order specified by the vector ORDER.  The input/output
% dimensions of A are unchanged.  The resultant array B has the same
% values of A but the order of the subscripts needed to access any
% particular element are rearranged as specified by ORDER.  The elements
% of ORDER must be a rearrangement of the numbers from 1 to N.
% 
% PERMUTE and IPERMUTE are a generalization of transpose (.') 
% for UMAT arrays.
% 
% Example
%  a = randumat(3,2);
%  aa = a*rand(2,2,3,4,5);
%  size(permute(aa,[1 2 5 4 3])) %now 2 input/2 output 5-by-4-by-3 UMAT array.
% 
%  See also IPERMUTE.
function m = permute(m, index)

% Copyright 2003-2004 The MathWorks, Inc.
index = index(:).';
szm = size(m);
if (length(index)>=length(szm)-2 || (length(szm)==4 && szm(4)==1)) && ...
      isa(index,'double') && isequal(sort(index),1:length(index))
   % do not allow change OUTPUT/INPUT
   modindex = [index];
   try
      m.NDlftData = permute(m.NDlftData,modindex);
   catch fred
     throw(fred);
   end
else
    msg = 'ORDER contains an invalid permutation index.';
    error(msg);
end


