function b = repmat(a,varargin)
% REPMAT Replicate and tile an array of UMAT objects.
%
%  B = REPMAT(A,M,N) creates a large matrix B consisting of an M-by-N
%  tiling of copies of A.
% 
%  B = REPMAT(A,[M N]) accomplishes the same result as REPMAT(A,M,N).
% 
%  B = REPMAT(A,[M N P ...]) tiles the array A to produce a
%  M-by-N-by-P-by-... block array.  A can be N-D.
% 
%  REPMAT(A,M,N) when A is a scalar is commonly used to produce
%  an M-by-N matrix filled with A's value.
% 
%  See also BLKDIAG.

% Copyright 2003-2004 The MathWorks, Inc.
b = a;
b.NDlftData = repmat(a.NDlftData,varargin{:});
