function B = reshape(A,varargin)
%RESHAPE  Reshape UMAT array
%  
%   B = RESHAPE(A,S1,S2,...,Sk) reshapes the UMAT array A
%   into a S1-by-S2-by...-Sk UMAT array.  There must
%   be S1*S2*...*Sk array elements to begin with.
%   
%   B = RESHAPE(A,[S1 S2 ... Sk]) is the same thing.
%  
%   See also SIZE, NDIMS.

% Copyright 2003-2004 The MathWorks, Inc.

newshape = [varargin{:}];

if length(newshape)<2
   error('At least two dimensions must be specified.');
end

sza = size(A);
if ~all(newshape(1:2)==sza(1:2))
   error('Cannot RESHAPE along Row/Column dimensions in a UMAT.');
elseif ~prod(sza)==prod(newshape)
   error('To RESHAPE, the number of elements must not change.');
else
   B = A;
   B.NDlftData = reshape(A.NDlftData,newshape);
end