function out = simplify(a,varargin)
%SIMPLIFY Simplify representation of UMAT
%   B = SIMPLIFY(A) performs model-reduction-like techniques to
%   detect and eliminate redundant copies of uncertain elements.
%   Depending on the result, the CLASS of B may be lower than A
%   (which is UMAT).  Hence B may be a DOUBLE, ATOM or UMAT.
%   The AutoSimplify property of each uncertain element in
%   A determines what reduction methods are used.  After reduction,
%   any uncertain element which does not actually affect the result
%   is deleted from the representation.
%
%   B = SIMPLIFY(A,'full') overrides all uncertain element's
%   AutoSimplify property, and uses 'full' reduction techniques.
%
%   B = SIMPLIFY(A,'basic') overrides all uncertain element's
%   AutoSimplify property, and uses 'basic' reduction techniques.
%
%   B = SIMPLIFY(A,'class') does no reduction.  However, any uncertain
%   elements in A with 0 occurances are eliminated, and the class of
%   B may be lower than class of A.

% Copyright 2003-2004 The MathWorks, Inc.

out = umat(simplify(a.NDlftData,varargin{:}));

% XXXCONSOLIDATE - all code below is common to all 3 versions
depromotechoices = {'nodepromote'};
vin = nargin-1;

nodepflg = 0;
for i=1:vin
   idx = find(strncmpi(depromotechoices,varargin{i},length(varargin{i})));
   if length(idx)==1
      nodepflg = 1;
   end
end

if nodepflg==0
   out = depromote(out);
end
