function [varargout] = size(m,arg2)
%  SIZE   Size of an UMAT object.  
%     D = SIZE(X), for M-by-N uncertain matrix X, returns the 
%     two-element row vector D = [M, N] containing the number 
%     of rows and columns in the matrix.  For N-D arrays, SIZE(X) 
%     returns a 1-by-N vector of dimension lengths.  Trailing 
%     singleton dimensions are ignored.
%  
%     [M,N] = SIZE(X) for uncertain matrix X, returns the number 
%     of rows and columns in X as separate output variables. 
%     
%     [M1,M2,M3,...,MN] = SIZE(X) returns the sizes of the first N 
%     dimensions of array X.  If the number of output arguments N does
%     not equal NDIMS(X), then for:
%  
%     N > NDIMS(X), size returns ones in the "extra" variables,
%                   i.e., outputs NDIMS(X)+1 through N.
%     N < NDIMS(X), MN contains the product of the sizes of the 
%                   remaining dimensions, i.e., dimensions N+1 through
%                   NDIMS(X).
%  
%     M = SIZE(X,DIM) returns the length of the dimension specified
%     by the scalar DIM.  For example, SIZE(X,1) returns the number
%     of rows.
%  
%     See also length, ndims

% Copyright 2003-2004 The MathWorks, Inc.
%XXXOK, XXXCOMMENT, conforms to TMW behavior, but may be too slow.
out = size(m.NDlftData);
if nargin==1
   arg2 = nan;
end
varargout = csize(out,arg2,nargin,nargout);