function G = tf(num,den,TS)
%TF Creation of an uncertain transfer function.
%   GUSS = TF(NUM,DEN,TS) converts a numerator, NUM, denominator, DEN,
%   representing a single-input/single-output uncertain transfer function
%   into an uncertain system, GUSS.   NUM and/or DEN must be of class
%   UMAT.  The default value for TS is 0.
%
%   See also:  TF, USS.

% Copyright 2003-2004 The MathWorks, Inc.

% XXXMINOR, useful, needs work, but how much is enough?
% First cut at TF for numerators and/or denominators which are UMAT's
% needs additional work/decisions XXX

if nargin==1
   den = 1;
   TS = 0;
elseif nargin==2
   TS = 0;
end

if isa(num,'atom')
   num = umat(num);
end
if isa(den,'atom')
   den = umat(den);
end
szn = size(num);
szd = size(den);
if ndims(szn)==2 & szn(1)==1 & ndims(szd)==2 & szd(1)==1
   [a,b,c,d] = LOCALnd2ssms(num,den);
   %G = ss(a,b,c,d);
   if nargin == 2
       G = uss(a,b,c,d);
   else
       G = uss(a,b,c,d,TS);
   end
   if isa(G,'ss')
      if length(den)>1
         rd = roots(den);
         if max(real(rd))<-10*(length(den)+1)*eps
            G = balreal(G); % or sysbal...
         else
            G = ssbal(G);
         end
      end
   elseif isa(G,'uss')
      % SSBAL still needs work, for picking the midfrequency, XXXX
      G = ssbal(G,[],1,1);
   end
else
   error('NUM and DEN need to be ROW vectors')
end

% function [a,b,c,d] = LOCALnd2ssms(num,den)

function [a,b,c,d] = LOCALnd2ssms(num,den)

sznum = size(num);
nrn = sznum(1);
ncn = sznum(2);
nlen = ncn;
if nrn > 1
   if ncn > 1
      disp('NUM should be a vector');
      return
   else
      num = num.';
      nlen = nrn;
   end
end
szden = size(den);
nrd = szden(1);
ncd = szden(2);
dlen = ncd;
if nrd > 1
   if ncd > 1
      disp('DEN should be a vector');
      return
   else
      den = den.';
      dlen = nrd;
   end
end

    if isa(den,'double')
        dstart = min(find(abs(den)>0));
    else
        dstart = 1;
    end
    %den is a UMAT, why doesn't single integer index work?
    L.type = '()';
    L.subs = {dstart:dlen};
    den = subsref(den,L);
    dlen = length(den);
    if isa(num,'double')
        nstart = min(find(abs(num)>0));
    else
        nstart = 1;
    end
    L.type = '()';
    L.subs = {nstart:nlen};
    num = subsref(num,L);
    nlen = length(num);

    if nlen > dlen
        disp('NUM order should be <= DEN order')
        return
    elseif dlen == 1
        a = [];
        d = num/den;
        b = zeros(0,size(d,2));
        c = zeros(size(d,1),0);
    else
        L.type = '()';
        L.subs = {1};
        M.subs = {2:dlen}; 
        M.type = '()';
        num = subsref(den,L)\num;
        den = [1 subsref(den,L)\subsref(den,M)];
        if nlen == dlen
            %cd = [fliplr(num(2:dlen)) 0] + num(1)/den(1)*[-fliplr(den(2:dlen)) 1];
            M.subs = {1};
            d = subsref(num,M)/subsref(den,L);
            c = num - d*den;
            M.subs = {2:dlen};
            c = fliplr(subsref(c,M));
        else
            d = 0;
            c = fliplr([zeros(1,dlen-1-nlen) num]);
        end
        M.subs = {2:dlen};
        a = [zeros(dlen-2,1) eye(dlen-2);-fliplr(subsref(den,M))];
        b = [zeros(dlen-2,1);1];
    end

