function y = uss(varargin)

% Copyright 2004 The MathWorks, Inc.

nin = nargin;
nvp = 0;
i = 1;
while nin>i && isa(varargin{nin-i},'char')
   nvp = nvp + 1;
   i = i + 2;
end
ain = nin - 2*nvp;
if nvp==0
   % do the calls as laid out in the question/response
   if ain==1  % USS(UMAT)
      x = varargin{1};
      sx = size(x);
      Xs.UmatData = x;  % x must be a UMAT in this case
      Xs.StateName = cell(0,1);
      Xs.Ts = utarbts;
      a4 = dynamicsys(sx(1),sx(2));
      y = uss(Xs,a4);
   elseif ain==2 % USS(UMAT,CELL), USS(UMAT,CHAR), USS(UMAT,DOUBLE)
      x = varargin{1};
      arg2 = varargin{2};
      a3 = utarbts; 
      if isa(x,'umat')
         szm = size(x);
         switch class(arg2) 
         case {'cell' 'char'}  % USS(UMAT,CELL), USS(UMAT,CHAR)
            if isa(arg2,'char')
               arg2 = cellstr(arg2);
            end
            if ~iscellstr(arg2)
               error('StateName should be a Cell-array of strings.');
            end
            a2 = arg2;
         case {'double'} % USS(UMAT,DOUBLE)
            if ndims(arg2)==2 && all(size(arg2)==[1 1]) && floor(arg2)==ceil(arg2) && arg2>0
               a2 = cell(arg2,1);
               for i=1:arg2
                  a2{i} = '';
               end
            else
               error('Second input must be a positive integer.');
            end
         end
         nx = size(a2,1);
         a4 = dynamicsys(szm(1)-nx,szm(2)-nx);
         Xs.UmatData = x;
         Xs.StateName = a2;
         Xs.Ts = a3;
         y = uss(Xs,a4);
      else
         error('First input argument must be a UMAT.');
      end
   elseif ain>=4
      ourcall = 0;
      cain = zeros(1,4);
      for i=1:4
         cain(i) = isa(varargin{i},'double') || isa(varargin{i},'umat') || ...
            isa(varargin{i},'ureal') || isa(varargin{i},'ultidyn') || ...
            isa(varargin{i},'ucomplex') || isa(varargin{i},'ucomplexm') || ...
            isa(varargin{i},'udyn');
      end
      if all(cain==1) % USS(D/UM/ATOM,D/UM/ATOM,D/UM/ATOM,D/UM/ATOM)
         A = umat(varargin{1});
         B = umat(varargin{2});
         C = umat(varargin{3});
         D = umat(varargin{4});
         [a1,a2,a4,ecode,estr] = LOCALfunctionSS(A,B,C,D);
         if ecode
            error(estr)
         end
         Xs.UmatData = a1;
         Xs.StateName = a2;
         Xs.Ts = 0;
      elseif isa(varargin{1},'umat') & ...
            isa(varargin{3},'double') & isa(varargin{4},'dynamicsys')
         % handles USS(UMAT,CELL,DOUBLE,DS)  and  USS(UMAT,CHAR,DOUBLE,DS)
         ourcall = 1;
         Xs.UmatData = varargin{1};
         if isa(varargin{2},'char')
            Xs.StateName = cellstr(varargin{2});
         elseif ~isa(varargin{2},'cell')
            error('StateName should be a Cell-array of strings');
         else
            Xs.StateName = varargin{2};
         end
         if ~iscellstr(Xs.StateName)
            error('StateName should be a Cell-array of strings');
         end
         Xs.Ts = varargin{3};
         a4 = varargin{4};
      else
         badarg = find(cain==0);
         error(['Argument #' int2str(badarg(1)) ' is incorrect class']);
      end
      if ain==5
         if ourcall==1
            error('No 5th argument allowed for USS(UMATDATA,STATENAME,TS,dynamicsys)');
         else
            arg5 = varargin{5};
            if isa(arg5,'double') & ndims(arg5)==2 & all(size(arg5)==[1 1]) & ...
                  isreal(arg5)
                % USS(D/UM/ATOM,D/UM/ATOM,D/UM/ATOM,D/UM/ATOM,TS)
               Xs.Ts = arg5;
            elseif isa(arg5,'ss') || isa(arg5,'tf') || isa(arg5,'zpk') || isa(arg5,'frd')
                % USS(D/UM/ATOM,D/UM/ATOM,D/UM/ATOM,D/UM/ATOM,TF/SS/ZPK/FRD)
               [a4,Xs.Ts] = lti2ds(arg5);
            elseif isa(arg5,'dynamicsys');
                % USS(D/UM/ATOM,D/UM/ATOM,D/UM/ATOM,D/UM/ATOM,DS)
               a4 = arg5;
            else
               error('5th argument should be SampleTime or REFSYS');
            end
         end
      end
      y = uss(Xs,a4);
   end
else
   y = uss(varargin{1:ain});
   y = set(y,varargin{ain+1:end});
end


function [umatdata,sn,dsfield,ecode,estr] = LOCALfunctionSS(a,b,c,d)
ecode = 0; estr = '';
sza = size(a);
szb = size(b);
szc = size(c);
szd = size(d);
if sza(1)~=sza(2)
   ecode = 1;
   estr = 'A-matrix needs to be square';
end
if sza(1)~=szb(1)
   ecode = 1;
   estr = 'A and B matrices need the same number of rows';
end
if sza(2)~=szc(2)
   ecode = 1;
   estr = 'A and C matrices need the same number of columns';
end
dtest = simplify(d,'class');
if isempty(dtest) || (isa(dtest,'double') ...
      && ndims(dtest)==2 && all(size(dtest)==[1 1]) && dtest==0)
   d = zeros(szc(1),szb(2));
end
szd = size(d);
if szd(1)~=szc(1)
   ecode = 1;
   estr = 'C and D matrices need the same number of rows';
end
if szd(2)~=szb(2)
   ecode = 1;
   estr = 'B and D matrices need the same number of columns';
end
if ~ecode
   umatdata = [a b;c d];
   dsfield = dynamicsys(szc(1),szb(2));
   sn = cell(sza(1),1);
   for i=1:sza(1)
      sn{i} = '';
   end
elseif isempty(a) && isempty(b) && isempty(c)
   umatdata = d;
   dsfield = dynamicsys(szd(1),szd(2));
   sn = cell(0,1);
   ecode = 0;
   estr = '';
else
   umatdata = [];
   dsfield = [];
   sn = [];
end
