%VERTCAT  Vertical concatenation of UMAT objects.
%  
%MAT = VERTCAT(MAT1,MAT2,...) performs a concatenation operation of 
%  MAT = [MAT1; MAT2; ...].
%
%See also: HORZCAT, UMAT
function out = vertcat(varargin)
%

% Copyright 2003-2004 The MathWorks, Inc.

if nargin==1
   out = varargin{1};
elseif nargin==2
   if isa(varargin{1},'umat') & isa(varargin{2},'umat')
      sza = size(varargin{1});
      szb = size(varargin{2});
      if ~isexdok(sza(3:end),szb(3:end))
         error('Array dimensions are incompatible.')
      end
      if sza(1)==0
         out = varargin{2};
         if sza(2)>0 & sza(2)~=szb(2)
            warning('Concatenation involves an empty array with incorrect number of columns.');
         end
      elseif szb(1)==0
         out = varargin{1};
         if szb(2)>0 & szb(2)~=sza(2)
            warning('Concatenation involves an empty array with incorrect number of columns.');
         end
      elseif sza(2)==szb(2)
         try
            out = binop(varargin{1},varargin{2},'vertcat');
         catch
            rethrow(lasterror)
         end
      else
         msg = 'All rows in the bracketed expression must have the same\nnumber of columns.';
         msg = sprintf(msg);
         error(msg);
      end
   else
      out = vertcat(umat(varargin{1}),umat(varargin{2}));
   end

elseif nargin>2
   out = vertcat(vertcat(umat(varargin{1}),umat(varargin{2})),varargin{3:end});
end