function [MAXNORM,WCUNCERTAINTY,INFO] =  wcnorm(m,varargin)
%WCNORM  Worst-case norm of an uncertain matrix.
%   [MAXNORM,MAXNORMUNC,INFO] = WCNORM(MAT) calculates the worst-case 
%   norm of an uncertain matrix.  "Worst-case" refers to maximum over
%   the uncertainty (and array dimensions), and "norm" is measured by
%   the maximum singular value of the matrix.
%
%   The outputs arguments for WCNORM are:
%     MAXNORM  Structure with the following fields:
%            LowerBound: Upper bound on worst-case gain. 
%            UpperBound: Lower bound on worst-case gain. 
%     MAXNORMUNC  Structure of worst-case uncertainty values. 
%     INFO  Structure with the following fields:
%           Sensitivity: Structure of percentages corresponding to MAXNORM 
%                        sensitivity to variations in each uncertainty
%                        level.
%            ArrayIndex: Array index (for arrays of uncertain matrices)
%                        where the maximum norm occurs.  Value is scalar,
%                        for single index use.
%
%   [MAXNORM,MAXNORMUNC,INFO] = WCNORM(MAT,OPTS) specifies options
%   described in OPTS.  See WCGOPT for more details on the options for
%   WCNORM. 
% 
%   See also NORM, SVD, WCGAIN, WCGOPT.

% Copyright 2003-2004 The MathWorks, Inc.
szm = size(m);
lszm = length(szm);
mg = ufrd(permute(m,[1 2 lszm+1 3:lszm]),1);
mg.Ts = 0;

[MAXNORM,WCUNCERTAINTY,INFO] =  wcgain(mg,varargin{:});

MAXNORM = rmfield(MAXNORM,'CriticalFrequency');
INFO.Sensitivity = INFO.Sensitivity;
INFO = rmfield(INFO,'Frequency');

