function avalue = normalized2actual(atom,nvalue)
%NORMALIZED2ACTUAL Converts the value for an atom in normalized
%  coordinates to the corresponding actual value.
%
%  AVALUE = NORMALIZED2ACTUAL(ATOM,NVALUE) converts the value for ATOM
%  in normalized coordinates, NVALUE, to the corresponding actual value.
%  If NVALUE is an N-d array of values, then AVALUE will be an array of 
%  the same dimension.
%
%  See also: actual2normalized

% Copyright 2004 The MathWorks, Inc.

% XXX or...avalue2nvalue?

if nargin<2
   error('avalue = normalized2actual(atom,nvalue)');
end
if ~isa(nvalue,'double') && isreal(nvalue)
   error('For UREAL, the normalized value must be a real double.');
end

szn = size(nvalue);
avalue = zeros(szn);
N=pvget(atom,'Normalizer');
CriticalDelta = N.CriticalDelta;
S11 = N.S(1,1);
S12 = N.S(1,2);
S21 = N.S(2,1);
S22 = N.S(2,2);

if CriticalDelta>0 && any( nvalue > CriticalDelta ) 
   error('Normalized Value beyond Critical Value.'); % XXX: Better error?
elseif CriticalDelta<0 && any( nvalue < CriticalDelta ) 
   error('Normalized Value beyond Critical Value.'); % XXX: Better error?
end

%avalue = lft(nvalue,N.S);    
avalue=S22+S12*S21*nvalue./(1-S11*nvalue);
