%PVGET  Get values of public UREAL object properties.
%  
%VALUES = PVGET(MAT) returns all public values in a cell
%array VALUES.
%  
%VALUE = PVGET(SYS,PROPERTY) returns the value of the
%single property with name PROPERTY.
%  
%See also GET.

% Copyright 2003-2004 The MathWorks, Inc.
function [out,pflag,errstring] = pvget(p,property,varargin)

pflag = 1;
errstring = '';
switch property
case {'Name','NominalValue','AutoSimplify'}
   out = pvget(p.atom,property);
case 'PropNames'
%    if isempty(p.Data)
%       TypeDepSStr = {};
%       TypeDepGStr = {};
%       TypeDepSDesc = {};
%    else
      nom = pvget(p.atom,'NominalValue');
      if nom~=0
            TypeDepSStr = {'Mode';'Range';'PlusMinus';'Percentage'};
            TypeDepGStr = {'Mode';'Range';'PlusMinus';'Percentage'};
            TypeDepSDesc = {'Range | PlusMinus | Percentage';'1x2 DOUBLE';...
                    '1x2 or scalar DOUBLE';'1x2 or scalar DOUBLE'};
      else
            TypeDepSStr = {'Mode';'Range';'PlusMinus';'Percentage'};
            TypeDepGStr = {'Mode';'Range';'PlusMinus';'Percentage'};
            TypeDepSDesc = {'Range | PlusMinus';'1x2 DOUBLE';...
                    '1x2 or scalar DOUBLE';'Not settable since Nominal==0'};
      end
%    end
   aGS = pvget(p.atom,'PropNames');
   nidx = find(strcmpi(aGS.SPropNames,'NominalValue'));
   if length(nidx)>0
      aGS.SPropDescription{nidx} = '1x1 real DOUBLE';
   end
   ng = length(aGS.GPropNames);
   ns = length(aGS.SPropNames);
   asing = find(strcmpi(aGS.GPropNames,'AutoSimplify'));
   asins = find(strcmpi(aGS.SPropNames,'AutoSimplify'));
   if length(asing)>0
      use = 1:ng;
      use(asing) = [];
      out.GPropNames = [aGS.GPropNames(use);TypeDepGStr;aGS.GPropNames(asing)];
%    else
%       % Never does this, since currently we have AutoSimplify
%       out.GPropNames = [aGS.GPropNames;TypeDepGStr];
   end
   if length(asins)>0
      use = 1:ns;
      use(asins) = [];
      out.SPropNames = [aGS.SPropNames(use);TypeDepSStr;aGS.SPropNames(asins)];
      out.SPropDescription = [aGS.SPropDescription(use);TypeDepSDesc;aGS.SPropDescription(asins)];
%    else
%       % Never does this, since currently we have AutoSimplify
%       out.SPropNames = [aGS.SPropNames;TypeDepSStr];
%       out.SPropDescription = [aGS.SPropDescription;TypeDepSDesc];
   end
case 'atom'
   out = p.atom;
case 'Data'
   out = p.Data;
case 'Instance'
%    if ~isempty(p.Data)
      if nargin==2
         Npts = 1;
      else
         Npts = varargin{1};
      end
      N = pvget(p,'SNormalizer');
      rv = (rand(Npts,1)-.5)*2;
      out = N(3) + rv*N(2)./(1-rv*N(1));
%    else
%       out = [];
%    end
case 'Range'
%    if isempty(p.Data)
%       out = zeros(0,2);
%    else
      out = p.Data([1 2]);
%    end
case 'PlusMinus'
%    if isempty(p.Data)
%       out = zeros(0,2);
%    else
      out = p.Data([1 2]) - pvget(p.atom,'NominalValue');
%    end
case 'Percentage'
%    if isempty(p.Data)
%       out = zeros(0,2);
%    else
      nom = pvget(p.atom,'NominalValue');
      if nom==0
         out = [-inf inf];
      else
         out = 100*(p.Data(1:2) - nom)/abs(nom);
      end
%    end
case 'Normalizer'
   lnr = [p.Data(1) pvget(p.atom,'NominalValue') p.Data(2)];
   % starp(delta_normal,S) = delta_actual
   rad = (lnr(3)-lnr(1))/2;
   cen = (lnr(3)+lnr(1))/2;
   t12 = sqrt( (lnr(3)-lnr(2)) * (lnr(2)-lnr(1))/rad );
   S = [(cen-lnr(2))/rad t12;t12 lnr(2)];
   out.S = S;
   l1 = lnr(1);
   n1 = lnr(2);
   r1 = lnr(3);
   c1 = cen;
   R1 = rad;
   % Change the defined lnr to have the Nominal value at the Center. we know
   % this is a "good" range/nominal is the original range/nominal was "bad",
   % i.e. S is singular.
   if r1*l1==n1*c1 & n1~=c1  % bad range/nominal
      % note, if N==C, then it cannot be that R*L==N*C.  But, for
      % very small ranges, this numerically possible, since the mismatch
      % will be to 2nd order.
      % Construct the new valid transformation for a singular S.
      betatild = n1/(c1-n1);   
      T1S2 = zeros(2,2);
      T1S2(1,1) = -1;
      T1S2(1,2) = sign(betatild)*sqrt(abs(betatild));
      T1S2(2,1) = sqrt(abs(betatild));
      T1S2(2,2) = 1;
      T1S2 = (R1/c1)*T1S2;
      % warning(['Bad range/nominal data for real parameter' ]);
      Sarb = [0 sqrt(R1);sqrt(R1) c1];
      Tarb = flipud(fliplr(inv(Sarb)));
      out.T = T1S2;
      out.Tarb = Tarb;
   else
      T = flipud(fliplr(inv(S)));
      out.T = T;
      out.Tarb = [];
   end
   % is the LFT singular??? 
   if cen>lnr(2)
      delcrit = 1/S(1,1);
      totalrange = [S(2,2)-S(1,2)*S(2,1)/S(1,1) Inf];
      %totalrange = [(lnr(2)^2-lnr(1)*lnr(3))/2/(lnr(2)-cen) Inf];
   elseif cen<lnr(2)
      delcrit = 1/S(1,1);
      totalrange = [-Inf S(2,2)-S(1,2)*S(2,1)/S(1,1)];
      %totalrange = [-Inf (lnr(2)^2-lnr(1)*lnr(3))/2/(lnr(2)-cen)];
   else
      delcrit = Inf;
      totalrange = [-Inf Inf];
   end
   out.CriticalDelta = delcrit;
   out.TotalRange = totalrange;
case 'SNormalizer'
   lnr = [p.Data(1) pvget(p.atom,'NominalValue') p.Data(2)];
   rad = (lnr(3)-lnr(1))/2;
   cen = (lnr(3)+lnr(1))/2;
   t12s = (lnr(3)-lnr(2)) * (lnr(2)-lnr(1))/rad ;
   out = [(cen-lnr(2))/rad t12s lnr(2)];
case 'IntType'
   out = 1;
case 'Traits'
   out.scalar = 1;
   out.linear = 1;
   out.zizo = 1;
   out.dynamic = 0;
   out.uncertain = 1;
case 'Mode'
%    if ~isempty(p.Data)
      pv = {'Range';'PlusMinus';'Percentage'};
      out = pv{p.Data(3)};
%    else
%       out = [];
%    end
case 'BlockSize' % XXX what/why is this
   out = size(pvget(p.atom,'NominalValue'));
otherwise
   out = [];
   pflag = 0;
   errstring = ['Invalid UREAL property: ''' property '''.'];
end
if nargout<=1 && pflag==0
   error(errstring);
end
