%PVSET  Set properties of UREAL objects.
%  
%MAT = PVSET(MAT,'Property1',Value1,'Property2',Value2,...)
%sets the values of the properties with exact names 'Property1',
%'Property2',...
%  
%See also SET.

% Copyright 2003-2004 The MathWorks, Inc.
function [p,pflag,errstring] = pvset(p,property,value)

pflag = 1;
errstring = '';
switch property
case 'Range'
   mode = p.Data(3);
   % XXX can a UREAL have range equal to [1.2 1.2], for instance?
   if isa(value,'double') && isequal(size(value),[1 2]) && ...
    all(imag(value)==0) && value(1)<value(2)
      nom = pvget(p.atom,'NominalValue');
      p.Data = [value mode];
      if value(1)>=nom || nom>=value(2)
         warning('Resetting NominalValue to midpoint of new Range')
         nom = (value(1)+value(2))/2;
         A = pvset(p.atom,'NominalValue',nom);
         p.Data = [value mode];
         p.atom = A;
      end
   else
      pflag = 0;
      errstring = ['Invalid Range, Range should be 1-by-2 real.'];
   end
case 'PlusMinus'
    if isa(value,'double') && ndims(value)==2 && size(value,1)==1 && all(imag(value))==0
        mode = p.Data(3);
        if size(value,2)==1 && abs(value)>0
            value = [-abs(value) abs(value)];
        elseif size(value,2)==2
            if ~(value(1)<0 && value(2)>0)
               pflag = 0;
               errstring = ['Invalid PlusMinus values, 1st entry must be negative, 2nd positive.'];
            end
        else
            pflag = 0;
            errstring = ['Invalid PlusMinus values, PlusMinus must be nonzero 1-by-1 or 1-by-2.'];
        end
        p.Data = [pvget(p.atom,'NominalValue')+value mode];
    else
        pflag = 0;
        errstring = ['Invalid PlusMinus value, PlusMinus must be real row vector.'];
    end
case 'Percentage'
    if isa(value,'double') && ndims(value)==2 && size(value,1)==1 && all(imag(value))==0
        mode = p.Data(3);
        if size(value,2)==1 && abs(value)>0
            value = [-abs(value) abs(value)];
        elseif size(value,2)==2
            if ~(value(1)<0 && value(2)>0)
               pflag = 0;
               errstring = ['Invalid Percentage values, 1st entry must be negative, 2nd positive.'];
            end
        else
            pflag = 0;
            errstring = ['Invalid Percentage values, Percentage must be nonzero 1-by-1 or 1-by-2.'];
        end
        nom = pvget(p.atom,'NominalValue');
        p.Data = [nom+abs(nom)*value/100 mode];
    else
        pflag = 0;
        errstring = ['Invalid Percentage value, Percentage must be real row vector.'];
    end
case {'Name','AutoSimplify'}
   [A,pflag,errstring] = pvset(p.atom,property,value);
   if pflag==1
      p.atom = A;
   end
case {'NominalValue'}
   if isa(value,'double') && isequal(size(value),[1 1]) && imag(value)==0
      if p.Data(3)==1
         if p.Data(1)>=value || value>=p.Data(2)
%             warning('Reseting Range to unit interval around new Nominal');
%             p.Data = [value-1 value+1 1];
            error('NominalValue must be inside of Range. Change Range or Mode');
         else
            p.Data = [p.Data(1:2) 1];
         end
      elseif p.Data(3)==2
         pl = pvget(p,'PlusMinus');
         p.Data(1:2) = value+pl;
      elseif p.Data(3)==3
         perc = pvget(p,'Percentage');
         p.Data(1:2) = value+abs(value)*perc/100;
      end
      A = pvset(p.atom,'NominalValue',value);
      p.atom = A;
   else
      pflag = 0;
      errstring = ['Invalid NominalValue, NominalValue must be real, scalar.'];
   end
case 'Mode'
   pv = {'Range';'PlusMinus';'Percentage'};
   idx = find(strncmpi(pv,value,length(value)));
   if length(idx)==1
      if idx==3
         nom = pvget(p.atom,'NominalValue');
         if nom==0
            pflag = 0;
            errstring = ['Cannot set MODE to Percentage when NominalValue equals 0.'];
         end
      end
      p.Data(3) = idx;
   else
      error(['''' value '''' ' is an invalid mode'])
   end
end
if nargout<=1 && pflag==0
   error(errstring);
end

