%  APPEND  Group uncertain models by appending their inputs and outputs.
%  
%     SYS = APPEND(SYS1,SYS2, ...) produces the aggregate system
%   
%                   [ SYS1  0       ]
%             SYS = [  0   SYS2     ]
%                   [           .   ]
%                   [             . ]
%  
%     APPEND concatenates the input and output vectors of the uncertain
%     models SYS1, SYS2,... to produce the resulting model SYS.
%  
%     If SYS1,SYS2,... are arrays of uncertain models, APPEND returns an 
%     uncertain array SYS of the same size where 
%        SYS(:,:,k) = APPEND(SYS1(:,:,k),SYS2(:,:,k),...) .
%  
%     See also SERIES, PARALLEL, FEEDBACK.
function out = append(varargin)

% Copyright 2003-2004 The MathWorks, Inc.

try
   out = blkdiag(varargin{:});
catch einfo
   throw(einfo);
end


