%BLK2IOCH Converts an uncertainty to an input/output block.
%
%[OUT,IDFLG] = BLK2IOCH(MAT,BLKID,TB) opens up a LFT channel, moving the 
%exposed inputs/outputs into the top (or bottom) of the I/O block. MAT is
%an uncertain matrix, BLKID is the name or numerical index of the uncertain 
%that is present in MAT and TB is a string that determines if the block 
%should be moved to the top, 't' or the bottom, 'b' of the inputs and outputs
%of OUT. OUT is an uncertain matrix with BLKID at either the top, if
%TB='t', or bottom, TB='b', of the inputs/outputs of the original uncertain
%matrix MAT. IDFLG is 1 if the BLKID was an uncertain atom in MAT otherwise
%it is 0.
%
%BLK2IOCH returns a USS object.
%
%See also: NORMALIZE

% XXXHELP, XXXMNINOR, I/OGroups of open channel, compare to MofMDelta

% Copyright 2003-2004 The MathWorks, Inc.
function [out,idflg] = blk2ioch(mat,blkid,tb,litreq)

if nargin<3
   estr = 'Invalid Syntax.';
   error(estr);
elseif nargin==3
   litreq = 'literalZERO';
end

[outtmp,idflg] = blk2ioch(mat.UmatData,blkid,tb,litreq);
% OUTTMP is always a UMAT

szo = size(outtmp);
szm = size(mat.UmatData);
szdiff = [szo(1:2)-szm(1:2)];
oisize = szm - length(mat.StateName);

% puts blanks for names in these channels
tmpds = dynamicsys(szdiff(1),szdiff(2));
if strcmp(tb,'t')
   tmpds = utAppend(tmpds,mat.dynamicsys);
elseif strcmp(tb,'b')
   tmpds = utAppend(mat.dynamicsys,tmpds);
end
out = uss(outtmp,mat.StateName,mat.Ts,tmpds);

