%  BLKDIAG  Group uncertain models by appending their inputs and outputs.
%  
%     SYS = BLKDIAG(SYS1,SYS2, ...) produces the aggregate system
%   
%                   [ SYS1  0       ]
%             SYS = [  0   SYS2     ]
%                   [           .   ]
%                   [             . ]
%  
%     BLKDIAG concatenates the input and output vectors of the uncertain
%     models SYS1, SYS2,... to produce the resulting model SYS.
%  
%     If SYS1,SYS2,... are arrays of uncertain models, BLKDIAG returns an 
%     uncertain array SYS of the same size where 
%        SYS(:,:,k) = BLKDIAG(SYS1(:,:,k),SYS2(:,:,k),...) .
%  
%     See also SERIES, PARALLEL, FEEDBACK.
function out = blkdiag(varargin)

% Copyright 2003-2004 The MathWorks, Inc.

if nargin==1
   out = varargin{1};
elseif nargin==2
   if isa(varargin{1},'uss') & isa(varargin{2},'uss')
      sza = size(varargin{1});
      szb = size(varargin{2});
      if ~isexdok(sza(3:end),szb(3:end))
         error('Invalid extra dimensions')
      end
      if sza(1)==0 || sza(2)==0
         out = varargin{2};      
      elseif szb(1)==0 || szb(2)==0
         out = varargin{1};      
      else
         try
            out = binop(varargin{1},varargin{2},'blkdiag');
         catch einfo
            throw(einfo);
         end
      end   
   else
      out = blkdiag(uss(varargin{1}),uss(varargin{2}));
   end
else
   out = blkdiag(blkdiag(uss(varargin{1}),uss(varargin{2})),varargin{3:end});
end



