function sys = cat(dim,varargin)
%CAT  Concatenation of uncertain models.
%
%   USYS = CAT(DIM,USYS1,USYS2,...) concatenates the uncertain
%   models along the dimension DIM.  The values DIM=1,2 
%   correspond to the output and input dimensions, and the values 
%   DIM=3,4,... correspond to the USS array dimensions 1,2,...
%
%   For example,
%     * CAT(1,USYS1,USYS2) is equivalent to [USYS1 ; USYS2]
%     * CAT(2,USYS1,USYS2) is equivalent to [USYS1 , USYS2]
%     * CAT(4,USYS1,USYS2) is equivalent to STACK(2,USYS1,USYS2).
%
%   See also HORZCAT, VERTCAT, STACK, APPEND.

% Copyright 2004 The MathWorks, Inc.

switch dim
case 1
   sys = vertcat(varargin{:});
case 2
   sys = horzcat(varargin{:});
otherwise
   sys = stack(dim-2,varargin{:});
end
