% FRD  Creation of an uncertain frequency response model, UFRD.
% 
%   SYS = FRD(SYS,FREQS,'Units',UNITS) converts an arbitrary USS model SYS to
%   the UFRD model representation, generating the system response at each
%   frequency in FREQS.  UNITS specifies the units of the frequencies in FREQS,
%   which may be 'rad/s' or 'Hz', and will default to 'rad/s' if the last
%   two arguments are omitted.  The result is an UFRD model.
% 
% See also CHGUNITS, UFRD

function out = frd(sys,omega,varargin)
% Copyright 2003-2004 The MathWorks, Inc.

%XXXCONSOLIDATE - do we need this?  probably, so that they dont have
% to call UFRD.  what about allatomslti?  any connections to allreal?
if isa(sys,'uss') & isa(omega,'double') & ndims(omega)==2 & min(size(omega))==1
   if allatomslti(sys)
      out = ufrd(sys,omega,varargin{:});
   else
      error('Some atoms in SYS are not linear, time-invariant - FRD is not defined.');
   end
else
   error('SYS should be USS, and OMEGA should be DOUBLE vector.');
end
   