% INV  Inverse USS model.
% 
%  ISYS = INV(SYS) computes the inverse model ISYS such that
% 
%      y = SYS * u   <---->   u = ISYS * y 
% 
%  The USS model SYS must have the same number of inputs and
%  outputs.
% 
%  For arrays of USS models, INV is performed on each individual
%  model.
% 
%  See also mldivide, mrdivide, ltimodels.

% Copyright 2003-2004 The MathWorks, Inc.

function out = inv(a)

sza = size(a);
if sza(1)==sza(2)
   dnom = pvget(umat(pvget(a,'d')),'NominalValue');
   szd = size(dnom);
   for i=1:prod(szd(3:end))
      [l,u,p] = lu(dnom(:,:,i));
      if rcond(u)<10*eps,
         error('Cannot invert system with singular D matrix.');
      end
   end
   % cheat with mrdivide for now...
   % XXXMINOR   dynamicsys handled in mrdivide
   out = binop(uss(eye(sza(1))),a,'mrdivide');
else
   estr = 'Non-square models cannot be inverted.';
   error(estr)
end