%ISVALID  True if input is a USS.
%  
%[PFLG,A] = ISVALID(SYS) returns PFLG==1 (true) if the uncertain 
%system. SYS is an uncertain frequency response, and 0 
%otherwise. A  is the input argument SYS.

% Copyright 2003-2004 The MathWorks, Inc.
function [pflag,A,errstr] = isvalid(A)

%XXXISSUES
pflag = 1;
errstr = [];
[A.StateName,errmsg] = StateNameCheck(A.StateName);
error(errmsg); % skips if empty
NX = length(A.StateName);
szp = size(A.UmatData);
if isa(A.UmatData,'umat')
   if szp(1)>=NX && szp(2)>=NX
      [pflagU,A.UmatData] = isvalid(A.UmatData);
      if pflagU==0
         pflag = 0;
         errstr = 'Invalid UmatData';
      end
      NY = szp(1) - NX;
      NU = szp(2) - NX;
      %if NU==0
      %   warning('System has no inputs');
      %end
      %if NY==0
      %   warning('System has no outputs');
      %end
      
      % XXXMAJOR
      A.dynamicsys = utCheckSystem(A.dynamicsys,NY,NU);
   else
      pflag = 0;
      errstr = ['Dimensions of UmatData and StateName are incompatible.'];
   end
   if ~(isa(A.Ts,'double') && ndims(A.Ts)==2 && all(size(A.Ts)==[1 1]) && isreal(A.Ts) &&...
         (A.Ts>=0 || A.Ts == -1 || A.Ts == -2))
      pflag = 0;
      errstr = 'Invalid sample time.';
   end
else
   pflag = 0;
   errstr = 'Invalid UmatData.';
end

if pflag==0
   pflag = false;
else
   pflag = true;
end

%function [a,errmsg] = StateNameCheck(a)
% copied directly from @ss/set
function [a,errmsg] = StateNameCheck(a)
errmsg = '';
if isempty(a),  
   a = a(:);   % make 0x1
   return  
end

% Determine if first argument is an array or cell vector 
% of single-line strings.
if ischar(a) && ndims(a)==2,
   % A is a 2D array of padded strings
   a = cellstr(a);
   
elseif iscellstr(a) && ndims(a)==2 && min(size(a))==1,
   % A is a cell vector of strings. Check that each entry
   % is a single-line string
   a = a(:);
   if any(cellfun('ndims',a)>2) || any(cellfun('size',a,1)>1),
      errmsg = 'All cell entries of StateName must be single-line strings.';
   end
   
else
   errmsg = sprintf([ ...
     'StateName must be set to a 2D array of padded strings (like [''a'' ; ''b'' ; ''c''])\n' ...
     'or a cell vector of strings (like {''a'' ; ''b'' ; ''c''}).']);
end


