function sys=mktito(sys,nmeas,ncont)
%MKTITO Create a two-input-two-output partitioning of a USS object
%
% SYS=MKTITO(SYS,NMEAS,NCONT) adds TITO (two-input-two-output)
%      partitioning to an USS system by setting its InputGroup 
%      and OutputGroup properties as follows:
%         set(sys,'InputGroup',{ 1:c-ncont 'U1'; c-ncont+1:c 'U2' });
%         set(sys,'OutputGroup',{ 1:r-nmeas 'Y1'; r-nmeas+1:r 'Y2'});
%      Any pre-existing partition of SYS is overwritten.
% 
% See also AUGW HINFSYN H2SYN SDHFSYN

% Based on Gahinet's 01/98 partitioning specification
% Rev 11/15/02 M. G. Safonov

% Copyright 2003-2004 The MathWorks, Inc.
[r,c]=size(sys);
if ~isa(sys,'uss'), error('SYS is not an USS object'), end
if r<nmeas, error('NMEAS exceeds SYS output dimension'), end
if c<ncont, error('NCONT exceeds SYS input dimension'), end

set(sys,'InputGroup',{ 1:c-ncont 'U1'; c-ncont+1:c 'U2' });
set(sys,'OutputGroup',{ 1:r-nmeas 'Y1'; r-nmeas+1:r 'Y2'});

% ----------- End of MKTITO.M --------

