%PERMUTE  Permute array of USS models.
%   B = PERMUTE(A,ORDER) rearranges the array dimensions of A so that they
%   are in the order specified by the vector ORDER.  The input/output
%   dimensions of A are unchanged.  The resultant array B has the same
%   values of A but the order of the subscripts needed to access any
%   particular element are rearranged as specified by ORDER.  The elements
%   of ORDER must be a rearrangement of the numbers from 1 to N.
% 
%   PERMUTE and IPERMUTE are a generalization of transpose (.') 
%   for USS arrays.
%  
%   Example
%    a = rss(3,2,2);
%    del = ultidyn('del',[2 2]);
%    aa = (a+del)*rand(2,2,3,4,5);
%    size(permute(aa,[3 2 1])) %now 2 input/2 output 5-by-4-by-3 UMAT array.
%  
%   See also IPERMUTE.

% Copyright 2003-2004 The MathWorks, Inc.

function m = permute(m, index)

index = index(:).';
szm = size(m);
if (length(index)>=length(szm)-2 || (length(szm)==4 && szm(4)==1)) && ...
      isa(index,'double') && isequal(sort(index),1:length(index))
   modindex = [1 2 2+index]; %correct since UMAT/permute allows 1,2 indexs
   try
      m.UmatData = permute(m.UmatData,modindex);
   catch einfo
      throw(einfo);
   end
else
   msg = ['ORDER, ' int2str(index) ' contains an invalid permutation index.'];
   error(msg);
end

