%PVGET  Get values of public USS properties.
%  
%VALUES = PVGET(SYS) returns all public values in a cell
%array VALUES.
%  
%VALUE = PVGET(SYS,PROPERTY) returns the value of the
%single property with name PROPERTY.
%  
%See also GET.

% Copyright 2003-2006 The MathWorks, Inc.
function [out,pflag,errstring] = pvget(m,property,varargin)

pflag = 1;
errstring = '';
switch property
case 'UmatData'
   out = m.UmatData;
case 'StateName'
   out = m.StateName;
case 'Ts'
   out = m.Ts;
case {'dynamicsys' 'dynamicsys'}
   out = m.dynamicsys;
case {'Uncertainty' 'Literal' 'Copies'}
   out = pvget(m.UmatData,property);
case 'NominalValue'
   % m.Nominal of USS is SS
   nx = length(m.StateName);
   tmp = pvget(m.UmatData,'NominalValue'); % tmp is a DOUBLE
   szt = size(tmp);
   ny = szt(1) - nx;
   nu = szt(2) - nx;
   a = zeros([nx nx szt(3:end)]);
   a(1:nx,1:nx,:) = tmp(1:nx,1:nx,:);
   b = zeros([nx nu szt(3:end)]);
   b(1:nx,1:nu,:) = tmp(1:nx,nx+1:end,:);
   c = zeros([ny nx szt(3:end)]);
   c(1:ny,1:nx,:) = tmp(nx+1:end,1:nx,:);
   d = zeros([ny nu szt(3:end)]);
   d(1:ny,1:nu,:) = tmp(nx+1:end,nx+1:end,:);
%    Old code, eliminated Nov05, as LTI changed
%    out = ss(a,b,c,d,ds2lti(m.dynamicsys,0));
%    out = pvset(out,'Ts',utarbts(m.Ts),'StateName',m.StateName);
   out = ss(a,b,c,d);
   namevalues = [pnames(m.dynamicsys)';pvget(m.dynamicsys)'];
   out = pvset(out,namevalues{:});
   out = pvset(out,'StateName',m.StateName);
   out = setTs(out,utarbts(m.Ts));
case 'a'
   nx = length(m.StateName);
   szt = size(m.UmatData);
   ny = szt(1) - nx;
   nu = szt(2) - nx;
   out = simplify(m.UmatData(1:nx,1:nx));
case 'b'
   nx = length(m.StateName);
   szt = size(m.UmatData);
   ny = szt(1) - nx;
   nu = szt(2) - nx;
   out = simplify(m.UmatData(1:nx,nx+1:nx+nu));
case 'c'
   nx = length(m.StateName);
   szt = size(m.UmatData);
   ny = szt(1) - nx;
   nu = szt(2) - nx;
   out = simplify(m.UmatData(nx+1:nx+ny,1:nx));
case 'd'
   nx = length(m.StateName);
   szt = size(m.UmatData);
   ny = szt(1) - nx;
   nu = szt(2) - nx;
   out = simplify(m.UmatData(nx+1:nx+ny,nx+1:nx+nu));
case 'PropNames'
   g1 = {'a';'b';'c';'d'};
   s1 = cell(0,1);
   s1d = cell(0,1);
   com2 = {'StateName';'Ts'};
   com2d = {'Nx x 1 cell of strings';'scalar DOUBLE'};
   g3 = {'Nx'};
   s3 = cell(0,1);
   s3d = cell(0,1);
   com4 = {'InputName';'OutputName';'InputGroup';'OutputGroup'};
   com4d = {'Nu-by-1 cell array of strings';...        
    'Ny-by-1 cell array of strings';...        
    'STRUCT of Input groups';... 
    'STRUCT of Output groups'};
   g5 = [{'NominalValue';'Uncertainty'}];
   g5 = [{'NominalValue';'Uncertainty';'UNames'}];
   g5 = [{'NominalValue';'Uncertainty'}];
   s5 = [{'Uncertainty'}];
   s5d = {'not explicitly set by user, gateway to uncertain atoms'};
%    g6 = [{'Notes';'UserData';'Name'}];
%    s6 = [{'Notes';'UserData';'Name'}];
%    s6d = {'Array or cell array of strings';'Arbitrary';'String'};
   g6 = [{'Notes';'UserData'}];
   s6 = [{'Notes';'UserData'}];
   s6d = {'Array or cell array of strings';'Arbitrary'};
%    out.GPropNames = [g1;com2;g3;com4;g5;g6];
%    out.SPropNames = [s1;com2;s3;com4;s5;s6];
%    out.SPropDescription = [s1d;com2d;s3d;com4d;s5d;s6d];
   out.GPropNames = [g1;com2;com4;g5;g6];
   out.SPropNames = [s1;com2;com4;s5;s6];
   out.SPropDescription = [s1d;com2d;com4d;s5d;s6d];
case {'Instance'}
   out = usample(m);
case {'DELTAofMDelta' 'BlkInfoStructure' 'N23BlkStructure' 'theDELTAasNDLFT'}
   % N23BlkStructure allows for 3rd ALLOWSUBS argument,  that is what the
   % varargin is for.  
   out = pvget(m.UmatData,property,varargin{:});
case 'Nx'
   out = length(m.StateName);
case 'UNames'
   out = pvget(m.UmatData,'UNames');
case {'InputName','OutputName','InputGroup','OutputGroup',...
        'Notes','UserData','Version','Name'}
   out = pvget(m.dynamicsys,property);
otherwise
   out = [];
   pflag = 0;
   errstring = ['Invalid USS property: ''' property '''.'];
end
if nargout<=1 && pflag==0
   error(errstring);
end

function [RN,CN,RG,CG,RGs,CGs] = LOCALrcdata(blksz,nblk,blkn,rp,cp)
CN = repmat({''},[blksz(1) 1]);
RN = repmat({''},[blksz(2) 1]);
RG = cell(nblk,2);
CG = cell(nblk,2);
for i = 1:nblk
   RG{i,1}  = (rp(i):rp(i+1)-1);
   RG{i,2}  = [blkn{i} '_NC'];
   CG{i,1} = (cp(i):cp(i+1)-1);
   CG{i,2} = [blkn{i} '_NC'];
end
% Update to NEW GROUP
CGtmp = CG(:,[2 1])';
CGs = struct(CGtmp{:});
RGtmp = RG(:,[2 1])';
RGs = struct(RGtmp{:});

