%PVSET  Set properties of USS objects.
%  
%SYS = PVSET(SYS,'Property1',Value1,'Property2',Value2,...)
%sets the values of the properties with exact names 'Property1',
%'Property2',...
%  
%See also SET.

% Copyright 2003-2004 The MathWorks, Inc.
function [m,pflag,errstring] = pvset(m,property,value)

pflag = 1;
errstring = '';
switch property
case 'UmatData'
   m.UmatData = value;
case 'StateName'
   nx = length(m.StateName);
   if isa(value,'cell') & all(size(value)==[nx 1])
      m.StateName = value;
   else
      pflag = 0;
      errstring = 'StateName should be a cell array of strings';
   end
case 'Ts'
   if isa(value,'double') && ndims(value)==2 && isreal(value) && all(size(value)==[1 1]) ...
         && (value>=0 | value==-1)
      m.Ts = value;
   else
      pflag = 0;
      errstring  = 'Ts should be -1, 0 or positive scalar';
   end
case {'InputName','OutputName','InputGroup','OutputGroup',...
        'Notes','UserData','Name'}
   tmp = m.dynamicsys;
   tmp = pvset(tmp,property,value);
   m.dynamicsys = tmp;
case {'dynamicsys'}
   m.dynamicsys = value;
case 'Uncertainty'
%   error(' USS Uncertainty property is read only');
   try
      m.UmatData = pvset(m.UmatData,property,value);
   catch
      rethrow(lasterror)
   end
otherwise
   pflag = 0;
   errstring = ['Invalid USS property: ' property];
end
if nargout<=1 && pflag==0
   error(errstring);
end
      
