% SIZE  Size and order of UFRD models.
% 
%  D = SIZE(USYS) returns
%     * the two-entry row vector D = [NY NU] for a single UFRD model 
%       USYS with NY outputs and NU inputs
%     * the row vector D = [NY NU S1 S2 ... Sp] for a S1-by-...-by-Sp 
%       array of UFRD models with NY outputs and NU inputs.
%  SIZE(USYS) by itself makes a nice display.
% 
%  [NY,NU,S1,...,Sp] = SIZE(USYS) returns
%     * the number of outputs NY
%     * the number of inputs NU 
%     * the UFRD array sizes S1,...,Sp (for arrays of UFRD models)
%  in separate output arguments.  Alternatively,
%     NY = SIZE(USYS,1)   returns just the number of outputs.
%     NU = SIZE(USYS,2)   returns just the number of inputs.
%     Sk = SIZE(USYS,2+k) returns the length of the k-th UFRD array 
%                        dimension.
% 
%  NS = SIZE(USYS,'order') returns the model order (number of states 
%  for state-space models).  For LTI arrays, NS is scalar when all
%  models have the same order, and is an array listing the order of
%  each model otherwise.
% 
%  For FRD models,
%     NF = SIZE(SYS,'freq') 
%  returns the number of frequency points.
% 
%  See also NDIMS, ISEMPTY

% Copyright 2003-2004 The MathWorks, Inc.
function [varargout] = size(m,arg2)
% returns [output input] dimensions, and extra dimensions

%XXX need to work on the NARGOUT==0 display, to match with LTIARRAY
% NOTDONE, see above remark
%XXXMINOR, don't exactly know.
out = size(m.UmatData);
out(1:2) = out(1:2) - length(m.StateName);
if length(out)==3
   out = [out 1];
end
if nargin==1
   arg2 = nan;
end
varargout = csize(out,arg2,nargin,nargout);
