%VERTCAT  Vertical concatenation of USS objects.
%  
%SYS = VERTCAT(SYS1,SYS2,...) performs a concatenation operation of 
%  SYS = [SYS1; SYS2; ...].
%
%See also: HORZCAT

% Copyright 2003-2004 The MathWorks, Inc.
function out = vertcat(varargin)

if nargin==1
   out = varargin{1};
elseif nargin==2
   if isa(varargin{1},'uss') & isa(varargin{2},'uss')
      A = varargin{1};
	  B = varargin{2};
      sza = size(A);
      szb = size(B);
      if ~isexdok(sza(3:end),szb(3:end))
         error('Array dimensions are incompatible.')
      end
      if sza(1)==0 && (sza(2)==szb(2) | sza(2)==0)
         out = B;
      elseif szb(1)==0 && (sza(2)==szb(2) | szb(2)==0)
         out = A;
      elseif sza(2)==szb(2)
         try
            out = binop(A,B,'vertcat');
         catch einfo
            throw(einfo);
         end
      else
         error('In [SYS1 ; SYS2], SYS1 and SYS2 must have the same number of inputs.')
      end
   else
      out = vertcat(uss(varargin{1}),uss(varargin{2}));
   end
else
   out = vertcat(vertcat(uss(varargin{1}),uss(varargin{2})),varargin{3:end});
end



