% Robust Control Toolbox
% Version 3.3.3 (R2009a) 15-Jan-2009
% 
% Uncertain elements.
%   ucomplex    - Uncertain complex parameter.
%   ucomplexm   - Uncertain complex matrix.
%   udyn        - Uncertain dynamics.
%   ultidyn     - Uncertain linear time-invariant object.
%   ureal       - Uncertain real parameter.
%
% Uncertain matrices and systems
%   umat        - Uncertain matrix.
%   uss         - Uncertain State Space model.
%   ufrd        - Uncertain Frequency Response Data model.
%   randatom    - Create a random uncertain element.
%   randumat    - Create a random uncertain matrix.
%   randuss     - Create a random uncertain state-space model.
%
% Manipulation of uncertain models.
%   actual2normalized - Normalized-distance of an uncertain atom.
%   normalized2actual - Normalized to actual value uncertain atom conversion.
%   isuncertain - True for uncertain objects.
%   normalize   - Normalizes an atom.
%   simplify    - Simplify representation of an uncertain object.
%   usubs       - Substitutes values for uncertain elements.
%   usample     - Generate random samples of an uncertain object.
%   usimfill    - Fills in Uncertainty field in USS Simulink blocks.
%   usiminfo    - Find USS System blocks within Simulink model.
%   usimsamp    - Generate random instance of Simulink USS System blocks.
%   gridureal   - Grid real parameters over their range.
%   fliplr      - Flip an uncertain matrix in left/right direction.
%   flipud      - Flip an uncertain matrix in up/down direction.
%   ipermute    - Inverse permute array dimensions.
%   lftdata     - Extract LFT data from uncertain object.
%   permute     - Permute array dimensions.
%   repmat      - Replicate and tile an uncertain matrix.
%   ssbal       - Diagonal state/uncertainty scaling for an uncertain system.
%
% Interconnection of uncertain models.
%   imp2exp     - Converts implicit LFT relationship to Explicit I/O.
%   sysic       - Form interconnections of LTI and uncertain objects.
%   iconnect    - Creates an interconnection object (alternative to sysic).
%   icsignal    - Creates an ICsignal object for equations used with ICONNECT.
%   iczero      - Creates an identically-zero ICsignal used with ICONNECT.
%
% Type "help control" for additional system manipulation commands.
%
% Model order reduction.
%   reduce      - Main interface to model approximation algorithms.
%   balancmr    - Balanced truncation model reduction.
%   bstmr       - Balanced stochastic truncation model reduction.
%   hankelmr    - Optimal Hankel norm model approximation.
%   modreal     - State-space modal truncation/realization.
%   ncfmr       - Balanced normalized coprime factor model reduction.
%   schurmr     - Schur balanced truncation model reduction.
%   slowfast    - State-space slow-fast decomposition.
%   imp2ss      - Impulse response to state-space approximation.
%   
% Robustness and worst-case analysis.
%   cpmargin    - Coprime stability margin of plant-controller feedback loop.
%   gapmetric   - Computes the gap and the Vinnicombe gap metric.
%   loopmargin  - Comprehensive analysis of feedback loop.
%   loopsens    - Sensitivity functions of feedback loop.
%   mussv       - Bounds on the structured singular value (mu).
%   mussvextract- Extracts data from MUSSV output structure.
%   ncfmargin   - Normalized coprime stability margin of feedback loop.
%   popov       - Test for robust stability with Popov criterion.
%   robustperf  - Robust performance of uncertain systems.
%   robuststab  - Stability margins of uncertain systems.
%   robopt      - Creates a ROBUSTSTAB/ROBUSTPERF options object.
%   wcnorm      - Worst-case norm of an uncertain matrix.  
%   wcgain      - Worst-case gain of an uncertain system.
%   wcgopt      - Creates a WCNORM/WCGAIN options object.
%   wcmargin    - Worst-case gain/phase margins for feedback loop.
%   wcsens      - Worst-case sensitivity functions of feedback loop.
%
% Robustness analysis for parameter-dependent systems (P-system).
%   psys        - Specify a parameter-dependent system (P-system).
%   psinfo      - Query characteristics of a P-system.
%   ispsys      - True for parameter-dependent systems.
%   pvec        - Specify a vector of uncertain or time-varying parameters.
%   pvinfo      - Query characteristics of a parameter vector.
%   polydec     - Compute polytopic coordinates wrt. box corners.
%   uss         - Convert affine P-system to uncertain state-space model.
%   aff2pol     - Convert affine P-systems to polytopic representation.
%   quadstab    - Assess quadratic stability of parameter-dependent system.
%   quadperf    - Assess quadratic Hinf performance of P-systems.
%   pdlstab     - Test robust stability via parametric Lyapunov functions.
%   decay       - Compute quadratic decay rate.
%   pdsimul     - Simulate P-systems along parameter trajectories.
%
% Controller synthesis.
%   augw        - Augments plant weights for mixed-sensitivity control design.
%   h2hinfsyn   - Mixed H2/H-infinity controller synthesis.
%   h2syn       - H2 controller synthesis.
%   hinfsyn     - H-infinity controller synthesis.
%   sdhinfsyn   - H-infinity controller synthesis for continuous sampled-data system.
%   loopsyn     - H-infinity loop shaping controller synthesis.
%   ltrsyn      - Loop-transfer recovery controller synthesis.
%   mixsyn      - H-infinity mixed-sensitivity controller synthesis.
%   ncfsyn      - H-infinity normalized coprime factor controller synthesis.
%   mkfilter    - Construct a shaping filter.
%
% Mu synthesis.
%   cmsclsyn    - Constant matrix mu-synthesis.
%   dksyn       - Synthesis of a robust controller via mu-synthesis.
%   dkitopt     - Create a DKSYN options object.
%   drawmag     - Interactive mouse-based sketching and fitting tool.
%   fitfrd      - Fit frequency-dependent scaling with LTI model.
%   fitmagfrd   - Fit scaling magnitude data with stable, minimum-phase model.
%
% Sampled-data systems.
%   sdhinfnorm  - Induced L2 norm of a sample-data system.
%   sdlsim      - Time response of sampled-data feedback system.
%   sdhinfsyn   - Sample-data H-infinity controller synthesis.
% 
% Gain scheduling.
%   hinfgs      - Design gain-scheduled H-infinity controllers.
%
% Operations on frequency-response data (FRD) models.
%   frd/rcond       - Reciprocal condition estimator.
%   frd/schur       - Schur decomposition.
%   frd/svd         - Singular value decomposition.
%   frd/plot        - Linear plot.
%   frd/loglog      - Log-log scale plot.
%   frd/semilogx    - Semi-log scale plot.
%   frd/semilogy    - Semi-log scale plot.
% 
% Supporting utilities.
%   bilin       - State-space bilinear transform. 
%   mktito      - Partition LTI system via Input and Output Groups.
%   sectf       - Sector transformation for LTI systems.
%  
% LMI solvers.
%   Type "help rctlmi" for details.
%
% Demonstrations.
%   Type "demo" or "help robust" for a list of available demos.
