function b = imp2exp(a,yidx,uidx)
%IMP2EXP Converts an implicit linear relationship to explicit
%   input-output relation.
%
%   B = IMP2EXP(A,YIDX,UIDX) transforms a linear constraint on
%   variables Y and U of the form A(:,[YIDX;UIDX])*[Y;U] = 0 into
%   an explicit input/output relationship Y = B*U.  The index
%   vectors YIDX and UIDX refer to the columns (inputs) of A as
%   referenced by the explicit relationship for B.
%
%   See also INV

% Copyright 2004 The MathWorks, Inc.

if isa(a,'double') || isa(a,'ss') || isa(a,'zpk') || isa(a,'tf')
   % cheat and make DOUBLE/SS into USS
   try
      b = depromote(imp2exp(uss(a),yidx,uidx));
   catch
      rethrow(lasterror);
   end
else
   error('A should be a DOUBLE or SS/ZPK/TF');
end