%LFT  Generalized feedback interconnection of systems.
%
%   SYS = LFT(SYS1,SYS2,NU,NY) takes two systems SYS1 and SYS2 and
%   computes a model SYS of the following feedback interconnection:
%
%                        +-------+
%            w1 -------->|       |-------> z1
%                        |  SYS1 |
%                  +---->|       |-----+
%                  |     +-------+     |
%                u |                   | y
%                  |     +-------+     |
%                  +-----|       |<----+
%                        |  SYS2 |
%           z2 <---------|       |-------- w2
%                        +-------+
%
%   The feedback loop connects the first NU outputs of SYS2 to the
%   last NU inputs of SYS1 (signals u), and the last NY outputs of
%   SYS1 to the first NY inputs of SYS2 (signals y).  The resulting
%   model SYS maps the input vector [w1;w2] to the output vector
%   [z1;z2].  The resulting model SYS is also referred to as the
%   linear fractional transformation (LFT) of SYS1 and SYS2.
%
%   SYS = LFT(SYS1,SYS2) returns
%     * the lower LFT of SYS1 and SYS2 if SYS2 has fewer inputs and
%       outputs than SYS1.  This amounts to deleting w2,z2 in the
%       above diagram.
%     * the upper LFT of SYS1 and SYS2 if SYS1 has fewer inputs and
%       outputs than SYS2.  This amounts to deleting w1,z1 above.
%
%   If SYS1 and SYS2 are arrays of LTI models, LFT returns an LTI
%   array SYS of the same dimensions where
%      SYS(:,:,k) = LFT(SYS1(:,:,k),SYS2(:,:,k),NU,NY)
%
%   Note: The LFT command is applicable to a variety of objects,
%   including LTI models (SS, ZPK, TF, FRD), uncertain models
%   and matrices (USS, UFRD, UMAT), and elementary uncertainty (UREAL,
%   UCOMPLEX, ULTIDYN,...).
%
%   See also FEEDBACK, CONNECT, LTIMODELS.

function out = lft(A,B,dim1,dim2)

% Copyright 2003-2004 The MathWorks, Inc.


if nargin==1 | nargin==3
   error('Invalid syntax - need 2 (or 4) input arguments');
elseif nargin==0
   disp('usage: out = lft(A,B,dim1,dim2)');
   return
end

% XXX, need to fix this, even though it works ok
if ~isa(A,'double') && ~isa(A,'atom')
   error('A is not of correct class.')
elseif ~isa(B,'double') && ~isa(B,'atom')
   error('B is not of correct class.')
else
   A = umat(A);
   B = umat(B);
end
if nargin==2
   out = lft(A,B);
elseif nargin==4
   out = lft(A,B,dim1,dim2);
else
   error('Invalid # of arguments');
end
if isa(out,'umat')
   out = depromote(out);
end