% Copyright 2003-2004 The MathWorks, Inc.

function [mlist]=llist2mlist(llist)
% ROBUST/PRIVATE/LLIST2MLIST.M
%
% MATLIST=LLIST2MLIST({SYS1,SYS2,SYS3,...,SYSN})
%    Packs N LTI/SS systems contained in the cell vector
%        CV={SYS1,...,SYSN}
%    into a single matrix as LTISS (i.e., PCK) systems: 
%        MATLIST =[LTI2MAT(SYS1), LTI2MAT(SYS2), ..., LTI2MAT(SYSN)]
%    All of the systems SYS1,...,SYSN must have the same input, output
%    and state dimensions.

% This function is called by PSYS.M

% M. G. Safonov 1/7/03


% initialize

if isempty(llist)
    mlist=[];
    return
end

switch class(llist)
    case {'double'} % old LMI Toolbox format
        mlist=llist;
    case {'cell'} % SYSLIST={SYS1,...,SYSN} with each SYS an LTI of class SS
        % test data for consistency
        if isequal(2,ndims(llist)) & isequal(1,min(size(llist)))
            for i=1:length(llist),
                if ~isa(llist{i},'ss')
                    error('All elements in cell array input must be LTI SS systems')
                end
            end
            ns1=size(get(llist{1},'a'),1);
            [ny1,nu1]=size(llist{1});
            v1=[ns1, ny1, nu1];
            for i=2:length(llist),
                ns=size(get(llist{i},'a'),1);
                [ny,nu]=size(llist{i});
                vi=[ns,ny,nu];
                if ~isequal(v1,vi),
                    error('Each SYS1,...,SYSN must have identical state, input and output dimensions')
                end
            end
        else
            error('{SYS1,...,SYSN} must be a 1xN cell array')
        end
        [llist{:}]=lti2mat(llist{:});
        mlist=horzcat(llist{:});
    otherwise
        error('{SYS1,...,SYSN} must be a 1xN cell array of identically dimensioned SS or DSS systems')
    end
end % end switch
                    
%%%%%%%%%% End ROBUST/PRIVATE/LLIST2MLIST.M --- M. G. Safonov 1/7/03 %%%%%%%%%%

