function out = simplify(a,varargin)
%SIMPLIFY Simplify representation of Uncertain objects
%
%   B = SIMPLIFY(A) performs model-reduction-like techniques to
%   detect and eliminate redundant copies of uncertain elements.
%   Depending on the result, the CLASS of B may be lower than A.
%   The AutoSimplify property of each uncertain element in
%   A determines what reduction methods are used.  After reduction,
%   any uncertain element which does not actually affect the result
%   is deleted from the representation.
%
%   B = SIMPLIFY(A,'full') overrides all uncertain element's
%   AutoSimplify property, and uses 'full' reduction techniques.
%
%   B = SIMPLIFY(A,'basic') overrides all uncertain element's
%   AutoSimplify property, and uses 'basic' reduction techniques.
%
%   B = SIMPLIFY(A,'class') does no reduction.  However, any uncertain
%   elements in A with 0 occurances are eliminated, and the class of
%   B may be lower than class of A.

% Copyright 2004 The MathWorks, Inc.

out = a;
