% Simulation environment of a 2-link planar manipulator
% Copyright 2011 Adriano Siqueira
% File sim_const.m
% Define constants

disp('Initializing constants, please wait...');
disp(' ');

% default values
ic_d = 0*pi/180;  % default initial condition in rd
sp_d = 20*pi/180;  % default set-point in rd
ic_dflt = [ic_d; ic_d];
sp_dflt = [sp_d; sp_d];
grav_dflt = 0;  % default gravitational acceleration
grav_flag = -1;  % gravity is desabled
grav_comp_flag = -1; % gravity compensation is desabled
du_dflt = [1 1 1];  % default dynamic uncertainty
ki_flag =-1;

m_dflt = [5.0 5.0];  % mass of the links
l_dflt = [0.3 0.3];  % lenght of the links
lc_dflt = [0.15 0.15];  % center of mass of the links


% simulation parameters
terminate = 0;  % signal to terminate control loop
bke=0;
tfinal =60;  % run at most for tfinal seconds

dt = 0.003;  % integration step
run_after_done = 0.6/dt;  % if control is done, run for a while and stop
max_sample = fix(tfinal/dt);  % maximum number of samples
tau_sat = 50;  % saturation values of applied torque
max_error = [deg2rd(0.5) deg2rd(0.5)];  % maximum pos. and vel. errors

tor_flag = 1;  % torques are enabled

dist_flag = -1;



% initial assumptions
n = 2;  % number of joints

cont = 'PID';
traj = 1;
traj_t = 'Poly (5th)';
du = du_dflt;  % dynamic uncertainty (exact model = 1)

m = m_dflt;  % mass of the links
l = l_dflt;  % length of the links
lc = lc_dflt;  % center of mass of the links

grav = grav_dflt;  % gravitational acceleration
init_cond = ic_dflt;  % initial condition

set_point = sp_dflt;  % set_point

% graphical interface

graphic_step = 10;
erase_mode = 'xor';
gui_name = '2-link Planar Manipulator Control Environment';
gui_x_pos = 0.1;
gui_y_pos = 0.07;
gui_x_size = 0.8;
gui_y_size = 0.88;

move_color = 'blue';
reach_color = 'green';
act_move_color = 'blue';
act_reach_color = 'green';
act_wait_color = 'red';
act_edge_color = 'black';
pas_move_color = 'blue';
pas_brake_color = 'red';
pas_brake_def_color = 'green';
pas_edge_color = 'black';
link_color = 'black';
link_edge_color = 'black';
pseudo_color = 'magenta';

msg_frame_x_pos = 0.71;
msg_frame_y_pos = 0.01;
msg_frame_x_size = 0.28;
msg_frame_y_size = 0.97;
msg_frame_top = msg_frame_y_pos + msg_frame_y_size;
msg_frame_x_border = 0.02;
msg_frame_y_border = 0.045;

text_height = 0.040;
text_width_huge = 0.24;
text_width_big = 0.2;
text_width_medium = 0.1;
text_width_small = 0.06;
value_width = 0.11;
text_sep_y = 0.0012;
text_sep_x = 0.01;
height_size = 0.23;
button_size = 0.08;
popup_size = 0.12;

pos_x1 = msg_frame_x_pos + msg_frame_x_border;
pos_x5 = 0.5 + 21*text_sep_x;
pos_y1 = msg_frame_y_pos + msg_frame_y_size - msg_frame_y_border;
pos_y2 = pos_y1 - 25*text_sep_y - 0.01;
pos_y3 = pos_y2 - 25*text_sep_y - 0.01;
pos_y4 = pos_y3 - 25*text_sep_y - 0.01;
pos_y5 = pos_y4 - 25*text_sep_y - 0.01;
pos_y6 = pos_y5 - 25*text_sep_y - 0.01;
pos_y7 = pos_y2 - height_size;
pos_y8 = pos_y7 - height_size;
pos_y9 = pos_y8 - height_size - 0.01;
pos_y10 = pos_y9 - height_size;
pos_y11 = pos_y10 - height_size;
pos_y12 = pos_y11 - height_size;

% geometric dimensions
link_width = 550*0.025; % link width = 1 inch * multiplication factor
joint_radius = 3000*[0.0350 0.035 0.035 0.02]; % joint radii * mult. factor
