% Simulation environment of a 2-link planar manipulator
% Copyright 2011 Adriano Siqueira
% File sim_init_draw.m
% Initialize robot's links and actuators

% clean up movie_axis
set(graph_interface,'CurrentAxes',movie_axis);
cla;

% set axis
axis_limit = sum(l(1:2))*1.1;
axis([-axis_limit, axis_limit, -axis_limit, axis_limit]);

% clean up variables that have to be redrawn
clear link motor pseudo X Y

% set time field
set(time_value,'String','0.0');
% draw pseudo-links to represent the set-point
% first compute sum of link angles
for j = 1:2, 
  phi(j) = sum(set_point(1:j));
end;
cphi = cos(phi);
sphi = sin(phi);
% then run forward kinematic to find out where links should be at the SP
X(1) = 0;
Y(1) = 0;
for j = 2:2+1,
  X(j) = X(j-1) + l(j-1)*cphi(j-1);
  Y(j) = Y(j-1) + l(j-1)*sphi(j-1);
end;
 %display pseudo-links
pseudo = line(...
  'XData',X,...
  'YData',Y,...
  'Color',pseudo_color);

% create links and actuators objects based on initial conditions 
% first compute sum of joint angles
for j = 1:2, 
  phi(j) = sum(init_cond(1:j));
end;
cphi = cos(phi);
sphi = sin(phi);
% then run forward kinematic to find out where joints are
X(1) = 0;
Y(1) = 0;
for j = 2:2+1,
  X(j) = X(j-1) + l(j-1)*cphi(j-1);
  Y(j) = Y(j-1) + l(j-1)*sphi(j-1);
end;
% display the links and actuators in the initial position
link = line(...
  'XData',X,...
  'YData',Y,...
  'Color',link_color,...
  'LineWidth',link_width);

for j = 1:2+1,
  motor(j) = line(...
             'XData',X(j),...
             'YData',Y(j),...
             'Color',move_color,...
             'Marker','.',...
             'MarkerSize',joint_radius(j));
         
end;

if all(cont =='Car'),
   [xy_init_cond] = sim_forw_kin(init_cond,l);
    [xy_set_point] = sim_forw_kin(set_point,l);

pseudo2 = line(...
  'XData',[xy_init_cond(1),xy_set_point(1)],...
  'YData',[xy_init_cond(2),xy_set_point(2)],...
  'EraseMode',erase_mode,...
  'Color',pseudo_color);

end
    
    
drawnow;
