% Simulation environment of a 2-link planar manipulator
% Copyright 2011 Adriano Siqueira
% File sim_init_graphic.m
% Initialize the graphical user interface

% open graphic window
disp('Initializing graphical user interface, please wait...');
disp(' ');

% initialize constants
sim_const;

% create graphic figures
cont='PID'; 
cont_name = 'PID Control';
sim_init_graph;
cont='CTq'; 
cont_name = 'Computed Torque';
sim_init_graph;
cont='ADP'; 
cont_name = 'Adaptive Control';
sim_init_graph;
 cont='HIN'; 
cont_name = 'CT + H-infinity Control';
sim_init_graph;
 cont='LPV'; 
cont_name = 'LPV Control';
sim_init_graph;
 cont='CEV'; 
cont_name = 'Variable Structure Control';
sim_init_graph;
cont='Car'; 
cont_name = 'Cartesian Control';
sim_init_graph;

 
%create graphic interface
graph_interface = figure(...
  'Unit','normalized',...
  'NextPlot','add',...
  'Visible','on',...
  'Menubar','none',...
  'Toolbar','none',...
  'NumberTitle','off',... % 'MenuBar','off',...
  'Position',[gui_x_pos,gui_y_pos,gui_x_size,gui_y_size],...
  'Name',gui_name,...
  'Pointer','watch');

set(0,'Current',graph_interface);

% the axis named 'movie' shows the motion of the manipulator
movie_axis = axes(...
  'Units','normalized',...
  'Position',[.08 .08 0.6 .85],...
  'XLim',[-sum(l) sum(l)],...
  'YLim',[-sum(l) sum(l)],...
  'XGrid','on',...
  'YGrid','on');
BDF_movie_axis = [...
  '[cart(1), cart(2), button] = ginput(1);'...
  '[init_cond, set_point] = sim_inv_kin_graph(n, l, cart, button, init_cond, set_point);'...
  'sim_init_draw;'...
  'sim_set_msg;'];
set(movie_axis,'ButtonDownFcn',BDF_movie_axis);
xlabel('X COORDINATE (m)');
ylabel('Y COORDINATE (m)');
% define 'movie_axis' as the current one
set(graph_interface,'CurrentAxes',movie_axis);

% create frame to hold messages and button
msg_frame = uicontrol(...
  'Style','frame',...
  'Unit','normalized',...
  'Position',[msg_frame_x_pos,msg_frame_y_pos,...
  msg_frame_x_size,msg_frame_y_size]);
msg_frame_title1 = uicontrol(...
  graph_interface,...
  'Style','text',...
  'Unit','normalized',...
    'FontSize',9,...
  'Position',[pos_x1, pos_y1-0.01, text_width_big, text_height],...
  'Horiz','left',...
  'String','USER COMMANDS');
msg_frame_title2 = uicontrol(...
  graph_interface,...
  'Style','text',...
  'Unit','normalized',...
      'FontSize',9,...
  'Position',[pos_x1, pos_y7-0.005, text_width_big, text_height],...
  'Horiz','left',...
  'String','SIMULATION PARAMETERS');
msg_frame_title3 = uicontrol(...
  graph_interface,...
  'Style','text',...
  'Unit','normalized',...
      'FontSize',9,...
  'Position',[pos_x1, pos_y8-0.005, text_width_big, text_height],...
  'Horiz','left',...
  'String','DYNAMIC PARAMETERS');
msg_frame_title4 = uicontrol(...
  graph_interface,...
  'Style','text',...
  'Unit','normalized',...
      'FontSize',9,...
  'Position',[pos_x1, pos_y9+0.01, text_width_big, text_height],...
  'Horiz','left',...
  'String','CHANGE PARAMETERS');

% button to start the simulation
callStart = [...
  'set(gcf,''Pointer'',''watch'');'...
  'if (terminate==1), terminate=0;'...
  'end;'...
  'sim_init_draw;'...
  'sim_var;'...
  'set(uiStart,''Visible'',''off'');'...
  'sim_loop;'...
  'set(uiStart,''Visible'',''on'');'...
  'set(gcf,''Pointer'',''arrow'');'];
uiStart = uicontrol(...
  'Style','push',...
  'FontSize',9,...
  'Units','normalized',...
  'Position',[pos_x1, pos_y2, 3*button_size, text_height],...
  'String','Start Simulation',...
  'Interruptible','on',...
  'Callback',callStart);



% button to stop the simulation 
callStop = [...
  'set(uiStart,''Visible'',''on'');'...
  'set(gcf,''Pointer'',''arrow''); '...
  'terminate = 1'];
uiStop = uicontrol(...
  'Style','push',...
    'FontSize',9,...
  'Units','normalized',...
  'Position',[pos_x1+ 1.5*button_size, pos_y3, 1.5*button_size, text_height],...
  'String','STOP',...
  'Callback',callStop);

% button to posicionate robot at initial position
callPosi = [...
     'set(uiPosi,''Visible'',''off'');'...
     'sim_init_draw;'... 
     'set(uiPosi,''Visible'',''on'');'];
uiPosi = uicontrol(...
  'Style','push',...
    'FontSize',9,...
  'Units','normalized',...
  'Position',[pos_x1, pos_y3, 1.5*button_size, text_height],...
  'String','Initial position',...
  'Callback',callPosi);


% button to consider the gravity
callGrav = [...
     'grav_flag = grav_flag*-1;'... 
     'if grav_flag == 1,'...
     'set(uiGrav,''String'',''Gravity On'');'...
      'elseif grav_flag == -1,'...
     'set(uiGrav,''String'',''Gravity Off'');'...
     'end;'];
uiGrav = uicontrol(...
  'Style','push',...
    'FontSize',9,...
  'Units','normalized',...
  'Position',[pos_x1, pos_y4, 1.5*button_size, text_height],...
  'String','Gravity Off',...
  'Callback',callGrav);

% button to consider the gravity
callGravComp = [...
     'grav_comp_flag = grav_comp_flag*-1;'... 
     'if grav_comp_flag == 1,'...
     'set(uiGravComp,''String'',''Gravity Comp. On'');'...
      'elseif grav_comp_flag == -1,'...
     'set(uiGravComp,''String'',''Gravity Comp. Off'');'...
     'end;'];
uiGravComp = uicontrol(...
  'Style','push',...
    'FontSize',9,...
  'Units','normalized',...
  'Position',[pos_x1+ 1.5*button_size, pos_y4, 1.5*button_size, text_height],...
  'String','Gravity Comp. Off',...
  'Callback',callGravComp);

% button to eliminate torques
callTor = [...
     'tor_flag = tor_flag*-1;'... 
     'if tor_flag == 1,'...
     'set(uiTor,''String'',''Torques On'');'...
      'elseif tor_flag == -1,'...
     'set(uiTor,''String'',''Torques Off'');'...
     'end;'];
uiTor = uicontrol(...
  'Style','push',...
    'FontSize',9,...
  'Units','normalized',...
  'Position',[pos_x1, pos_y5, 1.5*button_size, text_height],...
  'String','Torques On',...
  'Callback',callTor);

% button to apply disturbance
callDist = [...
     'dist_flag = dist_flag*-1;'... 
     'if dist_flag == 1,'...
     'set(uiDist,''String'',''Disturbances On'');'...
      'elseif dist_flag == -1,'...
     'set(uiDist,''String'',''Disturbances Off'');'...
     'end;'];
uiDist = uicontrol(...
  'Style','push',...
    'FontSize',9,...
  'Units','normalized',...
  'Position',[pos_x1+ 1.5*button_size, pos_y5, 1.5*button_size, text_height],...
  'String','Disturbances Off',...
  'Callback',callDist);

% button to restart gui
callRestart = [...
  'close all;'...
  'clc;'...
  'sim_twolink;'];
uiRestart = uicontrol(...
  'Style','push',...
    'FontSize',9,...
  'Units','normalized',...
  'Position',[pos_x1, pos_y6, 1.5*button_size, text_height],...
  'String','Restart',...
  'Callback',callRestart);

% button to close gui
callClose = [...
  'close all;'...
  'more off;'...
  'clc;'...
  'clear;'];
uiClose = uicontrol('Style','push',...
      'FontSize',9,...
  'Units','normalized',...
  'Position',[pos_x1+ 1.5*button_size, pos_y6, 1.5*button_size, text_height],...
  'String','Close',...
  'Callback',callClose);



% create menu for controller definition
callCont = [...
  'set(mc_error_data,''Visible'',''off'');'...
  'value = get(cont_menu,''Value'');'...
  'set(cont_menu,''Value'',1);'...
  'if value==2,'...
  'cont=''PID'';'...
  'elseif value == 3,'...
  'cont=''CTq'';'...
  'elseif value == 4,'...
  'cont=''ADP'';'...
     'elseif value == 5,'...
  'cont=''HIN'';'...
  'elseif value == 6,'...
  'cont=''LPV'';'...
    'elseif value == 7,'...
  'cont=''CEV'';'...
  'elseif value == 8,'...
  'cont=''Car'';'...
  'end;'... 
  'sim_check_data;'...
  'sim_init_draw;'...
  'sim_set_msg;'];
cont_menu = uicontrol(...
  graph_interface,...
  'Style','popupmenu',...
  'Unit','normalized', ...
  'FontSize',9,...
  'Position',[pos_x1, pos_y3-height_size, popup_size, text_height],...
  'Horiz','left',...
  'Max',8,...  
  'String','Controller:|PID  Control    |Computed Torque   |Adaptive  Control     |CT + H-infinity Control |LPV Control |Variable Structure |Cartesian Control',...
  'Callback',callCont); 
cont_dsp = uicontrol(graph_interface,...
  'Style','text',...
  'Unit','normalized',...
   'FontSize',9,...
  'Position',[pos_x1+popup_size, pos_y3-height_size-0.005, text_width_medium, text_height],...
  'Horiz','left',...
  'String','');


% button to consider the gravity
callKi = [...
     'ki_flag = ki_flag*-1;'...
     'if ki_flag == 1,'...
     'set(uiKi,''String'',''Ki On'');'...
      'elseif grav_flag == -1,'...
     'set(uiKi,''String'',''Ki Off'');'...
     'end;'...
     'sim_set_msg;'];
uiKi = uicontrol(...
  'Style','push',...
    'FontSize',9,...
  'Units','normalized',...
  'Visible','off',...
  'Position',[pos_x1+popup_size+ text_width_small - 0.01, pos_y3-height_size+0.01, text_width_small, text_height*0.8],...
  'String','Ki Off',...
  'Callback',callKi);

% create menu for trajectory definition
callTraj = [...
  'set(mc_error_data,''Visible'',''off'');'...
  'value = get(traj_menu,''Value'');'...
  'set(traj_menu,''Value'',1);'...
  'if value==2,'...
  'traj= 1;'...
  'traj_t = ''Poly (5th)'';',...
  'elseif value == 3,'...
  'traj = 2;'...
    'traj_t = ''Trapezoidal Vel'';',...
  'elseif value == 4,'...
  'traj = 3;'...
    'traj_t = ''Senoidal'';',...
  'end;'... 
  'sim_check_data;'...
  'sim_init_draw;'...
  'sim_set_msg;'];
traj_menu = uicontrol(...
  graph_interface,...
  'Style','popupmenu',...
  'Unit','normalized', ...
   'FontSize',9,...
  'Position',[pos_x1, pos_y4-height_size, popup_size, text_height],...
  'Horiz','left',...
  'Max',4,...  
  'String','Trajectory:|Poly (5th)|Trapezoidal Vel |Senoidal  ',...
  'Callback',callTraj); 
traj_dsp = uicontrol(graph_interface,...
  'Style','text',...
  'Unit','normalized',...
   'FontSize',9,...
  'Position',[pos_x1+popup_size, pos_y4-height_size-0.005, text_width_medium, text_height],...
  'Horiz','left',...
  'String','');




% create menu for joints' initial condition
callIC = [...
  'set(mc_error_data,''Visible'',''off'');'...
  'value = get(ic_menu,''Value'');'...
  'set(ic_menu,''Value'',1);'...
  'if (value == 2), '...
  'init_cond = ic_dflt;',...
  'elseif (value == 3), '...
  'init_cond = str2num(get(text_prompt,''String''));'...
  'init_cond(1:2) = init_cond(1:2)*pi/180;'...
  'init_cond = init_cond'';'...
  'sim_check_data;'...
  'end;'...
  'sim_init_draw;'...
  'sim_set_msg;'];
ic_menu = uicontrol(...
  graph_interface,...
  'Style','popupmenu',...
  'Unit','normalized', ...
   'FontSize',9,...
  'Position',[pos_x1, pos_y5-height_size, popup_size, text_height],...
  'Horiz','left',...
  'Max',3,...
  'String','Initial angles:|Default|User defined',...
  'Callback',callIC);
ic_dsp = uicontrol(graph_interface,...
  'Style','text',...
  'Unit','normalized',...
   'FontSize',9,...
  'Position',[pos_x1+popup_size, pos_y5-height_size-0.005, text_width_medium+0.02, text_height],...
  'Horiz','left',...
  'String','');


% create menu for joints' set-point
callSP = [...
  'set(mc_error_data,''Visible'',''off'');'...
  'value = get(sp_menu,''Value'');'...
  'set(sp_menu,''Value'',1);'...
  'if (value == 2), '...
     'set_point = sp_dflt;',...
  'elseif (value == 3), '...
  'set_point = str2num(get(text_prompt,''String''));'...
  'set_point(1:2) = set_point(1:2)*pi/180;'...
  'set_point = set_point'';'...
  'sim_check_data;'...
  'end;'...
  'sim_init_draw;'...
  'sim_set_msg;'];
sp_menu = uicontrol(...
  graph_interface,...
  'Style','popupmenu',...
  'Unit','normalized',...
   'FontSize',9,...
  'Position',[pos_x1, pos_y6-height_size, popup_size, text_height],...
  'Horiz','left',...
  'Max',3,...
  'String','Set-point:|Default|User defined',...
  'Callback',callSP);
sp_dsp = uicontrol(graph_interface,...
  'Style','text',...
  'Unit','normalized',...
   'FontSize',9,...
  'Position',[pos_x1+popup_size, pos_y6-height_size-0.005, text_width_medium+0.02, text_height],...
  'Horiz','left',...
  'String','');


% create menus for dynamic parameters
callMass = [...
  'set(mc_error_data,''Visible'',''off'');'...
  'value = get(mass_menu,''Value''); '...
  'set(mass_menu,''Value'',1);'...
  'if (value == 2), '...
    'm = m_dflt;'...
  'elseif (value == 3), '...
    'm = str2num(get(text_prompt,''String''));'...
    'sim_check_data;'...
  'end; '...
  'sim_set_msg; '];
mass_menu = uicontrol(...
  graph_interface,...
  'Style','popupmenu',...
  'Unit','normalized', ...
   'FontSize',9,...
  'Position',[pos_x1, pos_y3-2*height_size, popup_size, text_height],...
  'Horiz','left',...
  'Max',3,...
  'String','Mass:|Default|User defined',...
  'Callback',callMass);
mass_dsp = uicontrol(...
  graph_interface,...
  'Style','text',...
  'Unit','normalized',...
   'FontSize',9,...
  'Position',[pos_x1+popup_size, pos_y3-2.03*height_size, text_width_medium, text_height],...
  'Horiz','left',...
  'String','');

callLength = [...
  'set(mc_error_data,''Visible'',''off'');'...
  'value = get(length_menu,''Value''); '...
  'set(length_menu,''Value'',1);'...
  'if (value == 2), '...
    'l = l_dflt;'...
    'lc = lc_dflt;'...
  'elseif (value == 3), '...
    'l = str2num(get(text_prompt,''String''));'...
    'lc = l/2;'...
    'sim_check_data;'...
  'end; '...
  'sim_set_msg;'...
  'sim_init_draw;'];
length_menu = uicontrol(...
  graph_interface,...
  'Style','popupmenu',...
  'Unit','normalized', ...
   'FontSize',9,...
  'Position',[pos_x1, pos_y4-2*height_size, popup_size, text_height],...
  'Horiz','left',...
  'Max',3,...
  'String','Length:|Default|User defined',...
  'Callback',callLength);
length_dsp = uicontrol(...
  graph_interface,...
  'Style','text',...
  'Unit','normalized',...
   'FontSize',9,...
  'Position',[pos_x1+popup_size, pos_y4-2.03*height_size, text_width_medium, text_height],...
  'Horiz','left',...
  'String','');

callCOM = [...
  'set(mc_error_data,''Visible'',''off'');'...
  'value = get(com_menu,''Value''); '...
  'set(com_menu,''Value'',1);'...
  'if (value == 2), '...
    'lc = lc_dflt;'...
  'elseif (value == 3), '...
    'lc = str2num(get(text_prompt,''String''));'...
    'sim_check_data;'...
  'end; '...
  'sim_set_msg; '];
com_menu = uicontrol(...
  graph_interface,...
  'Style','popupmenu',...
  'Unit','normalized', ...
   'FontSize',9,...
  'Position',[pos_x1, pos_y5-2*height_size, popup_size, text_height],...
  'Horiz','left',...
  'Max',3,...
  'String','Center of mass:|Default|User defined',...
  'Callback',callCOM);
com_dsp = uicontrol(...
  graph_interface,...
  'Style','text',...
  'Unit','normalized',...
   'FontSize',9,...
  'Position',[pos_x1+popup_size, pos_y5-2.03*height_size, text_width_medium, text_height],...
  'Horiz','left',...
  'String','');

call_du = [...
  'set(mc_error_data,''Visible'',''off'');'...
  'value = get(du_menu,''Value''); '...
  'set(du_menu,''Value'',1);'...
  'if (value == 2), '...
    'du = du_dflt;'...
  'elseif (value == 3), '...
    'du = str2num(get(text_prompt,''String''));'...
  'end; '...
  'sim_check_data;'...
  'sim_set_msg; '];
du_menu = uicontrol(...
  graph_interface,...
  'Style','popupmenu',...
  'Unit','normalized', ...
   'FontSize',9,...
  'Position',[pos_x1, pos_y6-2*height_size, popup_size, text_height],...
  'Horiz','left',...
  'Max',3,...
  'String','Dynamic unc.:|Default|User defined',...
  'Callback',call_du);
du_dsp = uicontrol(...
  graph_interface,...
  'Style','text',...
  'Unit','normalized',...
   'FontSize',9,...
  'Position',[pos_x1+popup_size, pos_y6-2.03*height_size, text_width_medium, text_height],...
  'Horiz','left',...
  'String','');

%create messages and prompt to change parameters
mc_title2 = uicontrol(...
  graph_interface,...
  'Style','text',...
  'Unit','normalized',...
   'FontSize',9,...
  'Position',[pos_x1, pos_y4-2.85*height_size+0.015, text_width_huge, text_height],...
  'Horiz','left',...
  'Visible','on',...
  'String','(Make sure to use correct dimensions.)');
mc_error_data = uicontrol(...
  graph_interface,...
  'Style','text',...
  'Unit','normalized',...
   'FontSize',9,...
  'Position',[pos_x1, pos_y4-2.85*height_size+0.015, text_width_huge, text_height],...
  'Horiz','left',...
  'Visible','off',...
  'String','Invalid data! Default values set.');
text_prompt = uicontrol(...
  graph_interface,...
  'Style','edit',...
  'Unit','normalized',...
   'FontSize',9,...
  'Position',[pos_x1, pos_y4-2.96*height_size+0.015, text_width_big, text_height],...
  'Horiz','left');


%create the button to open the graphic figures  
callGraphic = [...
  'value = get(graph_menu,''Value'');'... 
  'if (value ~= 1),',...
  'set(value-1,''Visible'',''on'');'...
  'end;'...
  'if (value == 4),',...
  'sim_graph_adp;',...
  'end'];


graph_menu = uicontrol(...
  graph_interface,...
  'Style','popupmenu',...
  'Unit','normalized', ...
   'FontSize',9,...
  'Position',[pos_x1+text_width_medium,pos_y4-3*height_size-0.04, popup_size, text_height],...
  'Horiz','left',...
  'Max',7,...
  'String','Controller:|PID  Control    |Computed Torque   |Adaptive  Control |CT + H-infinity Control  |LPV Control |Variable Structure |Cartesian Control',...
  'Callback',callGraphic);

graphics_dsp = uicontrol(...
  'Style','text',...
  'Units','normalized',...
   'FontSize',9,...
  'Position',[pos_x1,pos_y4-3*height_size-0.05,text_width_medium,text_height],...
  'Horiz','left',...
  'String','GRAPHICS');

% create message to display elapsed time
time_dsp = uicontrol(...
  graph_interface,...
  'Style','text',...
  'Unit','normalized',...
   'FontSize',9,...
  'Position',[pos_x1, pos_y6-3*height_size-0.02, text_width_medium, text_height],...
  'Horiz','left',...
  'String','Simulation time: ');
time_value = uicontrol(...
  graph_interface,...
  'Style','text',...
  'Unit','normalized', ...
   'FontSize',9,...
  'Position',...
   [pos_x1+text_width_medium, pos_y6-3*height_size-0.02, text_width_small, text_height],...
 'Horiz','left','String','0.0');

drawnow;
% additional initial assumptions
cont = 'CTq'; %start with Dynamic Inverse
set(graph_interface,'Pointer','arrow'); % return pointer to arrow
sim_set_msg;
sim_init_draw; % draw links
