% Simulation environment of a 2-link planar manipulator
% Copyright 2011 Adriano Siqueira
% File sim_inv_kin_graph.m
% Computes the inverse kinematics of the 2-link robot

function [init_cond, set_point] = uarm_inv_kin_graph(n, l, cart, button, ic, sp)

% check input and output arguments
if (nargin ~= 6),
  error('Wrong number of input arguments to UARM_INV_KIN_GRAPH.M');
elseif (nargout ~= 2),
  error('Wrong number of output arguments from UARM_INV_KIN_GRAPH.M');
end;

err = 0;

d = (cart(1))^2 + (cart(2))^2;

% check for existence of solution
if (sqrt(d) > l(1)+l(2)),
  err = 1;
  disp('Specified Cartesian point is beyond the reach of the arm');
end;
beta = atan2(cart(2),cart(1));
gamma = acos((d + l(1)^2 - l(2)^2) / (2*l(1)*sqrt(d)));
joint_angle(2,1) = acos((d - l(1)^2 - l(2)^2) / (2*l(1)*l(2)));
joint_angle(1,1) = beta - gamma*sign(joint_angle(2,1)) ;


for j = 1:2, 
   phi(j) = sum(joint_angle(1:j,1));
end;
   cphi = cos(phi);
   sphi = sin(phi);
   % run forward kinematic to find out where joints are
   X(1) = 0;
   Y(1) = 0;
for j = 2:2+1,
        X(j) = X(j-1) + l(j-1)*cphi(j-1);
        Y(j) = Y(j-1) + l(j-1)*sphi(j-1);
end;

if Y(2)>0 & Y(3)<0,
   err=1;
   disp('Specified Cartesian point is beyond the reach of the arm');
end     
      
% according to which button was pressed, change init_cond or set_point
if (~err),
  if (button == 1),
    init_cond = joint_angle;
    set_point = sp;
  else
     init_cond = ic; 
    set_point = joint_angle;
  end;
else
  if (button == 1),
    init_cond = zeros(n,1);
    set_point = sp;
  else
    init_cond = ic;
    set_point = zeros(n,1);
  end;
end;

return;
