% Simulation environment of a 2-link planar manipulator
% Copyright 2011 Adriano Siqueira
% File sim_var.m
% Define variables used by all controllers

more off;
disp('Initializing variables, please wait...');
disp(' ');

sim_time = 0;  % simulated time
time_clock = 0;  % wall-clock time
counter = 0;  % generic counter
stat = zeros(n,1);  % statistics collected
time = 0:dt:(tfinal-dt);  % time vector
i=1;
grav = ((1+grav_flag)/2)*.1;

mest = m*du(1);  % estimated masses of the links
lest = l*du(2);  % estimated lenghts of the links
lcest = lc*du(3);  % estimated center of masses of the links

q_d = zeros(n,max_sample);  % desired joint angles
qd_d = zeros(n,max_sample);  % desired joint velocities
qdd_d = zeros(n,max_sample);   % desired joint accelerations
q = zeros(n,max_sample);  % actual joint angles
qd = zeros(n,max_sample);  % actual joint velocities
qdd = zeros(n,max_sample);  % actual joint accelerations
qdd_c = zeros(n,max_sample);  % computed value of qdd
qd_r = zeros(n,max_sample);  % desired joint velocities
qdd_r = zeros(n,max_sample);   % desired joint accelerations
eq = zeros(n,max_sample);  %position error
eqd = zeros(n,max_sample);  %velocity error
theta = init_cond;  % current value of q
omega = zeros(n,1);  % current value of qd
errorq = zeros(n,1);  % position error
errorqd = zeros(n,1);  % velocity error
soma=zeros(n,1);
Theta = zeros(2,max_sample);  %position error
Thetad = zeros(2,max_sample);  %position error
x_d = zeros(n,max_sample);
xd_d = zeros(n,max_sample);
xdd_d = zeros(n,max_sample);
x = zeros(n,max_sample);
xd = zeros(n,max_sample);
xdd = zeros(n,max_sample);
r = zeros(n,max_sample);
incerteza = zeros(n,max_sample);

for j = 1:max_sample,
    q(:,j) = init_cond;
end;
velocity_f=[0;0];
q_d(:,1)=init_cond;

tau = zeros(n,max_sample);  % torque applied to the joints
torque = zeros(n,1);  % current value of tau
u = zeros(n,max_sample);  % control input 

pert = zeros(n,max_sample);  % disturbance applied to the joints

tsalto = [1 1];
amp = [.3 -.1];
dp = [.2 .2];
freq = [5.8 6.3];

reach1 = zeros(n,1);  % set-point reaching flag
reach = zeros(n,1);  % set-point reaching flag
count_reach1 = 0;  % number of active joints that converged
count_reach = 0;  % number of active joints that converged

M = zeros(n,n);  % inertia tensor matrix
b = zeros(n,1);  % non-inertial and gravitational torques
Mest = zeros(n,n);  % estimated inertia tensor matrix
best = zeros(n,1);  % estimated non-inertial and gravitational torques
C = zeros(n,n);
Cest = zeros(n,n);
G = zeros(n,1);  %gravitational torques
Gest = zeros(n,1);% estimated gravitational torques
[M,C,G,b] = pr2l(theta, omega, m, l, lc, grav);
[Mest,Cest,Gest,best] = pr2l_est(theta, omega,  mest, lest, lcest, grav);
W = inv(M);


%Trajetria desejada
if (all(cont =='CTq')|all(cont =='PID')|all(cont =='ADP')|all(cont =='CEV')|all(cont =='HIN')|all(cont =='LPV')),

    if traj == 1,
        %Estabelece 5 segundos para intervalos de 30 graus
        time_inicial = time(i);
        time_final = time(i) + (abs(set_point-init_cond)/0.3491)*2;
        qif =[q_d(:,i)'; qd_d(:,i)'; qdd_d(:,i)'; set_point';zeros(1,n); zeros(1,n)];
        for j=1:n
            [a_aaa(:,j)] = ctraj(time_inicial,time_final(j),qif(:,j));
        end;
        for k=1:max_sample,
            T = time(k);
            for j = 1:n
                if T < time_final(j),
                    [q_d(j,k), qd_d(j,k), qdd_d(j,k)]=vtraj(T,a_aaa(:,j));
                else
                    q_d(j,k)=set_point(j);
                    qd_d(j,k)=0;
                    qdd_d(j,k)=0;
                end;
            end
            % Degrau
%                q_d(:,k)=set_point;
%             		qd_d(:,k)=[0;0];
%                    qdd_d(:,k)=[0;0];

        end;

    elseif traj == 2,
        qd0 = zeros(n,1);
        qd1 = qd0;
        tb = [0.8;0.5];
        vmax = [5;10]*pi/180;
        [q_d,qd_d,qdd_d] = jtraj_lpb(init_cond,set_point,time,vmax,tb,qd0,qd1);

    elseif traj == 3,
        %seno
        for k=1:max_sample,
            T = time(k);
            q_d(:,k) = (30*pi/180)*sin(pi*time(k)/5)+(20*pi/180)*cos(pi*time(k)/3);
            qd_d(:,k) = pi*(30*pi/180)*cos(pi*time(k)/5)/5 -pi*(20*pi/180)*sin(pi*time(k)/3)/3;
            qdd_d(:,k) =-pi*pi*(30*pi/180)*sin(pi*time(k)/5)/25-pi*pi*(20*pi/180)*cos(pi*time(k)/3)/9;
        end


    end

elseif all(cont =='Car')
    [xy_init_cond] = sim_forw_kin(init_cond,l);
    [xy_set_point] = sim_forw_kin(set_point,l);

    time_inicial = time(i);
    time_final = time(i) + max((abs(xy_set_point-xy_init_cond)/0.4)*5)*[1;1];
    qif =[xy_init_cond'; zeros(1,n); zeros(1,n); xy_set_point';zeros(1,n); zeros(1,n)];
    for j=1:n
        [a_aaa(:,j)]=ctraj(time_inicial,time_final(j),qif(:,j));
    end;
    for k=1:max_sample,
        T = time(k);
        for j = 1:n
            if T < time_final(j),
                [x_d(j,k), xd_d(j,k), xdd_d(j,k)]=vtraj(T,a_aaa(:,j));
            else
                x_d(j,k)=xy_set_point(j);
                xd_d(j,k)=0;
                xdd_d(j,k)=0;
            end;
        end;
    end;

end


